SystemOrganization addCategory: #Monticello!
SystemOrganization addCategory: #'Monticello-Announcements'!
SystemOrganization addCategory: #'Monticello-Base'!
SystemOrganization addCategory: #'Monticello-Chunk Format'!
SystemOrganization addCategory: #'Monticello-Loading'!
SystemOrganization addCategory: #'Monticello-Merging'!
SystemOrganization addCategory: #'Monticello-Modeling'!
SystemOrganization addCategory: #'Monticello-Patching'!
SystemOrganization addCategory: #'Monticello-Repositories'!
SystemOrganization addCategory: #'Monticello-Storing'!
SystemOrganization addCategory: #'Monticello-Versioning'!
SystemOrganization addCategory: #'Monticello-file services'!

SharedPool subclass: #MCMockAPoolDictionary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Mocks'!

!Character methodsFor: '*Monticello-Storing' stamp: 'tk 1/17/2000 11:27'!
comeFullyUpOnReload: smartRefStream
	"Use existing an Character.  Don't use the new copy."

	^ self class value: value! !

Error subclass: #MCRepositoryError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCRepositoryError commentStamp: '<historical>' prior: 0!
I am an MC-specific errror!

!Set methodsFor: '*Monticello-Storing' stamp: 'tk 4/8/1999 13:05'!
comeFullyUpOnReload: smartRefStream
	"Symbols have new hashes in this image."

	self rehash.
	"^ self"
! !

!String methodsFor: '*monticello' stamp: 'avi 2/4/2004 14:14'!
extractNumber
	^ ('0', self select: [:ea | ea isDigit]) asNumber! !

Model subclass: #MCPackageManager
	instanceVariableNames: 'package modified'
	classVariableNames: 'PrivateAnnouncer'
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

MCPackageManager class
	instanceVariableNames: 'registry'!

!MCPackageManager commentStamp: 'StephaneDucasse 4/29/2011 20:42' prior: 0!
MCPackageManager is a kind of package wrapper knowing if the package is dirty or not.
The class side manages registered package managers.

Instance Variables:
	package	<MCPackage>
	modified	<Boolean>

Class Instance Variables:
	registry	<Dictionary>!

MCPackageManager class
	instanceVariableNames: 'registry'!

!MCPackageManager class methodsFor: 'accessing' stamp: 'ab 3/31/2003 20:45'!
allManagers
	^ self registry values! !

!MCPackageManager class methodsFor: 'private' stamp: 'EstebanLorenzano 8/3/2012 14:29'!
announcer
	^PrivateAnnouncer ifNil: [ SystemAnnouncer uniqueInstance  ]! !

!MCPackageManager class methodsFor: 'private' stamp: 'EstebanLorenzano 5/21/2012 17:33'!
announcer: anAnnouncer
	PrivateAnnouncer := anAnnouncer! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'GuillermoPolito 8/3/2012 13:51'!
classModified: anEvent
	self managersForClass: anEvent classAffected do:[:mgr| mgr modified: true].! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'CamilloBruni 8/30/2012 11:19'!
classMoved: anEvent
	self managersForPackage: anEvent oldPackage do: [ :mgr | mgr modified: true ].
	self managersForPackage: anEvent newPackage do: [ :mgr | mgr modified: true ].! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'EstebanLorenzano 2/18/2013 17:53'!
classRemoved: anEvent
	"Informs the registry who use to keep this class that its changed. Unlike #classModified:, class is not anymore in RPackages so it will not be found, that's why we look for system category instead if class is included or not"
	self registry do: [:mgr |
		(mgr packageSet includesSystemCategory: anEvent classAffected category)
			ifTrue: [ mgr modified: true ] ]! !

!MCPackageManager class methodsFor: 'cleaning' stamp: 'StephaneDucasse 7/11/2010 23:02'!
flushObsoletePackageInfos
	"Flush any and all PackageInfos that are not associated with an MCPackageManager."

	| pkgNames |
	pkgNames := self allManagers collect: [:wcs| wcs packageName] as: Set.
	PackageOrganizer default flushObsoletePackages: [:p|
		p class isObsolete or:[(pkgNames includes: p packageName) not].
	].! !

!MCPackageManager class methodsFor: 'accessing' stamp: 'EstebanLorenzano 5/21/2012 17:34'!
forPackage: aPackage
	^ self registry at: aPackage ifAbsent:
		[|mgr|
		mgr := self new initializeWithPackage: aPackage.
		self registry at: aPackage put: mgr.
		self changed: (Array with: #registered with: aPackage).
		self announcer announce: (MCWorkingCopyCreated workingCopy: mgr package: aPackage).
		mgr]! !

!MCPackageManager class methodsFor: 'initialize' stamp: 'CamilloBruni 8/1/2012 16:27'!
initialize
	"Remove this later"

	self unregisterForNotifications.! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'GuillermoPolito 9/21/2012 15:10'!
managersForCategory: aSystemCategory do: aBlock
	"Got to be careful here - we might get method categories where capitalization is problematic."
	| cat foundOne index |
	foundOne := false.
	cat := aSystemCategory ifNil:[^nil]. "yes this happens; for example in eToy projects"
	"first ask PackageInfos, their package name might not match the category"
	self registry do: [:mgr | 
		(mgr packageSet includesSystemCategory: aSystemCategory)	ifTrue: [
			aBlock value: mgr.
			foundOne := true.
		]
	].
	foundOne ifTrue: [^self].
	["Loop over categories until we found a matching one"
	self registry at: (MCPackage named: cat) ifPresent:[:mgr|
		aBlock value: mgr.
		foundOne := true.
	].
	index := cat lastIndexOf: $-.
	index > 0]whileTrue:[
		"Step up to next level package"
		cat := cat copyFrom: 1 to: index-1.
	].
	foundOne ifFalse:[
		"Create a new (but only top-level)"
		aBlock value: (MCWorkingCopy forPackage: (MCPackage named: aSystemCategory capitalized)).
	].! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'ar 4/27/2005 14:11'!
managersForClass: aClass category: methodCategory do: aBlock
	(methodCategory isEmptyOrNil or:[methodCategory first ~= $*]) ifTrue:[
		"Not an extension method"
		^self managersForClass: aClass do: aBlock.
	].
	self managersForCategory: methodCategory allButFirst do: aBlock.! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'EstebanLorenzano 9/12/2012 13:35'!
managersForClass: aClass do: aBlock

	self registry do: [:mgr |
		(mgr packageSet includesClass: aClass)
			ifTrue: [aBlock value: mgr]]! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'ar 4/26/2005 21:40'!
managersForClass: aClass selector: aSelector do: aBlock
	^self managersForClass: aClass category: (aClass organization categoryOfElement: aSelector) do: aBlock! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'EstebanLorenzano 9/12/2012 13:35'!
managersForPackage: aPackage do: aBlock
	self registry do: [:mgr | 
		(mgr packageSet includesSystemCategory: aPackage name) ifTrue: [
			aBlock value: mgr.
		]
	].! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'GuillermoPolito 8/3/2012 13:57'!
methodModified: anEvent
	^self managersForClass: anEvent methodClass selector: anEvent selector do:[:mgr| mgr modified: true].! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'CamilloBruni 9/22/2012 10:43'!
methodMoved: anEvent
	self managersForPackage: anEvent oldPackage do: [ :mgr | mgr modified: true ].
	self managersForPackage: anEvent newPackage do: [ :mgr | mgr modified: true ].! !

!MCPackageManager class methodsFor: 'system changes' stamp: 'GuillermoPolito 8/3/2012 14:15'!
methodRemoved: anEvent
	self managersForClass: anEvent methodClass category: anEvent protocol do:[:mgr| mgr modified: true].
! !

!MCPackageManager class methodsFor: 'event registration' stamp: 'EstebanLorenzano 2/18/2013 17:42'!
registerInterestOnSystemChangesOnAnnouncer: anAnnouncer
	anAnnouncer on: ClassAdded, ClassModifiedClassDefinition, ClassRenamed, ClassCommented send: #classModified: to: self.
	anAnnouncer on: ClassRepackaged send: #classMoved: to: self.
	anAnnouncer on: ClassRemoved send: #classRemoved: to: self.
	anAnnouncer on: MethodAdded, MethodModified send: #methodModified: to: self.
	anAnnouncer on: MethodRepackaged send: #methodMoved: to: self.
	anAnnouncer on: MethodRemoved send: #methodRemoved: to: self.! !

!MCPackageManager class methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
registry
	^ registry ifNil: [registry := Dictionary new]! !

!MCPackageManager methodsFor: 'private' stamp: 'EstebanLorenzano 5/21/2012 17:31'!
announcer
	^self class announcer! !

!MCPackageManager methodsFor: 'initialization' stamp: 'StephaneDucasse 5/7/2011 18:18'!
initialize
	super initialize.
	modified := false.
! !

!MCPackageManager methodsFor: 'initialization' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithPackage: aPackage
	package := aPackage.
	self initialize.! !

!MCPackageManager methodsFor: 'accessing' stamp: 'cwp 11/13/2003 14:12'!
modified
	^ modified! !

!MCPackageManager methodsFor: 'accessing' stamp: 'GuillermoPolito 7/31/2012 11:46'!
modified: aBoolean
	modified = aBoolean
		ifTrue: [ ^ self ].
	modified := aBoolean.
	self changed: #modified.
	self class changed: (Array with: #modified with: self).
	modified
		ifFalse: [ 
			Smalltalk logChange: '"' , self packageName , '"' ].
	self announcer announce: (MCPackageModified package: self package)
	! !

!MCPackageManager methodsFor: 'accessing' stamp: 'ab 7/7/2003 16:47'!
package
	^ package! !

!MCPackageManager methodsFor: 'accessing' stamp: 'EstebanLorenzano 9/14/2012 11:32'!
packageInfo
	self deprecated: 'Use #packageSet' on: '14 September 2012'  in: '2.0'.
	^ package packageInfo! !

!MCPackageManager methodsFor: 'accessing' stamp: 'ab 7/7/2003 12:18'!
packageName
	^ package name! !

!MCPackageManager methodsFor: 'accessing' stamp: 'ab 7/5/2003 23:18'!
packageNameWithStar
	^ modified
		ifTrue: ['* ', self packageName]
		ifFalse: [self packageName]! !

!MCPackageManager methodsFor: 'accessing' stamp: 'EstebanLorenzano 9/11/2012 17:01'!
packageSet
	^ package packageSet! !

!MCPackageManager methodsFor: 'operations' stamp: 'EstebanLorenzano 10/17/2012 17:00'!
unregister
	self class registry removeKey: package.
	self class changed: (Array with: #unregistered with: package).
	self announcer announce: (MCWorkingCopyDeleted workingCopy: self package: package)! !

!MCPackageManager methodsFor: 'system changes' stamp: 'EstebanLorenzano 9/12/2012 13:35'!
update: aSymbol
	InMidstOfFileinNotification signal ifFalse: [
	[((aSymbol = #recentMethodSubmissions)
		and: [self packageSet
				includesMethodReference: RecentMessageList uniqueInstance lastEntry])
					ifTrue: [self modified: true]]
		on: Error do: []]! !

MCPackageManager subclass: #MCWorkingCopy
	instanceVariableNames: 'versionInfo ancestry counter repositoryGroup requiredPackages'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCWorkingCopy commentStamp: 'StephaneDucasse 4/29/2011 20:44' prior: 0!
MCWorkingCopy represents one version of a package in memory. It provides support for ancestry access, required packages and the repositories in which the package is managed.

Instance Variables:
	versionInfo	<MCAncestry>
	ancestry	<MCWorkingAncestry>
	counter	<Number>
	repositoryGroup	<MCRepositoryGroup>
	requiredPackages	<Collection>!

!MCWorkingCopy class methodsFor: 'private' stamp: 'avi 9/13/2004 18:00'!
adoptVersionInfoFrom: anInstaller
	|viCache|
	viCache := Dictionary new.
	anInstaller versionInfo keysAndValuesDo: [:packageName :info |
		(self forPackage: (MCPackage named: packageName))
			versionInfo: (self infoFromDictionary:  info cache: viCache)].
	[anInstaller clearVersionInfo] on: Error do: ["backwards compat"].! !

!MCWorkingCopy class methodsFor: 'private' stamp: 'avi 2/17/2004 01:23'!
ancestorsFromArray: anArray cache: aDictionary
	^ anArray ifNotNil: [anArray collect: [:dict | self infoFromDictionary: dict cache: aDictionary]]! !

!MCWorkingCopy class methodsFor: 'querying' stamp: 'sd 1/24/2013 10:52'!
hasPackageNamed: aName
	" self hasPackageNamed: 'ConfigurationOfFuel' "
	
	^ MCWorkingCopy allManagers
		detect: [ :each | each packageName = aName ]
		ifNone: [ nil ].! !

!MCWorkingCopy class methodsFor: 'private' stamp: 'StephaneDucasse 8/17/2012 15:58'!
infoFromDictionary: aDictionary cache: cache
	| id |
	id := (aDictionary at: #id) asString.
	^ cache at: id ifAbsentPut:
		[MCVersionInfo
			name: (aDictionary at: #name ifAbsent: [''])
			id: (UUID fromString: id)
			message: (aDictionary at: #message ifAbsent: [''])
			date: ([Date fromString: (aDictionary at: #date)] ifError: [nil])
			time: ([Time fromString: (aDictionary at: #time)] ifError: [nil])
			author: (aDictionary at: #author ifAbsent: [''])
			ancestors: (self ancestorsFromArray: (aDictionary at: #ancestors ifAbsent: []) cache: cache)
			stepChildren: (self ancestorsFromArray: (aDictionary at: #stepChildren ifAbsent: []) cache: cache)]! !

!MCWorkingCopy class methodsFor: 'initialize' stamp: 'CamilleTeruel 7/30/2012 14:09'!
initialize
	"self initialize"
	registry ifNotNil: [ registry rehash ].
	self registerForNotifications.! !

!MCWorkingCopy class methodsFor: 'event registration' stamp: 'MarianoMartinezPeck 9/5/2012 15:05'!
registerForNotifications
	"self registerForNotifications"
	<systemEventRegistration>
	
	MCPackageManager announcer: nil.
	SystemAnnouncer uniqueInstance unsubscribe: self.
	self registerInterestOnSystemChangesOnAnnouncer: SystemAnnouncer uniqueInstance.! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'avi 2/13/2004 01:07'!
adopt: aVersion
	ancestry addAncestor: aVersion info.
	self changed.! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'avi 2/13/2004 01:07'!
ancestors
	^ ancestry ancestors! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'avi 2/13/2004 01:13'!
ancestry
	^ ancestry! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'marcus.denker 11/10/2008 10:04'!
backportChangesTo: aVersionInfo
	| baseVersion fullPatch currentVersionInfo currentVersion newSnapshot newAncestry |
	currentVersionInfo := self currentVersionInfo.
	baseVersion := self repositoryGroup versionWithInfo: aVersionInfo.
	currentVersion := self repositoryGroup versionWithInfo: currentVersionInfo.
	fullPatch := currentVersion snapshot patchRelativeToBase: baseVersion snapshot.
	(MCChangeSelectionRequest new
		patch: fullPatch;
		label: 'Changes to Backport';
		signal ) ifNotNil:
		[:partialPatch |
		newSnapshot := MCPatcher apply: partialPatch to: baseVersion snapshot.
		newAncestry := MCWorkingAncestry new
							addAncestor: aVersionInfo;
							addStepChild: currentVersionInfo;
							yourself.
		MCPackageLoader updatePackage: package withSnapshot: newSnapshot.
		ancestry := newAncestry.
		self modified: false; modified: true]! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'CamilloBruni 1/19/2012 18:35'!
changesRelativeToRepository: aRepository
	^ self completeSnapshot patchRelativeToBase: (self closestAncestorSnapshotIn: aRepository).! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'stephaneducasse 2/4/2006 20:47'!
clearRequiredPackages
	requiredPackages := nil! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'CamilloBruni 3/2/2012 13:32'!
closestAncestorSnapshotIn: aRepository
	
	"Returns a complete Snapshot including all required packages by recurisveliy walking all required package and trying to find the closes ancestor in the given repository"
	
	| definitions ancestorVersion|
	
	ancestorVersion := aRepository closestAncestorVersionFor: ancestry ifNone: [].
	
	definitions := ancestorVersion 
						ifNil: [ OrderedCollection new ] 
						ifNotNil: [ ancestorVersion snapshot definitions ].
	
	definitions addAll: (self loadRemoteDependenciesIn: aRepository).
	
	^ MCSnapshot fromDefinitions: definitions! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'CamilloBruni 3/2/2012 15:07'!
collectDependenciesWithMessage: messageString
	
	^ self requiredPackages collect: [:aPackage | 
			MCVersionDependency
				package: aPackage
				info:  (aPackage workingCopy currentVersionInfoWithMessage: messageString)]
	! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'CamilloBruni 1/19/2012 18:34'!
completeSnapshot
	"return a complete snapshot of the loaded sources in this working copy. 
	unlike snapshot this includes also the snapshots of all packages"
	
	| definitions |
	
	definitions := self snapshot definitions asOrderedCollection.
	
	self requiredPackages 
		do: [ :aPackage| definitions addAll: aPackage workingCopy completeSnapshot definitions ]
		displayingProgress: [ :item| 'Loading dependencies from: ', item name ].
	
	^ MCSnapshot fromDefinitions: definitions! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'avi 2/13/2004 20:01'!
currentVersionInfo
	^ (self needsSaving or: [ancestry ancestors isEmpty])
		ifTrue: [self newVersion info]
		ifFalse: [ancestry ancestors first]! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'AndrewBlack 9/4/2009 14:00'!
currentVersionInfoWithMessage: aMessageString
	^ (self needsSaving or: [ancestry ancestors isEmpty])
		ifTrue: [(self newVersionWithMessage: aMessageString) info]
		ifFalse: [ancestry ancestors first]! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'bp 11/21/2010 18:33'!
description
	^ self packageNameWithStar, ' (', (ancestry ancestorStringWithout: self packageName), ')'! !

!MCWorkingCopy methodsFor: 'private' stamp: 'StephaneDucasse 1/16/2010 10:51'!
findSnapshotWithVersionInfo: aVersionInfo
	"when an ancestor inside the ancestor chain is not found, does not pass nil instead.
	With this change we can now browse history and delta between them without having to 
	have the complete history"

	^ aVersionInfo
		ifNil: [MCSnapshot empty]
		ifNotNil: [(self repositoryGroup versionWithInfo: aVersionInfo)
			ifNil: [MCSnapshot empty]
			ifNotNil: [:aVersion | aVersion snapshot]]
! !

!MCWorkingCopy methodsFor: 'private' stamp: 'stephaneducasse 2/4/2006 20:47'!
initialize
	super initialize.
	ancestry := MCWorkingAncestry new! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'CamilloBruni 3/2/2012 14:42'!
loadRemoteDependenciesIn: aRepository
	"load the remote dependencies of all packages in parallel"
	| dependencies |
	dependencies := self 
		requiredPackagesParallelCollect: [ :aPackage|
			(aPackage workingCopy closestAncestorSnapshotIn: aRepository) definitions.] 
		withMessage: 'Loading remote dependencies...'.
	^ dependencies 
		inject: OrderedCollection new
		into: [ :all :deps| all addAll: deps ]! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'stephaneducasse 2/4/2006 20:47'!
loaded: aVersion
	ancestry := MCWorkingAncestry new addAncestor: aVersion info.
	requiredPackages := OrderedCollection withAll: (aVersion dependencies collect: [:ea | ea package]).
	self modified: false.
	self changed! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'StephaneDucasse 4/27/2010 11:52'!
merge: targetVersion
	| ancestorInfo merger ancestorSnapshot packageSnapshot |
	targetVersion dependencies do: [:ea | ea resolve merge].
	ancestorInfo := targetVersion info commonAncestorWith: ancestry.
	
	ancestorInfo = targetVersion info ifTrue: [^ MCNoChangesException signal].
	
	packageSnapshot := package snapshot.
	ancestorSnapshot := ancestorInfo
							ifNotNil: [(self findSnapshotWithVersionInfo: ancestorInfo)]
							ifNil: [self notifyNoCommonAncestorWith: targetVersion.  MCSnapshot empty].
	
	(ancestry ancestors size = 1
		and: [ancestry ancestors first = ancestorInfo
				and: [(packageSnapshot patchRelativeToBase: ancestorSnapshot) isEmpty]])
				ifTrue: [^ targetVersion load].
	
	merger := MCThreeWayMerger 
				base: packageSnapshot
				target: targetVersion snapshot
				ancestor: ancestorSnapshot.
	((MCMergeResolutionRequest new merger: merger)
		signal: 'Merging ', targetVersion info name) = true ifTrue:
			[merger loadWithNameLike: targetVersion info name.
			ancestry addAncestor: targetVersion info].
	self changed! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'CamilloBruni 1/19/2012 16:12'!
merged: aVersion
	ancestry addAncestor: aVersion info.
	aVersion dependencies do: [:ea | self requirePackage: ea package].
	self changed! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'avi 2/13/2004 20:02'!
needsSaving
	^ self modified or: [self requiredPackages anySatisfy: [:ea | ea workingCopy needsSaving]]! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'StephaneDucasse 12/19/2012 14:21'!
newVersion
	^ (self requestVersionNameAndMessageWithSuggestion: self uniqueVersionName) ifNotNil:
		[:pair |
		self newVersionWithName: pair first trimBoth message: pair last].
! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'AndrewBlack 9/4/2009 14:09'!
newVersionWithMessage: aMessageString
	^ self newVersionWithName: self uniqueVersionName message: aMessageString.

	"^ (self requestVersionNameAndMessageWithSuggestion: self uniqueVersionName) ifNotNil:
		[:pair |
		self newVersionWithName: pair first message: aMessageString].
	"! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'CamilloBruni 3/2/2012 13:42'!
newVersionWithName: nameString message: messageString
	
	| info deps |
	info := ancestry infoWithName: nameString message: messageString.
	ancestry := MCWorkingAncestry new addAncestor: info.
	self modified: true; modified: false.
	
	deps := self collectDependenciesWithMessage: messageString.

	^ MCVersion
		package: package
		info: info
		snapshot: package snapshot
		dependencies: deps! !

!MCWorkingCopy methodsFor: 'private' stamp: 'MiguelCoba 7/25/2009 02:01'!
nextVersionName
	| branch oldName base author |
	branch := ''.
	ancestry ancestors isEmpty
		ifTrue: [counter ifNil: [counter := 0]. base := package name]
		ifFalse:
			[oldName := ancestry ancestors first name.
			oldName last isDigit
				ifFalse: [base := oldName]
				ifTrue: [
					base := oldName copyUpToLast: $-.
					branch := ((oldName copyAfterLast: $-) copyUpToLast: $.) copyAfter: $. ].
			counter ifNil: [
				counter := (ancestry ancestors collect: [:each |
					each name last isDigit
						ifFalse: [0]
						ifTrue: [(each name copyAfterLast: $.) extractNumber]]) max]].

	branch isEmpty ifFalse: [branch := '.',branch].
	counter := counter + 1.
	author := Author fullName collect: [ :each | each isAlphaNumeric ifTrue: [ each ] ifFalse: [ $_ ] ].
	^ base , '-' , author , branch , '.' , counter asString! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'avi 2/13/2004 01:07'!
notifyNoCommonAncestorWith: aVersion
	self notify:
'Could not find a common ancestor between (',
aVersion info name,
') and (',
ancestry ancestorString, ').
Proceeding with this merge may cause spurious conflicts.'! !

!MCWorkingCopy methodsFor: 'private' stamp: 'bf 9/8/2005 10:58'!
possiblyNewerVersions

	^Array streamContents: [:strm |
		self repositoryGroup repositories do: [:repo |
			strm nextPutAll: (self possiblyNewerVersionsIn: repo)]]! !

!MCWorkingCopy methodsFor: 'private' stamp: 'bf 9/8/2005 10:58'!
possiblyNewerVersionsIn: aRepository

	^aRepository possiblyNewerVersionsOfAnyOf: self ancestors! !

!MCWorkingCopy methodsFor: 'printing' stamp: 'sd 3/15/2008 14:13'!
printOn: aStream

	super printOn: aStream.
	package name ifNotNil: [ aStream nextPutAll: '(' , package name asString, ')'].! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'stephane.ducasse 2/6/2009 18:31'!
removeRequiredPackage: aPackage

	requiredPackages remove: aPackage ifAbsent: []
! !

!MCWorkingCopy methodsFor: 'repositories' stamp: 'stephaneducasse 2/4/2006 20:47'!
repositoryGroup
	^ repositoryGroup ifNil: [repositoryGroup := MCRepositoryGroup new]! !

!MCWorkingCopy methodsFor: 'repositories' stamp: 'stephaneducasse 2/4/2006 20:47'!
repositoryGroup: aRepositoryGroup
	repositoryGroup := aRepositoryGroup! !

!MCWorkingCopy methodsFor: 'private' stamp: 'ab 8/24/2003 20:38'!
requestVersionNameAndMessageWithSuggestion: aString
	^ (MCVersionNameAndMessageRequest new suggestedName: aString) signal! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'avi 1/20/2004 16:04'!
requirePackage: aPackage
	(self requiredPackages includes: aPackage) ifFalse: [requiredPackages add: aPackage]! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
requiredPackages
	^ requiredPackages ifNil: [requiredPackages := OrderedCollection new]! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'SeanDeNigris 6/21/2012 09:07'!
requiredPackagesParallelCollect: aBlock withMessage: aMessage
	"load the remote dependencies of all packages in parallel"

	| packages progress results maxConcurrentConnections connectionThrottle addSemaphore |
	
	results := OrderedCollection new.
	addSemaphore := Semaphore forMutualExclusion.
	connectionThrottle := Semaphore new.
	progress := 0.
	maxConcurrentConnections := 5.
	maxConcurrentConnections timesRepeat: [ connectionThrottle signal ]. "make room for maxConcurrentConnections"
	packages := self requiredPackages.
	
	aMessage displayProgressFrom: 0 to: packages size during: [ :bar|
		packages do: [ :aPackage| 
			connectionThrottle wait. "no more than maxConcurrentConnections"
			[|result|
				[
					result := aBlock value: aPackage.
					addSemaphore critical: [ "update the non-threadsafe collection"
						results add: result.
						bar current: (progress := progress + 1)]
				] ensure: [ connectionThrottle signal. "free a connection" ].
			] fork ]].
	
	maxConcurrentConnections timesRepeat: [ connectionThrottle wait ]. "wait for the last connection to finish"
	
	^ results! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'CamilloBruni 9/30/2011 16:25'!
snapshot
	^ self package snapshot! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'SeanDeNigris 7/17/2012 15:50'!
theCachedRepository

	^ MCCacheRepository uniqueInstance.! !

!MCWorkingCopy methodsFor: 'private' stamp: 'CamilloBruni 4/24/2012 15:22'!
uniqueVersionName
	|versionName|
	counter := nil.
	'Creating unique version number' 
		displayProgressFrom: 0 
		to: 1 
		during: [ :arg|
			[versionName := self nextVersionName.
			self repositoryGroup includesVersionNamed: versionName] whileTrue ].
	^ versionName! !

!MCWorkingCopy methodsFor: 'operations' stamp: 'EstebanLorenzano 10/17/2012 17:00'!
unload
	MCPackageLoader unloadPackage: self package.
	package packageSet unregister.
	self unregister.! !

!MCWorkingCopy methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
versionInfo: aVersionInfo
	ancestry := MCWorkingAncestry new addAncestor: aVersionInfo! !

!MCWorkingCopy methodsFor: 'private' stamp: 'avi 2/4/2004 14:11'!
versionSeparator
	^ $_! !

!Trait methodsFor: '*Monticello' stamp: 'al 3/26/2006 21:44'!
asClassDefinition
	^ MCTraitDefinition
		name: self name
		traitComposition: self traitCompositionString
		category: self category 
		comment: self organization classComment asString
		commentStamp: self organization commentStamp.! !

!Trait methodsFor: '*Monticello' stamp: 'GuillermoPolito 4/30/2012 18:21'!
classDefinitions
	| definitions |
	definitions := OrderedCollection with: self asClassDefinition.
	(self hasClassTrait
		and: [self classTrait hasTraitComposition])
					ifTrue: [definitions add: self classTrait asMCDefinition].
	^definitions asArray! !

Announcement subclass: #MCPackageModified
	instanceVariableNames: 'package'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Announcements'!

!MCPackageModified commentStamp: '<historical>' prior: 0!
An MCPackageModified is raised when a MCPackage is modified!

!MCPackageModified class methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 4/14/2011 13:52'!
package: aMCPackage

	^ self new package: aMCPackage! !

!MCPackageModified methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 4/14/2011 13:49'!
package

		^ package! !

!MCPackageModified methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 4/14/2011 13:49'!
package: anObject
	
		package := anObject! !

Announcement subclass: #MCVersionCreated
	instanceVariableNames: 'name nameString version'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Announcements'!

!MCVersionCreated commentStamp: '<historical>' prior: 0!
An MCVersionCreated is raised when a MCVersion is created
!

!MCVersionCreated class methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 5/24/2011 14:57'!
name: aName message: aString version: aMCVersion

	^ self new
			name: aName;
			nameString: aString;
			version: aMCVersion;
			yourself! !

!MCVersionCreated class methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 4/14/2011 13:56'!
name: aName nameString: aString version: aMCVersion

	^ self new
			name: aName;
			nameString: aString;
			version: aMCVersion;
			yourself! !

!MCVersionCreated methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 4/14/2011 13:56'!
name: anObject
	
		name := anObject! !

!MCVersionCreated methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 4/14/2011 13:56'!
nameString

		^ nameString! !

!MCVersionCreated methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 4/14/2011 13:56'!
nameString: anObject
	
		nameString := anObject! !

!MCVersionCreated methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 4/14/2011 13:56'!
version

		^ version! !

!MCVersionCreated methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 4/14/2011 13:56'!
version: anObject
	
		version := anObject! !

Announcement subclass: #MCWorkingCopyCreated
	instanceVariableNames: 'package workingCopy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Announcements'!

!MCWorkingCopyCreated commentStamp: '<historical>' prior: 0!
A MCWorkingCopyCreated is a announcement raised when a new MCWorkingCopy is created!

!MCWorkingCopyCreated class methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 2/25/2012 19:57'!
workingCopy: workingCopy

	^ self new
		workingCopy: workingCopy;
		yourself! !

!MCWorkingCopyCreated class methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 2/25/2012 19:56'!
workingCopy: aWC package: package

	^ self new
		workingCopy: aWC;
		package: package;
		yourself! !

!MCWorkingCopyCreated methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 2/25/2012 19:55'!
package
	^ package! !

!MCWorkingCopyCreated methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 2/25/2012 19:55'!
package: anObject
	package := anObject! !

!MCWorkingCopyCreated methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 2/25/2012 19:56'!
workingCopy
	^ workingCopy! !

!MCWorkingCopyCreated methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 2/25/2012 19:56'!
workingCopy: anObject
	workingCopy := anObject! !

Announcement subclass: #MCWorkingCopyDeleted
	instanceVariableNames: 'package workingCopy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Announcements'!

!MCWorkingCopyDeleted commentStamp: '<historical>' prior: 0!
A MCWorkingCopyDeleted class is an announcement raised when a MCWorkingCopy is removed!

!MCWorkingCopyDeleted class methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 2/25/2012 20:24'!
workingCopy: workingCopy

	^ self new
		workingCopy: workingCopy;
		yourself! !

!MCWorkingCopyDeleted class methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 2/25/2012 20:24'!
workingCopy: aWC package: package

	^ self new
		workingCopy: aWC;
		package: package;
		yourself! !

!MCWorkingCopyDeleted methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 2/25/2012 20:24'!
package
	^ package! !

!MCWorkingCopyDeleted methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 2/25/2012 20:24'!
package: anObject
	package := anObject! !

!MCWorkingCopyDeleted methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 2/25/2012 20:24'!
workingCopy
	^ workingCopy! !

!MCWorkingCopyDeleted methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 2/25/2012 20:24'!
workingCopy: anObject
	workingCopy := anObject! !

Notification subclass: #MCChangeSelectionRequest
	instanceVariableNames: 'patch label'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCChangeSelectionRequest methodsFor: 'as yet unclassified' stamp: 'avi 9/14/2004 15:02'!
label
	^ label! !

!MCChangeSelectionRequest methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
label: aString
	label := aString! !

!MCChangeSelectionRequest methodsFor: 'as yet unclassified' stamp: 'avi 9/11/2004 15:12'!
patch
	^ patch! !

!MCChangeSelectionRequest methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
patch: aPatch
	patch := aPatch! !

Notification subclass: #MCMergeResolutionRequest
	instanceVariableNames: 'merger'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCMergeResolutionRequest methodsFor: 'as yet unclassified' stamp: 'ab 7/18/2003 18:19'!
merger
	^ merger! !

!MCMergeResolutionRequest methodsFor: 'as yet unclassified' stamp: 'EstebanLorenzano 11/7/2012 16:13'!
merger: aMerger
	merger := aMerger! !

Notification subclass: #MCVersionNameAndMessageRequest
	instanceVariableNames: 'suggestion suggestedLogComment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCVersionNameAndMessageRequest methodsFor: 'as yet unclassified' stamp: 'AndrewBlack 9/4/2009 14:16'!
suggestedLogComment

	^ suggestedLogComment! !

!MCVersionNameAndMessageRequest methodsFor: 'as yet unclassified' stamp: 'AndrewBlack 9/4/2009 14:16'!
suggestedLogComment: aLogMessage

	suggestedLogComment := aLogMessage! !

!MCVersionNameAndMessageRequest methodsFor: 'as yet unclassified' stamp: 'ab 7/10/2003 01:07'!
suggestedName
	^ suggestion! !

!MCVersionNameAndMessageRequest methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
suggestedName: aString
	suggestion := aString! !

!ClassTrait methodsFor: '*Monticello' stamp: 'damiencassou 7/30/2009 12:10'!
asMCDefinition
	^MCClassTraitDefinition
		baseTraitName: self baseTrait name
		classTraitComposition: self traitCompositionString
		category: self category
			! !

Stream subclass: #MCDataStream
	instanceVariableNames: 'byteStream topCall basePos'
	classVariableNames: 'TypeMap'
	poolDictionaries: ''
	category: 'Monticello-Storing'!

!MCDataStream commentStamp: '<historical>' prior: 0!
This is the save-to-disk facility. A DataStream can store one or more objects in a persistent form.

To handle objects with sharing and cycles, you must use a
ReferenceStream instead of a DataStream.  (Or SmartRefStream.)  ReferenceStream is typically
faster and produces smaller files because it doesn't repeatedly write the same Symbols.

Here is the way to use DataStream and ReferenceStream:
	rr := ReferenceStream fileNamed: 'test.obj'.
	rr nextPut: <your object>.
	rr close.

To get it back:
	rr := ReferenceStream fileNamed: 'test.obj'.
	<your object> := rr next.
	rr close.

Each object to be stored has two opportunities to control what gets stored.  On the high level, objectToStoreOnDataStream allows you to substitute another object on the way out.  The low level hook is storeDataOn:. The read-in counterparts to these messages are comeFullyUpOnReload and (class) readDataFrom:size:. See these methods for more information about externalizing and internalizing.

NOTE: A DataStream should be treated as a write-stream for writing.  It is a read-stream for reading.  It is not a ReadWriteStream.
!

!MCDataStream class methodsFor: 'cleanup' stamp: 'StephaneDucasse 3/9/2010 16:24'!
cleanUp
	"Re-initialize DataStream to avoid hanging onto obsolete classes"

	self initialize! !

!MCDataStream class methodsFor: 'instance creation' stamp: 'CarloTeixeira 2/26/2012 00:18'!
detectFile: aBlock do: anotherBlock

	^aBlock value
		ifNil: [nil]
      	ifNotNil: [:file| [anotherBlock value: file] ensure: [file close]]! !

!MCDataStream class methodsFor: 'instance creation' stamp: 'MicheleLanza 10/23/2010 10:36'!
fileNamed: aString
	"Here is the way to use DataStream and ReferenceStream:

| rr |
rr := ReferenceStream fileNamed: 'test.obj'.
rr nextPut: 'Zork'.
rr close.
"

	| strm |
	strm := self on: (FileStream fileNamed: aString).		"will be binary"
	strm byteStream setFileTypeToObject.
		"Type and Creator not to be text, so can attach correctly to an email msg"
	^ strm! !

!MCDataStream class methodsFor: 'instance creation' stamp: 'CarloTeixeira 2/26/2012 00:19'!
fileNamed: fileName do: aBlock
	"Returns the result of aBlock."
	
	^ self detectFile: [ self fileNamed: fileName ] do: aBlock! !

!MCDataStream class methodsFor: 'initialize-release' stamp: 'MarcusDenker 5/31/2012 10:52'!
initialize
	"TypeMap maps Smalltalk classes to type ID numbers which identify the data stream primitive formats.  nextPut: writes these IDs to the data stream.  NOTE: Changing these type ID numbers will invalidate all extant data stream files.  Adding new ones is OK.  
	Classes named here have special formats in the file.  If such a class has a subclass, it will use type 9 and write correctly.  It will just be slow.  (Later write the class name in the special format, then subclasses can use the type also.)
	 See nextPut:, next, typeIDFor:, & ReferenceStream>>isAReferenceType:"

	"DataStream initialize"

	| refTypes t |
	refTypes := OrderedCollection new.
	t := TypeMap := Dictionary new: 80.	"sparse for fast hashing"
	t at: UndefinedObject put: 1.
	refTypes add: 0.
	t at: True put: 2.
	refTypes add: 0.
	t at: False put: 3.
	refTypes add: 0.
	t at: SmallInteger put: 4.
	refTypes add: 0.
	t at: ByteString put: 5.
	refTypes add: 1.
	t at: ByteSymbol put: 6.
	refTypes add: 1.
	t at: ByteArray put: 7.
	refTypes add: 1.
	t at: Array put: 8.
	refTypes add: 1.	"(type ID 9 is for arbitrary instances of any class, cf. typeIDFor:)"
	refTypes add: 1.	"(type ID 10 is for references, cf. ReferenceStream>>tryToPutReference:)"
	refTypes add: 0.
	t at: Bitmap put: 11.
	refTypes add: 1.
	t at: Metaclass put: 12.
	refTypes add: 0.	"Type ID 13 is used for HyperSqueak User classes that must be reconstructed."
	refTypes add: 1.
	t at: Float put: 14.
	refTypes add: 1.
	t at: Rectangle put: 15.
	refTypes add: 1.	"Allow compact Rects."	"type ID 16 is an instance with short header.  See beginInstance:size:"
	refTypes add: 1.
	self flag: #ByteArray.
	t at: ByteString put: 17.
	refTypes add: 1.	"new String format, 1 or 4 bytes of length"
	t at: WordArray put: 18.
	refTypes add: 1.	"bitmap-like"
	Smalltalk globals
		at: #SoundBuffer
		ifPresent: [ :class | 
			t at: class put: 20. ].
	refTypes add: 1.	"And all other word arrays, both 
		16-bit and 32-bit.  See methods in ArrayedCollection.  Overridden in SoundBuffer."
	t at: CompiledMethod put: 21.
	refTypes add: 1.	"special creation method"	"t at:  put: 22.  refTypes add: 0."
	Smalltalk do: [ :cls | cls isBehavior ifTrue: [ 
							cls isPointers not & cls isVariable & cls isWords
								ifTrue: [ (t includesKey: cls)
										ifFalse: [ t at: cls put: 20 ] ] ] ]! !

!MCDataStream class methodsFor: 'instance creation' stamp: 'di 2/15/98 14:03'!
new
	^ self basicNew! !

!MCDataStream class methodsFor: 'instance creation' stamp: 'MicheleLanza 10/23/2010 10:37'!
newFileNamed: aString
	"Here is the way to use DataStream and ReferenceStream:
	|rr|
	rr := ReferenceStream fileNamed: 'test.obj'.
	rr nextPut: 'Zork'.
	rr close.
"

	| strm |
	strm :=  self on: (FileStream newFileNamed: aString).		"will be binary"
	strm byteStream setFileTypeToObject.
		"Type and Creator not to be text, so can attach correctly to an email msg"
	^ strm! !

!MCDataStream class methodsFor: 'instance creation' stamp: 'MicheleLanza 10/23/2010 10:36'!
oldFileNamed: aString
	"Here is the way to use DataStream and ReferenceStream:

	|rr |
	rr := ReferenceStream oldFileNamed: 'test.obj'.
	^ rr nextAndClose.
"

	| strm ff |
	ff := FileStream oldFileOrNoneNamed: aString.
	ff ifNil: [^ nil].
	strm := self on: (ff binary).
	^ strm! !

!MCDataStream class methodsFor: 'instance creation' stamp: 'di 6/24/97 00:18'!
on: aStream
	"Open a new DataStream onto a low-level I/O stream."

	^ self basicNew setStream: aStream
		"aStream binary is in setStream:"
! !

!MCDataStream class methodsFor: 'instance creation' stamp: 'MicheleLanza 10/23/2010 10:39'!
readOnlyFileNamed:  aString
	"Here is the way to use DataStream and ReferenceStream:
	
	|rr|
	rr := ReferenceStream fileNamed: 'test.obj'.
	rr nextPut: 'Zork'.
	rr close.
"

	| strm |
	strm := self on: (FileStream readOnlyFileNamed:   aString).		"will be binary"
	strm byteStream setFileTypeToObject.
		"Type and Creator not to be text, so can attach correctly to an email msg"
	^ strm! !

!MCDataStream class methodsFor: 'instance creation' stamp: 'RAA 7/28/2000 08:38'!
streamedRepresentationOf: anObject

	| file |
	file := (RWBinaryOrTextStream on: (ByteArray new: 5000)).
	file binary.
	(self on: file) nextPut: anObject.
	^file contents! !

!MCDataStream class methodsFor: 'instance creation' stamp: 'RAA 7/28/2000 08:33'!
unStream: aString

	^(self on: ((RWBinaryOrTextStream with: aString) reset; binary)) next! !

!MCDataStream methodsFor: 'other' stamp: ''!
atEnd
    "Answer true if the stream is at the end."

    ^ byteStream atEnd! !

!MCDataStream methodsFor: 'write and read' stamp: '6/9/97 08:14 tk'!
beginInstance: aClass size: anInteger
	"This is for use by storeDataOn: methods.
	 Cf. Object>>storeDataOn:."

		"Addition of 1 seems to make extra work, since readInstance
		has to compensate.  Here for historical reasons dating back
		to Kent Beck's original implementation in late 1988.

		In ReferenceStream, class is just 5 bytes for shared symbol.

		SmartRefStream puts out the names and number of class's instances variables for checking."

	byteStream nextNumber: 4 put: anInteger + 1.

	self nextPut: aClass name! !

!MCDataStream methodsFor: 'write and read' stamp: 'yo 12/3/2004 17:12'!
beginReference: anObject
    "We're starting to read anObject. Remember it and its reference
     position (if we care; ReferenceStream cares). Answer the
     reference position."

    ^ 0! !

!MCDataStream methodsFor: 'other' stamp: ''!
byteStream
	^ byteStream! !

!MCDataStream methodsFor: 'other' stamp: ''!
close
	"Close the stream."

	| bytes |
	byteStream closed 
		ifFalse: [
			bytes := byteStream position.
			byteStream close]
		ifTrue: [bytes := 'unknown'].
	^ bytes! !

!MCDataStream methodsFor: 'other' stamp: 'nk 3/12/2004 21:56'!
contents
	^byteStream contents! !

!MCDataStream methodsFor: 'other' stamp: 'CamilloBruni 8/1/2012 16:05'!
errorWriteReference: anInteger
    "PRIVATE -- Raise an error because this case of nextPut:'s perform:
     shouldn't be called."

    self error: 'This should never be called'! !

!MCDataStream methodsFor: 'other' stamp: 'CamilloBruni 8/1/2012 16:06'!
flush
    "Guarantee that any writes to me are actually recorded on disk."

    ^ byteStream flush! !

!MCDataStream methodsFor: 'write and read' stamp: ''!
getCurrentReference
    "PRIVATE -- Return the currentReference posn.
     Overridden by ReferenceStream."

    ^ 0! !

!MCDataStream methodsFor: 'write and read' stamp: 'tk 4/8/1999 13:11'!
maybeBeginReference: internalObject
	"Do nothing.  See ReferenceStream|maybeBeginReference:"

	^ internalObject! !

!MCDataStream methodsFor: 'write and read' stamp: 'MarianoMartinezPeck 6/5/2012 15:22'!
next
	"Answer the next object in the stream."
	| type selector anObject isARefType pos internalObject |

	type := byteStream next.
	type ifNil: [pos := byteStream position.	"absolute!!!!"
		byteStream close.	"clean up"
		byteStream position = 0 
			ifTrue: [self error: 'The file did not exist in this directory'] 
			ifFalse: [self error: 'Unexpected end of object file'].
		pos.	"so can see it in debugger"
		^ nil].
	type = 0 ifTrue: [pos := byteStream position.	"absolute!!!!"
		byteStream close.	"clean up"
		self error: 'Expected start of object, but found 0'.
		^ nil].
	isARefType := self noteCurrentReference: type.
	selector := #(readNil readTrue readFalse readInteger	"<-4"
			readStringOld readSymbol readByteArray		"<-7"
			readArray readInstance readReference readBitmap	"<-11"
			readClass readUser readFloat readRectangle readShortInst 	"<-16"
			readString readWordArray foo 	"foo is a replacement for readWordArrayForSegment<-19"
			readWordLike readMethod "<-21") at: type.
	selector = 0 ifTrue: [pos := byteStream position.	"absolute!!!!"
			byteStream close. 
			self error: 'file is more recent than this system'. ^ nil].
	anObject := self perform: selector. "A method that recursively
		calls next (readArray, readInstance, objectAt:) must save &
		restore the current reference position."
	isARefType ifTrue: [self beginReference: anObject].

		"After reading the externalObject, internalize it.
		 #readReference is a special case. Either:
		   (1) We actually have to read the object, recursively calling
			   next, which internalizes the object.
		   (2) We just read a reference to an object already read and
			   thus already interalized.
		 Either way, we must not re-internalize the object here."
	selector == #readReference ifTrue: [^ anObject].
	internalObject := anObject comeFullyUpOnReload: self.
	internalObject == String ifTrue:[
		"This is a hack to figure out if we're loading a String class 
		that really should be a ByteString. Note that these days this
		will no longer be necessary since we use #withClassVersion:
		for constructing the global thus using a different classVersion
		will perfectly do the trick."
		((anObject globalObjectName == #String)
			and:[anObject constructorSelector == #yourself]) ifTrue:[
				internalObject := ByteString]].
	^ self maybeBeginReference: internalObject! !

!MCDataStream methodsFor: 'other' stamp: ''!
next: anInteger
    "Answer an Array of the next anInteger objects in the stream."
    | array |

    array := Array new: anInteger.
    1 to: anInteger do: [:i |
        array at: i put: self next].
    ^ array! !

!MCDataStream methodsFor: 'other' stamp: 'tk 3/5/2002 09:51'!
nextAndClose
	"Speedy way to grab one object.  Only use when we are inside an object binary file.  Do not use for the start of a SmartRefStream mixed code-and-object file."

	| obj |
	obj := self next.
	self close.
	^ obj! !

!MCDataStream methodsFor: 'write and read' stamp: 'MarcusDenker 7/14/2012 01:31'!
nextPut: anObject
	"Write anObject to the receiver stream. Answer anObject."
	| typeID selector objectToStore |

	typeID := self typeIDFor: anObject.
	(self tryToPutReference: anObject typeID: typeID)
		ifTrue: [^ anObject].

	objectToStore := (self objectIfBlocked: anObject).
	objectToStore == anObject ifFalse: [typeID := self typeIDFor: objectToStore].

	byteStream nextPut: typeID.
	selector := #(writeNil: writeTrue: writeFalse: writeInteger: 
		writeStringOld: writeSymbol: writeByteArray:
		writeArray: writeInstance: errorWriteReference: writeBitmap:
		writeClass: writeUser: writeFloat: writeRectangle: == "<-16 short inst" 
		writeString: writeBitmap: writeBitmap: writeWordLike: 
		writeInstance: "CompiledMethod") at: typeID.
	self perform: selector with: objectToStore.

	^ anObject! !

!MCDataStream methodsFor: 'write and read' stamp: ''!
nextPutAll: aCollection
    "Write each of the objects in aCollection to the
     receiver stream. Answer aCollection."

    ^ aCollection do: [:each | self nextPut: each]! !

!MCDataStream methodsFor: 'write and read' stamp: 'yo 12/3/2004 17:25'!
noteCurrentReference: typeID
    "PRIVATE -- If we support references for type typeID, remember
     the current byteStream position so we can add the next object to
     the 'objects' dictionary, and return true. Else return false.
     This method is here to be overridden by ReferenceStream"

    ^ false! !

!MCDataStream methodsFor: 'write and read' stamp: ' 
	6/9/97'!
objectAt: anInteger
	"PRIVATE -- Read & return the object at a given stream position.  08:18 tk  anInteger is a relative file position. "
	| savedPosn anObject refPosn |

	savedPosn := byteStream position.	"absolute"
	refPosn := self getCurrentReference.	"relative position"

	byteStream position: anInteger + basePos.	"was relative"
	anObject := self next.

	self setCurrentReference: refPosn.	"relative position"
	byteStream position: savedPosn.		"absolute"
	^ anObject! !

!MCDataStream methodsFor: 'write and read' stamp: 'tk 3/13/98 22:16'!
objectIfBlocked: anObject
	"We don't do any blocking"

	^ anObject! !

!MCDataStream methodsFor: 'write and read' stamp: '6/9/97 08:46 tk'!
outputReference: referencePosn
	"PRIVATE -- Output a reference to the object at integer stream position referencePosn (relative to basePos). To output a weak reference to an object not yet written, supply (self vacantRef) for referencePosn."

	byteStream nextPut: 10. "reference typeID"
	byteStream nextNumber: 4 put: referencePosn	"relative position"! !

!MCDataStream methodsFor: 'write and read' stamp: '6/9/97 08:32 tk'!
readArray
	"PRIVATE -- Read the contents of an Array.
	 We must do beginReference: here after instantiating the Array
	 but before reading its contents, in case the contents reference
	 the Array. beginReference: will be sent again when we return to
	 next, but that's ok as long as we save and restore the current
	 reference position over recursive calls to next."
	| count array refPosn |

	count := byteStream nextNumber: 4.

	refPosn := self beginReference: (array := Array new: count).		"relative pos"
	1 to: count do: [:i |
		array at: i put: self next].
	self setCurrentReference: refPosn.		"relative pos"
	^ array! !

!MCDataStream methodsFor: 'write and read' stamp: ''!
readBitmap
	"PRIVATE -- Read the contents of a Bitmap."

	^ Bitmap newFromStream: byteStream
	"Note that the reader knows that the size is in long words, but the data is in bytes."! !

!MCDataStream methodsFor: 'write and read' stamp: 'jm 8/19/1998 17:00'!
readByteArray
	"PRIVATE -- Read the contents of a ByteArray."

	| count |
	count := byteStream nextNumber: 4.
	^ byteStream next: count  "assume stream is in binary mode"
! !

!MCDataStream methodsFor: 'write and read' stamp: 'MarianoMartinezPeck 6/5/2012 15:31'!
readClass
	^ self error: 'Classes cannot be materialized with DataStream'! !

!MCDataStream methodsFor: 'write and read' stamp: ''!
readFalse
    "PRIVATE -- Read the contents of a False."

    ^ false! !

!MCDataStream methodsFor: 'write and read' stamp: ''!
readFloat
	"PRIVATE -- Read the contents of a Float.
	 This is the fast way to read a Float.
	 We support 8-byte Floats here.  Non-IEEE"

	| new |
	new := Float new: 2.		"To get an instance"
	new at: 1 put: (byteStream nextNumber: 4).
	new at: 2 put: (byteStream nextNumber: 4).
	^ new! !

!MCDataStream methodsFor: 'write and read' stamp: 'lr 3/14/2010 21:13'!
readInstance
	"PRIVATE -- Read the contents of an arbitrary instance.
	 ASSUMES: readDataFrom:size: sends me beginReference: after it
	   instantiates the new object but before reading nested objects.
	 NOTE: We must restore the current reference position after
	   recursive calls to next.
	Let the instance, not the class read the data.  "

	| instSize aSymbol refPosn anObject newClass |
	instSize := (byteStream nextNumber: 4) - 1.
	refPosn := self getCurrentReference.
	aSymbol := self next.
	newClass := Smalltalk globals at: aSymbol asSymbol.
	anObject := newClass isVariable
		ifFalse: [ newClass basicNew ]
		ifTrue: [ newClass basicNew: instSize - newClass instSize ].	"Create object here"
	self setCurrentReference: refPosn.	"before readDataFrom:size:"
	anObject := anObject readDataFrom: self size: instSize.
	self setCurrentReference: refPosn.	"before returning to next"
	^ anObject! !

!MCDataStream methodsFor: 'write and read' stamp: ''!
readInteger
    "PRIVATE -- Read the contents of a SmallInteger."

    ^ byteStream nextInt32	"signed!!!!!!"! !

!MCDataStream methodsFor: 'write and read' stamp: 'lr 3/14/2010 21:13'!
readMethod
	"PRIVATE -- Read the contents of an arbitrary instance.
	 ASSUMES: readDataFrom:size: sends me beginReference: after it
	   instantiates the new object but before reading nested objects.
	 NOTE: We must restore the current reference position after
	   recursive calls to next.
	Let the instance, not the class read the data.  "

	| instSize refPosn newClass className xxHeader nLits byteCodeSizePlusTrailer newMethod lits |
	instSize := (byteStream nextNumber: 4) - 1.
	refPosn := self getCurrentReference.
	className := self next.
	newClass := Smalltalk globals at: className asSymbol.
	xxHeader := self next.	"nArgs := (xxHeader >> 24) bitAnd: 16rF."	"nTemps := (xxHeader >> 18) bitAnd: 16r3F."	"largeBit := (xxHeader >> 17) bitAnd: 1."
	nLits := xxHeader >> 9 bitAnd: 16rFF.	"primBits := ((xxHeader >> 19) bitAnd: 16r600) + (xxHeader bitAnd: 16r1FF)."
	byteCodeSizePlusTrailer := instSize - newClass instSize - ((nLits + 1) * 4).	"0"
	newMethod := newClass newMethod: byteCodeSizePlusTrailer header: xxHeader.
	self setCurrentReference: refPosn.	"before readDataFrom:size:"
	self beginReference: newMethod.
	lits := newMethod numLiterals + 1.	"counting header"
	2 to: lits do: [ :ii | newMethod objectAt: ii put: self next ].
	lits * 4 + 1 to: newMethod basicSize do: [ :ii | newMethod basicAt: ii put: byteStream next ].	"Get raw bytes directly from the file"
	self setCurrentReference: refPosn.	"before returning to next"
	^ newMethod! !

!MCDataStream methodsFor: 'write and read' stamp: ''!
readNil
    "PRIVATE -- Read the contents of an UndefinedObject."

    ^ nil! !

!MCDataStream methodsFor: 'write and read' stamp: ' 6/9/97'!
readRectangle
    "Read a compact Rectangle.  Rectangles with values outside +/- 2047 were stored as normal objects (type=9).  They will not come here.  17:22 tk"

	"Encoding is four 12-bit signed numbers.  48 bits in next 6 bytes.  17:24 tk"
	| acc left top right bottom |
	acc := byteStream nextNumber: 3.
	left := acc bitShift: -12.
	(left bitAnd: 16r800) ~= 0 ifTrue: [left := left - 16r1000].	"sign"
	top := acc bitAnd: 16rFFF.
	(top bitAnd: 16r800) ~= 0 ifTrue: [top := top - 16r1000].	"sign"

	acc := byteStream nextNumber: 3.
	right := acc bitShift: -12.
	(right bitAnd: 16r800) ~= 0 ifTrue: [right := right - 16r1000].	"sign"
	bottom := acc bitAnd: 16rFFF.
	(bottom bitAnd: 16r800) ~= 0 ifTrue: [bottom := bottom - 16r1000].	"sign"
	
    ^ Rectangle left: left right: right top: top bottom: bottom
! !

!MCDataStream methodsFor: 'write and read' stamp: 'tk 1/5/2000 11:47'!
readReference
	"Read the contents of an object reference. (Cf. outputReference:)  File is not now positioned at this object."
	| referencePosition |

	^ (referencePosition := (byteStream nextNumber: 4)) = self vacantRef	"relative"
		ifTrue:  [nil]
		ifFalse: [self objectAt: referencePosition]		"relative pos"! !

!MCDataStream methodsFor: 'write and read' stamp: 'lr 3/14/2010 21:13'!
readShortInst
	"Read the contents of an arbitrary instance that has a short header.
	 ASSUMES: readDataFrom:size: sends me beginReference: after it
	   instantiates the new object but before reading nested objects.
	 NOTE: We must restore the current reference position after
	   recursive calls to next.
	Let the instance, not the class read the data.  "

	| instSize aSymbol refPosn anObject newClass |
	instSize := byteStream next - 1.	"one byte of size"
	refPosn := self getCurrentReference.
	aSymbol := self readShortRef.	"class symbol in two bytes of file pos"
	newClass := Smalltalk globals at: aSymbol asSymbol.
	anObject := newClass isVariable
		ifFalse: [ newClass basicNew ]
		ifTrue: [ newClass basicNew: instSize - newClass instSize ].	"Create object here"
	self setCurrentReference: refPosn.	"before readDataFrom:size:"
	anObject := anObject readDataFrom: self size: instSize.
	self setCurrentReference: refPosn.	"before returning to next"
	^ anObject! !

!MCDataStream methodsFor: 'write and read' stamp: 'tk 7/12/1998 13:32'!
readShortRef
	"Read an object reference from two bytes only.  Original object must be in first 65536 bytes of the file.  Relative to start of data.  vacantRef not a possibility."

	^ self objectAt: (byteStream nextNumber: 2)! !

!MCDataStream methodsFor: 'write and read' stamp: 'tk 6/8/1998 21:03'!
readString

	| str |
	byteStream ascii.
	str := byteStream nextString.
	byteStream binary.
	^ str
! !

!MCDataStream methodsFor: 'write and read' stamp: 'tk 6/8/1998 21:27'!
readStringOld

   ^ byteStream nextStringOld! !

!MCDataStream methodsFor: 'write and read' stamp: 'TestRunner 1/21/2010 21:44'!
readSymbol
    "PRIVATE -- Read the contents of a Symbol."
    ^ self readString asSymbol! !

!MCDataStream methodsFor: 'write and read' stamp: ''!
readTrue
    "PRIVATE -- Read the contents of a True."

    ^ true! !

!MCDataStream methodsFor: 'write and read' stamp: 'tk 3/4/1999 22:58'!
readUser
	"Reconstruct both the private class and the instance.  Still used??"

	^ self readInstance.		"Will create new unique class"
! !

!MCDataStream methodsFor: 'write and read' stamp: 'tk 1/24/2000 23:20'!
readWordArray
	"PRIVATE -- Read the contents of a WordArray."

	^ WordArray newFromStream: byteStream
	"Size is number of long words."! !

!MCDataStream methodsFor: 'write and read' stamp: 'lr 3/14/2010 21:13'!
readWordLike
	| refPosn aSymbol newClass anObject |
	"Can be used by any class that is bits and not bytes (WordArray, Bitmap, SoundBuffer, etc)."
	refPosn := self getCurrentReference.
	aSymbol := self next.
	newClass := Smalltalk globals at: aSymbol asSymbol.
	anObject := newClass newFromStream: byteStream.	"Size is number of long words."
	self setCurrentReference: refPosn.	"before returning to next"
	^ anObject! !

!MCDataStream methodsFor: 'write and read' stamp: 'tk 9/24/2000 15:39'!
replace: original with: proxy
	"We may wish to remember that in some field, the original object is being replaced by the proxy.  For the hybred scheme that collects with a DummyStream and writes an ImageSegment, it needs to hold onto the originals so they will appear in outPointers, and be replaced."

	"do nothing"! !

!MCDataStream methodsFor: 'other' stamp: ''!
reset
    "Reset the stream."

    byteStream reset! !

!MCDataStream methodsFor: 'other' stamp: 'tk 5/29/97'!
rootObject
	"Return the object at the root of the tree we are filing out.  "

	^ topCall! !

!MCDataStream methodsFor: 'other' stamp: 'tk 5/29/97'!
rootObject: anObject
	"Return the object at the root of the tree we are filing out.  "

	topCall := anObject! !

!MCDataStream methodsFor: 'write and read' stamp: ''!
setCurrentReference: refPosn
    "PRIVATE -- Set currentReference to refPosn.
     Noop here. Cf. ReferenceStream."! !

!MCDataStream methodsFor: 'other' stamp: '6/9/97 08:03 di'!
setStream: aStream
	"PRIVATE -- Initialization method."

	aStream binary.
	basePos := aStream position.	"Remember where we start.  Earlier part of file contains a class or method file-in.  Allow that to be edited.  We don't deal in absolute file locations."
	byteStream := aStream.! !

!MCDataStream methodsFor: 'other' stamp: 'tk 8/18/1998 08:59'!
setStream: aStream reading: isReading
	"PRIVATE -- Initialization method."

	aStream binary.
	basePos := aStream position.	"Remember where we start.  Earlier part of file contains a class or method file-in.  Allow that to be edited.  We don't deal in absolute file locations."
	byteStream := aStream.! !

!MCDataStream methodsFor: 'other' stamp: ''!
size
    "Answer the stream's size."

    ^ byteStream size! !

!MCDataStream methodsFor: 'write and read' stamp: 'yo 12/3/2004 16:59'!
tryToPutReference: anObject typeID: typeID
    "PRIVATE -- If we support references for type typeID, and if
       anObject already appears in my output stream, then put a
       reference to the place where anObject already appears. If we
       support references for typeID but didn't already put anObject,
       then associate the current stream position with anObject in
       case one wants to nextPut: it again.
     Return true after putting a reference; false if the object still
       needs to be put.
     For DataStream this is trivial. ReferenceStream overrides this."

    ^ false! !

!MCDataStream methodsFor: 'write and read' stamp: 'tk 2/20/1999 23:02'!
typeIDFor: anObject
	"Return the typeID for anObject's class.  This is where the tangle of objects is clipped to stop everything from going out.  
	Classes can control their instance variables by defining objectToStoreOnDataStream.
	Any object in blockers is not written out.  See ReferenceStream.objectIfBlocked: and DataStream nextPut:.
	Morphs do not write their owners.  See Morph.storeDataOn:   Each morph tells itself to 'prepareToBeSaved' before writing out."
	
	^ TypeMap at: anObject class ifAbsent: [9 "instance of any normal class"]	
"See DataStream initialize.  nil=1. true=2. false=3. a SmallInteger=4. (a String was 5). a Symbol=6.  a ByteArray=7. an Array=8. other = 9.  a Bitmap=11. a Metaclass=12. a Float=14.  a Rectangle=15. any instance that can have a short header=16.  a String=17 (new format). a WordArray=18."! !

!MCDataStream methodsFor: 'other' stamp: 'tk 7/12/1998 13:16'!
vacantRef
	"Answer the magic 32-bit constant we use ***ON DISK*** as a stream 'reference
	 position' to identify a reference that's not yet filled in. This must be a
	 value that won't be used as an ordinary reference. Cf. outputReference: and
	 readReference. -- 
	 NOTE: We could use a different type ID for vacant-refs rather than writing
		object-references with a magic value. (The type ID and value are
		overwritten by ordinary object-references when weak refs are fullfilled.)"

	^ SmallInteger maxVal! !

!MCDataStream methodsFor: 'write and read' stamp: ''!
writeArray: anArray
	"PRIVATE -- Write the contents of an Array."

	byteStream nextNumber: 4 put: anArray size.
	self nextPutAll: anArray.! !

!MCDataStream methodsFor: 'write and read' stamp: 'tk 6/8/1998 21:07'!
writeBitmap: aBitmap
	"PRIVATE -- Write the contents of a Bitmap."

	aBitmap writeOn: byteStream
	"Note that this calls (byteStream nextPutAll: aBitmap) which knows enough to put 4-byte quantities on the stream!!  Reader must know that size is in long words."! !

!MCDataStream methodsFor: 'write and read' stamp: 'tk 6/8/1998 21:06'!
writeByteArray: aByteArray
	"PRIVATE -- Write the contents of a ByteArray."

	byteStream nextNumber: 4 put: aByteArray size.
	"May have to convert types here..."
	byteStream nextPutAll: aByteArray.! !

!MCDataStream methodsFor: 'write and read' stamp: 'MarianoMartinezPeck 6/5/2012 15:32'!
writeClass: aClass
    ^ self error: 'Classes cannot be serialized with DataStream'! !

!MCDataStream methodsFor: 'write and read' stamp: ''!
writeFalse: aFalse
    "PRIVATE -- Write the contents of a False."! !

!MCDataStream methodsFor: 'write and read' stamp: ''!
writeFloat: aFloat
	"PRIVATE -- Write the contents of a Float.
	  We support 8-byte Floats here."

	byteStream nextNumber: 4 put: (aFloat at: 1).
	byteStream nextNumber: 4 put: (aFloat at: 2).
! !

!MCDataStream methodsFor: 'write and read' stamp: ''!
writeInstance: anObject
    "PRIVATE -- Write the contents of an arbitrary instance."

    ^ anObject storeDataOn: self! !

!MCDataStream methodsFor: 'write and read' stamp: ''!
writeInteger: anInteger
	"PRIVATE -- Write the contents of a SmallInteger."

	byteStream nextInt32Put: anInteger	"signed!!!!!!!!!!"! !

!MCDataStream methodsFor: 'write and read' stamp: ''!
writeNil: anUndefinedObject
    "PRIVATE -- Write the contents of an UndefinedObject."! !

!MCDataStream methodsFor: 'write and read' stamp: 'jm 7/31/97 16:16'!
writeRectangle: anObject
    "Write the contents of a Rectangle.  See if it can be a compact Rectangle (type=15).  Rectangles with values outside +/- 2047 were stored as normal objects (type=9).  17:22 tk"

	| ok right bottom top left acc |
	ok := true.
	(right := anObject right) > 2047 ifTrue: [ok := false].
	right < -2048 ifTrue: [ok := false].
	(bottom := anObject bottom) > 2047 ifTrue: [ok := false].
	bottom < -2048 ifTrue: [ok := false].
	(top := anObject top) > 2047 ifTrue: [ok := false].
	top < -2048 ifTrue: [ok := false].
	(left := anObject left) > 2047 ifTrue: [ok := false].
	left < -2048 ifTrue: [ok := false].
	ok := ok & left isInteger & right isInteger & top isInteger & bottom isInteger.

	ok ifFalse: [
		byteStream skip: -1; nextPut: 9; skip: 0. "rewrite type to be normal instance"
	    ^ anObject storeDataOn: self].

	acc := ((left bitAnd: 16rFFF) bitShift: 12) + (top bitAnd: 16rFFF).
	byteStream nextNumber: 3 put: acc.
	acc := ((right bitAnd: 16rFFF) bitShift: 12) + (bottom bitAnd: 16rFFF).
	byteStream nextNumber: 3 put: acc.! !

!MCDataStream methodsFor: 'write and read' stamp: 'tk 6/8/1998 20:57'!
writeString: aString
	"PRIVATE -- Write the contents of a String."

	byteStream nextStringPut: aString.! !

!MCDataStream methodsFor: 'write and read' stamp: 'tk 6/8/1998 21:23'!
writeStringOld: aString
	"PRIVATE -- Write the contents of a String."

	| length |
	aString size < 16384 
		ifTrue: [
			(length := aString size) < 192
				ifTrue: [byteStream nextPut: length]
				ifFalse: 
					[byteStream nextPut: (length // 256 + 192).
					byteStream nextPut: (length \\ 256)].
			aString do: [:char | byteStream nextPut: char asciiValue]]
		ifFalse: [self writeByteArray: aString].	"takes more space"! !

!MCDataStream methodsFor: 'write and read' stamp: 'TestRunner 1/21/2010 21:58'!
writeSymbol: aSymbol
    "PRIVATE -- Write the contents of a Symbol."

    self writeString: aSymbol! !

!MCDataStream methodsFor: 'write and read' stamp: ''!
writeTrue: aTrue
    "PRIVATE -- Write the contents of a True."! !

!MCDataStream methodsFor: 'write and read' stamp: 'CamilloBruni 8/1/2012 16:18'!
writeUser: anObject
    "Write the contents of an arbitrary User instance (and its devoted class)."

	"If anObject is an instance of a unique user class, will lie and say it has a generic class"
    ^ anObject storeDataOn: self! !

!MCDataStream methodsFor: 'write and read' stamp: 'tk 2/5/2000 21:53'!
writeWordLike: aWordArray
	"Note that we put the class name before the size."

	self nextPut: aWordArray class name.
	aWordArray writeOn: byteStream
	"Note that this calls (byteStream nextPutAll: aBitmap) which knows enough to put 4-byte quantities on the stream!!  Reader must know that size is in long words or double-bytes."! !

!Stream methodsFor: '*Monticello' stamp: 'cwp 8/9/2003 12:02'!
isMessageStream
	^ false! !

Exception subclass: #MCNoChangesException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCNoChangesException methodsFor: 'as yet unclassified' stamp: 'jf 8/21/2003 19:49'!
defaultAction
	self inform: 'No changes'! !

!HashedCollection methodsFor: '*Monticello-Storing' stamp: 'StephaneDucasse 10/18/2010 14:54'!
comeFullyUpOnReload: smartRefStream
	"Symbols have new hashes in this image."

	self  compact.
	"^ self"
! !

PackageInfo subclass: #MCDirtyPackageInfo
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Mocks'!

!MCDirtyPackageInfo class methodsFor: 'as yet unclassified' stamp: 'avi 2/22/2004 14:04'!
initialize
	[self new register] on: MessageNotUnderstood do: []! !

!MCDirtyPackageInfo class methodsFor: 'as yet unclassified' stamp: 'cwp 7/21/2003 19:45'!
wantsChangeSetLogging
	^ false! !

!MCDirtyPackageInfo methodsFor: 'as yet unclassified' stamp: 'ab 7/7/2003 23:21'!
classes
	^ Array new: 0.! !

!MCDirtyPackageInfo methodsFor: 'as yet unclassified' stamp: 'GuillermoPolito 7/6/2012 11:06'!
methods
	^ (Smalltalk at: #MCMockClassA) selectors
		select: [:ea | ea beginsWith: 'ordinal']
		thenCollect:
			[:ea | RGMethodDefinition realClass: (Smalltalk at: #MCMockClassA) selector: ea ]! !

!MCDirtyPackageInfo methodsFor: 'as yet unclassified' stamp: 'ab 7/7/2003 23:21'!
packageName
	^ 'MCDirtyPackage'! !

PackageInfo subclass: #MCEmptyPackageInfo
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Mocks'!

!MCEmptyPackageInfo class methodsFor: 'as yet unclassified' stamp: 'avi 2/22/2004 14:04'!
initialize
	[self new register] on: MessageNotUnderstood do: []! !

!MCEmptyPackageInfo class methodsFor: 'as yet unclassified' stamp: 'cwp 7/21/2003 19:45'!
wantsChangeSetLogging
	^ false! !

!MCEmptyPackageInfo methodsFor: 'as yet unclassified' stamp: 'ab 7/7/2003 23:21'!
classes
	^ #()! !

!MCEmptyPackageInfo methodsFor: 'as yet unclassified' stamp: 'ab 7/7/2003 23:21'!
methods
	^ #()! !

!MCEmptyPackageInfo methodsFor: 'as yet unclassified' stamp: 'ab 7/7/2003 23:21'!
packageName
	^ 'MCEmptyPackage'! !

!Class methodsFor: '*Monticello' stamp: 'al 3/26/2006 21:31'!
asClassDefinition
	^ MCClassDefinition
		name: self name
		superclassName: self superclass name
		traitComposition: self traitCompositionString
		classTraitComposition: self class traitCompositionString
		category: self category 
		instVarNames: self instVarNames
		classVarNames: self classVarNames
		poolDictionaryNames: self poolDictionaryNames
		classInstVarNames: self class instVarNames
		type: self typeOfClass
		comment: self organization classComment	 asString
		commentStamp: self organization commentStamp	! !

!Class methodsFor: '*Monticello' stamp: 'avi 3/10/2004 13:32'!
classDefinitions
	^ Array with: self asClassDefinition! !

Object subclass: #MCAncestry
	instanceVariableNames: 'ancestors stepChildren'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCAncestry commentStamp: '<historical>' prior: 0!
Abstract superclass of records of ancestry.!

!MCAncestry methodsFor: 'ancestry' stamp: 'avi 2/12/2004 20:57'!
allAncestorsDo: aBlock
	self ancestors do:
		[:ea |
		aBlock value: ea.
		ea allAncestorsDo: aBlock]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'avi 2/12/2004 20:57'!
allAncestorsOnPathTo: aVersionInfo
	^ MCFilteredVersionSorter new
		target: aVersionInfo;
		addAllVersionInfos: self ancestors;
		sortedVersionInfos! !

!MCAncestry methodsFor: 'ancestry' stamp: 'avi 2/12/2004 20:57'!
ancestorString
	^ String streamContents:
		[:s | self ancestors do: [:ea | s nextPutAll: ea name] separatedBy: [s nextPutAll: ', ']]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'bp 11/21/2010 18:33'!
ancestorStringWithout: packageName
	^ String streamContents:
		[:s | self ancestors do: [:ea | s nextPutAll: (ea nameWithout: packageName)] separatedBy: [s nextPutAll: ', ']]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'avi 2/12/2004 20:57'!
ancestors
	^ ancestors ifNil: [#()]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'bf 12/22/2004 21:55'!
ancestorsDoWhileTrue: aBlock
	self ancestors do:
		[:ea |
		(aBlock value: ea) ifTrue: 
			[ea ancestorsDoWhileTrue: aBlock]]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'avi 9/17/2005 16:03'!
breadthFirstAncestors
	^ Array streamContents: [:s | self breadthFirstAncestorsDo: [:ea | s nextPut: ea]]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'stephaneducasse 2/4/2006 20:47'!
breadthFirstAncestorsDo: aBlock
	| seen todo next |
	seen := Set with: self.
	todo := OrderedCollection with: self.
	[todo isEmpty] whileFalse:
		[next := todo removeFirst.
		next ancestors do:
			[:ea |
			(seen includes: ea) ifFalse:
				[aBlock value: ea.
				seen add: ea.
				todo add: ea]]]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'stephaneducasse 2/4/2006 20:47'!
commonAncestorWith: aNode
	| commonAncestors |
	commonAncestors := self commonAncestorsWith: aNode.
	^ commonAncestors at: 1 ifAbsent: [nil]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'stephaneducasse 2/4/2006 20:47'!
commonAncestorsWith: aVersionInfo

	| sharedAncestors mergedOrder sorter |
	sorter := MCVersionSorter new
						addVersionInfo: self;
						addVersionInfo: aVersionInfo.
	mergedOrder := sorter sortedVersionInfos.
	sharedAncestors := (sorter allAncestorsOf: self) intersection: (sorter allAncestorsOf: aVersionInfo).
	^ mergedOrder select: [:ea | sharedAncestors includes: ea]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'jrp 7/12/2004 08:16'!
hasAncestor: aVersionInfo
	^ self
		hasAncestor: aVersionInfo
		alreadySeen: OrderedCollection new! !

!MCAncestry methodsFor: 'ancestry' stamp: 'jrp 7/12/2004 08:16'!
hasAncestor: aVersionInfo alreadySeen: aList
	(aList includes: self) ifTrue: [^ false].
	aList add: self.

	^ self = aVersionInfo or: [self ancestors anySatisfy: [:ea | ea hasAncestor: aVersionInfo alreadySeen: aList]]
! !

!MCAncestry methodsFor: 'initializing' stamp: 'alain.plantec 5/28/2009 10:02'!
initialize
	super initialize.
	ancestors := #().
	stepChildren := #()! !

!MCAncestry methodsFor: 'ancestry' stamp: 'avi 2/12/2004 20:58'!
isRelatedTo: aVersionInfo
	^ aVersionInfo timeStamp < self timeStamp
		ifTrue: [self hasAncestor: aVersionInfo]
		ifFalse: [aVersionInfo hasAncestor: self]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'avi 9/11/2004 15:08'!
stepChildren
	^ stepChildren ifNil: [#()]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'avi 9/14/2004 15:21'!
stepChildrenString
	^ String streamContents:
		[:s | self stepChildren do: [:ea | s nextPutAll: ea name] separatedBy: [s nextPutAll: ', ']]! !

!MCAncestry methodsFor: 'ancestry' stamp: 'nice 1/5/2010 15:59'!
topologicalAncestors
	
	^ Array streamContents:
		[:s | | f frontier |
		frontier := MCFrontier frontierOn: self.
		[f := frontier frontier.
		s nextPutAll: f.
		frontier removeAll: f.
		f isEmpty] whileFalse] ! !

!MCAncestry methodsFor: 'ancestry' stamp: 'avi 9/17/2005 16:03'!
withBreadthFirstAncestors
	^ (Array with: self), self breadthFirstAncestors! !

MCAncestry subclass: #MCVersionInfo
	instanceVariableNames: 'id name message date time author'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCVersionInfo commentStamp: '<historical>' prior: 0!
Adds to the record of ancestry, other identifying details.!

!MCVersionInfo class methodsFor: 'as yet unclassified' stamp: 'avi 9/11/2004 10:44'!
name: vName id: id message: message date: date time: time author: author ancestors: ancestors
	^ self 
		name: vName
		id: id
		message: message
		date: date
		time: time
		author: author
		ancestors: ancestors
		stepChildren: #()! !

!MCVersionInfo class methodsFor: 'as yet unclassified' stamp: 'avi 9/11/2004 10:43'!
name: vName id: id message: message date: date time: time author: author ancestors: ancestors stepChildren: stepChildren
	^ self new
		initializeWithName: vName
		id: id
		message: message
		date: date
		time: time
		author: author
		ancestors: ancestors
		stepChildren: stepChildren! !

!MCVersionInfo methodsFor: 'comparing' stamp: 'ab 7/5/2003 14:23'!
= other
	^ other species = self species
		and: [other hasID: id]! !

!MCVersionInfo methodsFor: 'converting' stamp: 'StephaneDucasse 8/17/2012 15:54'!
asDictionary
	^ Dictionary new
		at: #name put: name;
		at: #id put: id asString;
		at: #message put: message;
		at: #date put: date;
		at: #time put: time;
		at: #author put: author;
		at: #ancestors put: (self ancestors collect: [:a | a asDictionary]);
		yourself! !

!MCVersionInfo methodsFor: 'pillaging' stamp: 'cwp 8/1/2003 00:26'!
author
	^ author! !

!MCVersionInfo methodsFor: 'pillaging' stamp: 'cwp 8/1/2003 00:26'!
date
	^ date! !

!MCVersionInfo methodsFor: 'private' stamp: 'ab 7/5/2003 14:10'!
hasID: aUUID
	^ id = aUUID! !

!MCVersionInfo methodsFor: 'comparing' stamp: 'ab 7/5/2003 14:09'!
hash
	^ id hash! !

!MCVersionInfo methodsFor: 'pillaging' stamp: 'cwp 8/1/2003 00:24'!
id
	^ id ! !

!MCVersionInfo methodsFor: 'initialization' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithName: vName id: aUUID message: aString date: aDate time: aTime author: initials ancestors: aCollection stepChildren: stepCollection
	name := vName.
	id := aUUID.
	message := aString.
	date := aDate.
	time := aTime.
	author := initials.
	ancestors :=  aCollection.
	stepChildren := stepCollection! !

!MCVersionInfo methodsFor: 'accessing' stamp: 'ab 7/12/2003 00:04'!
message
	^ message ifNil: ['']! !

!MCVersionInfo methodsFor: 'accessing' stamp: 'ab 7/11/2003 23:33'!
name
	^ name ifNil: ['<working copy>']! !

!MCVersionInfo methodsFor: 'accessing' stamp: 'BernhardPieber 11/21/2010 18:48'!
nameWithout: packageName
	| result |
	result := self name.
	(result beginsWith: packageName , '-') ifTrue: [
		result := result copyFrom: packageName size + 2 to: result size].
	^result! !

!MCVersionInfo methodsFor: 'printing' stamp: 'ab 7/5/2003 18:00'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(; nextPutAll: self name; nextPut: $)
	! !

!MCVersionInfo methodsFor: 'accessing' stamp: 'adrian_lienhard 1/7/2009 17:31'!
summary
	^ String streamContents:
		[:s |
		s
			nextPutAll: self summaryHeader; cr; cr;
			nextPutAll: self message.
		]! !

!MCVersionInfo methodsFor: 'accessing' stamp: 'avi 9/14/2004 15:22'!
summaryHeader
	^ String streamContents:
		[:s |
		s
			nextPutAll: 'Name: '; nextPutAll: self name; cr.
		date ifNotNil:
			[s
				nextPutAll: 'Author: '; nextPutAll: author; cr;
				nextPutAll: 'Time: '; nextPutAll:  date asString, ', ', time asString; cr].
		id ifNotNil:
			[s nextPutAll: 'UUID: '; nextPutAll: id asString; cr].
		s
			nextPutAll: 'Ancestors: '; nextPutAll: self ancestorString.
		self stepChildren isEmpty ifFalse:
			[s cr; nextPutAll: 'Backported From: '; nextPutAll: self stepChildrenString].
		]! !

!MCVersionInfo methodsFor: 'pillaging' stamp: 'cwp 8/1/2003 00:26'!
time
	^ time! !

!MCVersionInfo methodsFor: 'accessing' stamp: 'avi 1/22/2004 16:45'!
timeStamp
	^ TimeStamp date: date time: time! !

!MCVersionInfo methodsFor: 'accessing' stamp: 'avi 9/17/2003 11:24'!
timeString
	^ date asString, ', ', time asString! !

MCAncestry subclass: #MCWorkingAncestry
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCWorkingAncestry commentStamp: '<historical>' prior: 0!
The interim record of ancestry for a working copy, gets merged version added to the ancestry, and is used to create the VersionInfo when the working copy becomes a version. !

!MCWorkingAncestry methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
addAncestor: aNode
	ancestors := (self ancestors reject: [:each | aNode hasAncestor: each])
		copyWith: aNode! !

!MCWorkingAncestry methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
addStepChild: aVersionInfo
	stepChildren := stepChildren copyWith: aVersionInfo! !

!MCWorkingAncestry methodsFor: 'as yet unclassified' stamp: 'MiguelCoba 7/25/2009 02:01'!
infoWithName: nameString message: messageString
	^ MCVersionInfo
		name: nameString
		id: UUID new
		message: messageString
		date: Date today
		time: Time now
		author: Author fullName
		ancestors: ancestors asArray
		stepChildren: self stepChildren asArray! !

!MCWorkingAncestry methodsFor: 'as yet unclassified' stamp: 'avi 2/13/2004 01:14'!
name
	^ '<working copy>'! !

!MCWorkingAncestry methodsFor: 'as yet unclassified' stamp: 'avi 2/13/2004 01:14'!
summary
	^ 'Ancestors: ', self ancestorString! !

Object subclass: #MCDefinition
	instanceVariableNames: ''
	classVariableNames: 'Instances'
	poolDictionaries: ''
	category: 'Monticello-Base'!

!MCDefinition commentStamp: '<historical>' prior: 0!
A MCDefinition is the root of inheritance of entities representing code. !

MCDefinition subclass: #MCClassDefinition
	instanceVariableNames: 'name superclassName variables category type comment commentStamp traitComposition classTraitComposition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCClassDefinition commentStamp: '<historical>' prior: 0!
A MCClassDefinition represents a class.!

!MCClassDefinition class methodsFor: 'obsolete' stamp: 'ab 4/1/2003 01:22'!
name: nameString
superclassName: superclassString
category: categoryString 
instVarNames: ivarArray
classVarNames: cvarArray
poolDictionaryNames: poolArray
classInstVarNames: civarArray
comment: commentString
	^ self	name: nameString
			superclassName: superclassString
			category: categoryString 
			instVarNames: ivarArray
			classVarNames: cvarArray
			poolDictionaryNames: poolArray
			classInstVarNames: civarArray
			type: #normal
			comment: commentString
! !

!MCClassDefinition class methodsFor: 'obsolete' stamp: 'cwp 8/10/2003 16:33'!
name: nameString
superclassName: superclassString
category: categoryString 
instVarNames: ivarArray
classVarNames: cvarArray
poolDictionaryNames: poolArray
classInstVarNames: civarArray
type: typeSymbol
comment: commentString
	^ self 
		name: nameString
		superclassName: superclassString
		category: categoryString 
		instVarNames: ivarArray
		classVarNames: cvarArray
		poolDictionaryNames: poolArray
		classInstVarNames: civarArray
		type: typeSymbol
		comment: commentString
		commentStamp: nil! !

!MCClassDefinition class methodsFor: 'instance creation' stamp: 'al 10/9/2005 19:16'!
name: nameString
superclassName: superclassString
category: categoryString 
instVarNames: ivarArray
classVarNames: cvarArray
poolDictionaryNames: poolArray
classInstVarNames: civarArray
type: typeSymbol
comment: commentString
commentStamp: stampString
	^ self instanceLike:
		(self new initializeWithName: nameString
					superclassName: superclassString
					traitComposition: '{}'
					classTraitComposition: '{}'
					category: categoryString 
					instVarNames: ivarArray
					classVarNames: cvarArray
					poolDictionaryNames: poolArray
					classInstVarNames: civarArray
					type: typeSymbol
					comment: commentString
					commentStamp: stampString)! !

!MCClassDefinition class methodsFor: 'obsolete' stamp: 'ab 4/1/2003 01:22'!
name: nameString
superclassName: superclassString
category: categoryString 
instVarNames: ivarArray
comment: commentString
	^ self	name: nameString
			superclassName: superclassString
			category: categoryString 
			instVarNames: ivarArray
			classVarNames: #()
			poolDictionaryNames: #()
			classInstVarNames: #()
			comment: commentString
! !

!MCClassDefinition class methodsFor: 'instance creation' stamp: 'al 10/10/2005 13:58'!
name: nameString
superclassName: superclassString
traitComposition: traitCompositionString
classTraitComposition: classTraitCompositionString
category: categoryString 
instVarNames: ivarArray
classVarNames: cvarArray
poolDictionaryNames: poolArray
classInstVarNames: civarArray
type: typeSymbol
comment: commentString
commentStamp: stampString
	
	^ self instanceLike:
		(self new initializeWithName: nameString
					superclassName: superclassString
					traitComposition: traitCompositionString
					classTraitComposition: classTraitCompositionString
					category: categoryString 
					instVarNames: ivarArray
					classVarNames: cvarArray
					poolDictionaryNames: poolArray
					classInstVarNames: civarArray
					type: typeSymbol
					comment: commentString
					commentStamp: stampString)! !

!MCClassDefinition methodsFor: 'visiting' stamp: 'nice 10/31/2009 13:11'!
= aDefinition
	^(super = aDefinition)
		and: [superclassName = aDefinition superclassName
		and: [self traitCompositionString = aDefinition traitCompositionString
		and: [self classTraitCompositionString = aDefinition classTraitCompositionString
		and: [category = aDefinition category
		and: [type = aDefinition type
		and: [self sortedVariables = aDefinition sortedVariables
		and: [comment = aDefinition comment]]]]]]]! !

!MCClassDefinition methodsFor: 'visiting' stamp: 'al 10/9/2005 19:33'!
accept: aVisitor
	aVisitor visitClassDefinition: self.
	(self hasClassInstanceVariables or: [self hasClassTraitComposition])
		ifTrue: [aVisitor visitMetaclassDefinition: self].
! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'lr 3/14/2010 21:13'!
actualClass
	^ Smalltalk globals classNamed: self className! !

!MCClassDefinition methodsFor: 'initializing' stamp: 'cwp 7/7/2003 23:19'!
addVariables: aCollection ofType: aClass
	variables addAll: (aCollection collect: [:var | aClass name: var asString]).! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'cwp 11/24/2002 06:23'!
category
	^ category! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'cwp 7/7/2003 23:32'!
classInstVarNames
	^ self selectVariables: #isClassInstanceVariable! !

!MCClassDefinition methodsFor: 'printing' stamp: 'cwp 7/7/2003 23:35'!
classInstanceVariablesString
	^ self stringForVariablesOfType: #isClassInstanceVariable! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'ab 11/13/2002 01:52'!
className
	^ name! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 20:07'!
classTraitComposition
	^classTraitComposition! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 21:55'!
classTraitCompositionString
	^self classTraitComposition ifNil: ['{}'].! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'nice 10/20/2009 22:03'!
classVarNames
	^(self selectVariables: #isClassVariable) asArray sort! !

!MCClassDefinition methodsFor: 'printing' stamp: 'eem 4/30/2009 16:47'!
classVariablesString
	^ self stringForSortedVariablesOfType: #isClassVariable! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'ab 1/15/2003 13:42'!
comment
	^ comment! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'cwp 8/10/2003 16:40'!
commentStamp
	^ commentStamp! !

!MCClassDefinition methodsFor: 'installing' stamp: 'MartinDias 1/14/2013 15:56'!
createClass
	| superClass class composition |
	superClass := superclassName == #nil ifFalse:
						[ Smalltalk globals at: superclassName ].
	[ class := ClassBuilder new
		name: name
		inEnvironment: superClass environment
		subclassOf: superClass
		type: type
		instanceVariableNames: self instanceVariablesString
		classVariableNames: self classVariablesString
		poolDictionaries: self sharedPoolsString
		category: category.] on: Warning, DuplicatedVariableError do: [:ex| ex resume].

	self traitComposition
		ifNotNil: [ composition := self class evaluatorClass evaluate: self traitComposition.
				  ((((composition isCollection and: [ ( composition includes: nil ) not])  
					or: [composition isKindOf: TraitComposition ])
						or: [composition isKindOf: Trait ])
							or: [composition isKindOf: TraitTransformation ])
					ifTrue: [ class setTraitComposition: composition asTraitComposition ] ].
	self classTraitComposition
		ifNotNil: [ composition := self class evaluatorClass evaluate: self classTraitComposition.
				  ((((composition isCollection and: [ ( composition includes: nil ) not])  
					or: [composition isKindOf: TraitComposition ])
						or: [composition isKindOf: Trait ])
							or: [composition isKindOf: TraitTransformation ])
					ifTrue: [ class class setTraitComposition: composition asTraitComposition ] ].
	^ class! !

!MCClassDefinition methodsFor: 'initializing' stamp: 'NicoPaez 10/1/2010 10:41'!
defaultCommentStamp
	^ String new

	"The version below avoids stomping on stamps already in the image

	^ (Smalltalk globals at: name ifPresent: [:c | c organization commentStamp])
		ifNil: ['']
	"
! !

!MCClassDefinition methodsFor: 'printing' stamp: 'cwp 8/2/2003 02:03'!
definitionString
	^ String streamContents: [:stream | self printDefinitionOn: stream]! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'ab 12/5/2002 21:24'!
description
	^ Array with: name
! !

!MCClassDefinition methodsFor: 'testing' stamp: 'cwp 8/2/2003 02:54'!
hasClassInstanceVariables
	^ (self selectVariables: #isClassInstanceVariable) isEmpty not! !

!MCClassDefinition methodsFor: 'testing' stamp: 'al 10/9/2005 21:59'!
hasClassTraitComposition
	^self classTraitCompositionString ~= '{}'! !

!MCClassDefinition methodsFor: 'testing' stamp: 'al 10/9/2005 20:13'!
hasComment
	^ comment isEmptyOrNil not! !

!MCClassDefinition methodsFor: 'testing' stamp: 'al 3/29/2006 00:27'!
hasTraitComposition
	^self traitCompositionString ~= '{}'! !

!MCClassDefinition methodsFor: 'comparing' stamp: 'stephaneducasse 2/4/2006 20:47'!
hash
	| hash |
	hash := String stringHash: name initialHash: 0.
	hash := String stringHash: superclassName initialHash: hash.
	hash := String stringHash: self traitCompositionString initialHash: hash.
	hash := String stringHash: self classTraitComposition asString initialHash: hash.
	hash := String stringHash: (category ifNil: ['']) initialHash: hash.
	hash := String stringHash: type initialHash: hash.
	variables do: [
		:v |
		hash := String stringHash: v name initialHash: hash.
	].
	^ hash! !

!MCClassDefinition methodsFor: 'initializing' stamp: 'bf 8/12/2009 10:55'!
initializeWithName: nameString
superclassName: superclassString
category: categoryString 
instVarNames: ivarArray
classVarNames: cvarArray
poolDictionaryNames: poolArray
classInstVarNames: civarArray
type: typeSymbol
comment: commentString
commentStamp: stampStringOrNil
	name := nameString asSymbol.
	superclassName := superclassString ifNil: ['nil'] ifNotNil: [superclassString asSymbol].
	category := categoryString.
	name = #CompiledMethod ifTrue: [type := #compiledMethod] ifFalse: [type := typeSymbol].
	comment := commentString withSqueakLineEndings.
	commentStamp := stampStringOrNil ifNil: [self defaultCommentStamp].
	variables := OrderedCollection  new.
	self addVariables: ivarArray ofType: MCInstanceVariableDefinition.
	self addVariables: cvarArray asSortedCollection ofType: MCClassVariableDefinition.
	self addVariables: poolArray asSortedCollection ofType: MCPoolImportDefinition.
	self addVariables: civarArray ofType: MCClassInstanceVariableDefinition.! !

!MCClassDefinition methodsFor: 'initializing' stamp: 'bf 8/12/2009 10:55'!
initializeWithName: nameString
superclassName: superclassString
traitComposition: traitCompositionString
classTraitComposition: classTraitCompositionString
category: categoryString 
instVarNames: ivarArray
classVarNames: cvarArray
poolDictionaryNames: poolArray
classInstVarNames: civarArray
type: typeSymbol
comment: commentString
commentStamp: stampStringOrNil
	name := nameString asSymbol.
	superclassName := superclassString ifNil: ['nil'] ifNotNil: [superclassString asSymbol].
	traitComposition := traitCompositionString.
	classTraitComposition := classTraitCompositionString.
	category := categoryString.
	name = #CompiledMethod ifTrue: [type := #compiledMethod] ifFalse: [type := typeSymbol].
	comment := commentString withSqueakLineEndings.
	commentStamp := stampStringOrNil ifNil: [self defaultCommentStamp].
	variables := OrderedCollection  new.
	self addVariables: ivarArray ofType: MCInstanceVariableDefinition.
	self addVariables: cvarArray asSortedCollection ofType: MCClassVariableDefinition.
	self addVariables: poolArray asSortedCollection ofType: MCPoolImportDefinition.
	self addVariables: civarArray ofType: MCClassInstanceVariableDefinition.! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'cwp 7/7/2003 23:33'!
instVarNames
	^ self selectVariables: #isInstanceVariable! !

!MCClassDefinition methodsFor: 'printing' stamp: 'cwp 7/7/2003 23:35'!
instanceVariablesString
	^ self stringForVariablesOfType: #isInstanceVariable! !

!MCClassDefinition methodsFor: 'testing' stamp: 'ab 12/4/2002 21:52'!
isClassDefinition
	^ true! !

!MCClassDefinition methodsFor: 'testing' stamp: 'ab 5/24/2003 13:49'!
isCodeDefinition
	^ true! !

!MCClassDefinition methodsFor: 'printing' stamp: 'cwp 11/24/2002 22:16'!
kindOfSubclass
	type = #normal ifTrue: [^ ' subclass: '].
	type = #words ifTrue: [^ ' variableWordSubclass: '].
	type = #variable ifTrue: [^ ' variableSubclass: '].
	type = #bytes ifTrue: [^ ' variableByteSubclass: '].
	type = #weak ifTrue: [^ ' weakSubclass: ' ].
	type = #compiledMethod ifTrue: [^ ' variableByteSubclass: ' ].
	self error: 'Unrecognized class type'! !

!MCClassDefinition methodsFor: 'installing' stamp: 'EstebanLorenzano 8/3/2012 15:29'!
load
	self createClass ifNotNil: [:class |
		SystemAnnouncer uniqueInstance suspendAllWhile: [
			class class instanceVariableNames: self classInstanceVariablesString.
			self hasComment ifTrue: [class classComment: comment stamp: commentStamp]]].! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'cwp 7/7/2003 23:53'!
poolDictionaries
	^ self selectVariables: #isPoolImport! !

!MCClassDefinition methodsFor: 'annotations' stamp: 'StephaneDucasse 12/30/2012 18:04'!
printAnnotations: requests on: aStream
	"Add a string for an annotation pane, trying to fulfill the browser annotationRequests"

	requests do: [ :aRequest |
		aRequest == #requirements ifTrue: [
			self requirements do: [ :req | aStream nextPutAll: req ] separatedBy: [ aStream space ]]
	] separatedBy: [ aStream space ].! !

!MCClassDefinition methodsFor: 'printing' stamp: 'al 3/28/2006 23:42'!
printDefinitionOn: stream
		stream 
			nextPutAll: self superclassName;
			nextPutAll: self kindOfSubclass;
			nextPut: $# ;
			nextPutAll: self className;
			cr; tab.
		self hasTraitComposition ifTrue: [
			stream 
				nextPutAll: 'uses: ';
		 		nextPutAll: self traitCompositionString;
				cr; tab ].
		stream
			nextPutAll: 'instanceVariableNames: ';
			store: self instanceVariablesString;
			cr; tab;
			nextPutAll: 'classVariableNames: ';
			store: self classVariablesString;
			cr; tab;
			nextPutAll: 'poolDictionaries: ';
			store: self sharedPoolsString;
			cr; tab;
			nextPutAll: 'category: ';
			store: self category asString! !

!MCClassDefinition methodsFor: 'printing' stamp: 'lr 3/20/2010 21:23'!
printMetaDefinitionOn: stream
	stream 
		nextPutAll: self className;
		nextPutAll: ' class'; cr; tab.
	self hasClassTraitComposition ifTrue: [
		stream
			nextPutAll: 'uses: ';
			nextPutAll: self classTraitCompositionString;
			cr; tab ].
	stream
		nextPutAll: ' instanceVariableNames: '; 
		store: self classInstanceVariablesString! !

!MCClassDefinition methodsFor: 'comparing' stamp: 'ab 5/24/2003 14:12'!
provisions
	^ Array with: name! !

!MCClassDefinition methodsFor: 'comparing' stamp: 'StephaneDucasse 2/11/2012 09:14'!
requirements
	^superclassName == #nil
		ifTrue: [self poolDictionaries]
		ifFalse: [(Array with: superclassName), self poolDictionaries]! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'cwp 7/7/2003 23:28'!
selectVariables: aSelector
	^ variables select: [:v | v perform: aSelector] thenCollect: [:v | v name]! !

!MCClassDefinition methodsFor: 'printing' stamp: 'eem 4/30/2009 16:47'!
sharedPoolsString
	^ self stringForSortedVariablesOfType: #isPoolImport! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'ab 7/19/2003 18:00'!
sortKey
	^ self className! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'bf 8/29/2006 11:45'!
sortedVariables
	"sort variables for comparison purposes"

	| sorted |
	sorted := variables select: [:var | var isOrderDependend].
	sorted addAll: ((variables reject: [:var | var isOrderDependend])
		asSortedCollection: [:a :b | a name <= b name]).
	^sorted! !

!MCClassDefinition methodsFor: 'printing' stamp: 'cwp 7/10/2003 01:29'!
source
	^ self definitionString! !

!MCClassDefinition methodsFor: 'storing' stamp: 'al 7/4/2006 10:14'!
storeDataOn: aDataStream
	| instVarSize |
	instVarSize := (self hasTraitComposition or: [ self hasClassTraitComposition ])
		ifTrue: [ self class instSize ]
		ifFalse: [ self class instSize - 2 ].
	aDataStream
		beginInstance: self class
		size: instVarSize.
	1 to: instVarSize do: [ :index |
		aDataStream nextPut: (self instVarAt: index) ].! !

!MCClassDefinition methodsFor: 'installing' stamp: 'eem 4/30/2009 16:47'!
stringForSortedVariablesOfType: aSymbol
	^ String streamContents:
		[:stream |
		(self selectVariables: aSymbol) asSortedCollection
			do: [:ea | stream nextPutAll: ea]
			separatedBy: [stream space]]! !

!MCClassDefinition methodsFor: 'installing' stamp: 'cwp 2/3/2004 21:35'!
stringForVariablesOfType: aSymbol
	^ String streamContents:
		[:stream |
		(self selectVariables: aSymbol) 
			do: [:ea | stream nextPutAll: ea]
			separatedBy: [stream space]]! !

!MCClassDefinition methodsFor: 'printing' stamp: 'ab 11/16/2002 17:33'!
summary
	^ name! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'ab 11/13/2002 17:41'!
superclassName
	^ superclassName! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 20:16'!
traitComposition
	^traitComposition! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 21:55'!
traitCompositionString
	^self traitComposition ifNil: ['{}'].! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'cwp 11/24/2002 22:35'!
type
	^ type! !

!MCClassDefinition methodsFor: 'installing' stamp: 'NicoPaez 10/1/2010 10:21'!
unload
	Smalltalk globals removeClassNamed: name! !

!MCClassDefinition methodsFor: 'accessing' stamp: 'cwp 11/25/2002 06:51'!
variables
	^ variables! !

MCClassDefinition subclass: #MCTraitDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCTraitDefinition commentStamp: '<historical>' prior: 0!
A MCTraitDefinition represents a trait.!

!MCTraitDefinition class methodsFor: 'as yet unclassified' stamp: 'al 10/9/2005 20:28'!
name: classNameString traitComposition:  traitCompositionString category:  categoryString comment:  commentString commentStamp:   commentStamp
	^ self instanceLike:
		(self new initializeWithName: classNameString 
			traitComposition:  traitCompositionString
			category:  categoryString
			comment:  commentString  
			commentStamp:   commentStamp)
! !

!MCTraitDefinition methodsFor: 'visiting' stamp: 'nice 10/31/2009 13:10'!
= aDefinition
	self flag: #traits. "Ugly we harcoded the super superclass method.  We will have to refactor the definition hierarchy"
	
	^ (aDefinition isKindOf: MCDefinition)
		and: [(self isRevisionOf: aDefinition)
		and: [self traitCompositionString = aDefinition traitCompositionString
		and: [category = aDefinition category
		and: [comment = aDefinition comment]]]]! !

!MCTraitDefinition methodsFor: 'visiting' stamp: 'al 10/9/2005 20:28'!
accept: aVisitor
	^ aVisitor visitTraitDefinition: self
! !

!MCTraitDefinition methodsFor: 'visiting' stamp: 'jb 7/1/2011 10:52'!
createClass
	^Trait
		named: name
		uses: (self class evaluatorClass evaluate: self traitCompositionString)
		category: category
		
! !

!MCTraitDefinition methodsFor: 'testing' stamp: 'al 10/9/2005 20:28'!
hasClassInstanceVariables
	^ false

! !

!MCTraitDefinition methodsFor: 'comparing' stamp: 'al 6/5/2006 14:14'!
hash
	| hash |
	hash := String stringHash: name initialHash: 0.
	hash := String stringHash: self traitCompositionString initialHash: hash.
	hash := String stringHash: (category ifNil: ['']) initialHash: hash.
	^ hash
! !

!MCTraitDefinition methodsFor: 'initializing' stamp: 'al 6/5/2006 14:14'!
initializeWithName: classNameString 
	traitComposition:  traitCompositionString
	category:  categoryString
	comment:  commentString  
	commentStamp:   commentStampString
					
		name := classNameString asSymbol.
		traitComposition := traitCompositionString.
	     category := categoryString.
		comment := commentString withSqueakLineEndings.
		commentStamp :=  commentStampString ifNil: [self defaultCommentStamp]
! !

!MCTraitDefinition methodsFor: 'installing' stamp: 'marcus.denker 11/10/2008 10:04'!
load
	 self createClass ifNotNil: [:trait |
		self hasComment ifTrue: [trait classComment: comment stamp: commentStamp]]! !

!MCTraitDefinition methodsFor: 'printing' stamp: 'al 6/5/2006 14:15'!
printDefinitionOn: stream
	stream nextPutAll: 'Trait named: #', self className;
		 cr;
		 tab;
		 nextPutAll: 'uses: ';
		 nextPutAll: self traitCompositionString;
		 cr;
		 tab;
		 nextPutAll: 'category: ';
		 store: self category asString
! !

!MCTraitDefinition methodsFor: 'comparing' stamp: 'al 6/5/2006 14:13'!
requirements
	"Assuming that traits in a composition can be identified by
	testing for the first character beeing an uppercase character
	(and thus not a special character such as {, # etc.)"

	| tokens traitNames |
	self hasTraitComposition ifFalse: [ ^Array new ].
	tokens := Scanner new scanTokens: self traitComposition.
	traitNames := tokens select: [:each | each first isUppercase].
	^traitNames asArray! !

MCDefinition subclass: #MCClassTraitDefinition
	instanceVariableNames: 'baseTrait classTraitComposition category'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCClassTraitDefinition class methodsFor: 'instance creation' stamp: 'damiencassou 7/30/2009 12:32'!
baseTraitName: aString classTraitComposition: classTraitCompositionString
	^ self baseTraitName: aString classTraitComposition: classTraitCompositionString category: nil! !

!MCClassTraitDefinition class methodsFor: 'instance creation' stamp: 'damiencassou 7/30/2009 12:11'!
baseTraitName: aString classTraitComposition: classTraitCompositionString category: aCategoryString
	^self instanceLike: (
		self new
			initializeWithBaseTraitName: aString
			classTraitComposition: classTraitCompositionString
			category: aCategoryString).! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'nice 10/31/2009 13:11'!
= aDefinition
	^ (super = aDefinition)
		and: [baseTrait = aDefinition baseTrait
		and: [self classTraitCompositionString = aDefinition classTraitCompositionString]]

! !

!MCClassTraitDefinition methodsFor: 'as yet unclassified' stamp: 'al 10/9/2005 20:23'!
accept: aVisitor
	^ aVisitor visitClassTraitDefinition: self.! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 20:23'!
baseTrait
	^baseTrait
! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'MarcusDenker 10/29/2010 11:30'!
category
	^ category ifNil: [(Smalltalk classOrTraitNamed: self baseTrait)
							ifNotNil: [:bTrait | bTrait category]
							ifNil: [self error: 'Can''t detect the category']] ! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'al 12/15/2005 11:31'!
className
	^self baseTrait! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 20:23'!
classTraitComposition
	^classTraitComposition

! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'al 10/10/2005 10:12'!
classTraitCompositionString
	^self classTraitComposition ifNil: ['{}'].

! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 21:59'!
definitionString
	^self baseTrait , ' classTrait
	uses: ' , self classTraitCompositionString.
! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 20:24'!
description
	^Array
		with: baseTrait
		with: classTraitComposition! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'al 6/5/2006 14:04'!
hash
	| hash |
	hash := String stringHash: baseTrait initialHash: 0.
	hash := String stringHash: self classTraitCompositionString initialHash: hash.
	^hash
! !

!MCClassTraitDefinition methodsFor: 'initialization' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithBaseTraitName: aTraitName classTraitComposition: aString
	baseTrait := aTraitName.
	classTraitComposition := aString.! !

!MCClassTraitDefinition methodsFor: 'initialization' stamp: 'damiencassou 7/30/2009 12:12'!
initializeWithBaseTraitName: aTraitName classTraitComposition: aString category: aCategoryString
	baseTrait := aTraitName.
	classTraitComposition := aString.
	category := aCategoryString! !

!MCClassTraitDefinition methodsFor: 'testing' stamp: 'adrian-lienhard 5/11/2009 16:47'!
isClassDefinition
	"Traits are treated the same like classes."
	
	^ true! !

!MCClassTraitDefinition methodsFor: 'installing' stamp: 'jb 7/1/2011 10:52'!
load	
	self class evaluatorClass evaluate: self definitionString! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 20:24'!
requirements
	^Array with: baseTrait! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 20:25'!
sortKey
	^ self baseTrait name , '.classTrait'! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 20:25'!
source
	^self definitionString! !

!MCClassTraitDefinition methodsFor: 'accessing' stamp: 'al 10/9/2005 20:25'!
summary
	^self baseTrait , ' classTrait'
! !

!MCDefinition class methodsFor: 'cleanup' stamp: 'StephaneDucasse 3/9/2010 16:33'!
cleanUp
	"Flush caches"

	self clearInstances.! !

!MCDefinition class methodsFor: 'cleanup' stamp: 'stephaneducasse 2/4/2006 20:47'!
clearInstances
	WeakArray removeWeakDependent: Instances.
	Instances := nil! !

!MCDefinition class methodsFor: 'instance creation' stamp: 'stephaneducasse 2/4/2006 20:47'!
instanceLike: aDefinition
	Instances ifNil: [Instances := WeakSet new].
	^ (Instances like: aDefinition) ifNil: [Instances add: aDefinition]! !

!MCDefinition methodsFor: 'comparing' stamp: 'ab 7/19/2003 17:59'!
<= other
	^ self sortKey <= other sortKey! !

!MCDefinition methodsFor: 'comparing' stamp: 'nice 10/31/2009 13:08'!
= aDefinition
	^(aDefinition isKindOf: MCDefinition) and: [self isRevisionOf: aDefinition]! !

!MCDefinition methodsFor: 'installing' stamp: 'AdrianLienhard 1/21/2010 22:14'!
addMethodAdditionTo: aCollection
	self load! !

!MCDefinition methodsFor: 'annotations' stamp: 'StephaneDucasse 12/30/2012 17:54'!
annotations
	^self annotations: Annotation annotationRequests! !

!MCDefinition methodsFor: 'annotations' stamp: 'AlainPlantec 1/7/2010 22:19'!
annotations: requests
	"Answer a string for an annotation pane, trying to fulfill the CodeHolder annotationRequests."
	^String streamContents: [ :s | self printAnnotations: requests on: s ].! !

!MCDefinition methodsFor: 'comparing' stamp: 'ab 7/19/2003 18:14'!
description
	self subclassResponsibility! !

!MCDefinition methodsFor: 'comparing' stamp: 'nk 10/21/2003 23:18'!
fullTimeStamp
	^TimeStamp current! !

!MCDefinition methodsFor: 'comparing' stamp: 'ab 7/19/2003 18:14'!
hash
	^ self description hash! !

!MCDefinition methodsFor: 'testing' stamp: 'ab 12/4/2002 21:51'!
isClassDefinition
	^false! !

!MCDefinition methodsFor: 'testing' stamp: 'bf 11/12/2004 14:46'!
isClassDefinitionExtension
	"Answer true if this definition extends the regular class definition"
	^false! !

!MCDefinition methodsFor: 'testing' stamp: 'ab 12/4/2002 21:51'!
isMethodDefinition
	^false! !

!MCDefinition methodsFor: 'testing' stamp: 'cwp 7/11/2003 01:32'!
isOrganizationDefinition
	^false! !

!MCDefinition methodsFor: 'comparing' stamp: 'damiencassou 11/27/2008 18:15'!
isRevisionOf: aDefinition
	^ (aDefinition isKindOf: MCDefinition) and: [aDefinition description = self description]! !

!MCDefinition methodsFor: 'comparing' stamp: 'ab 7/19/2003 18:25'!
isSameRevisionAs: aDefinition
	^ self = aDefinition! !

!MCDefinition methodsFor: 'testing' stamp: 'bf 8/12/2009 22:55'!
isScriptDefinition
	^false! !

!MCDefinition methodsFor: 'installing' stamp: 'ab 7/18/2003 21:31'!
load
	! !

!MCDefinition methodsFor: 'installing' stamp: 'avi 2/17/2004 13:19'!
loadOver: aDefinition
	self load
	! !

!MCDefinition methodsFor: 'installing' stamp: 'ab 7/18/2003 19:48'!
postload! !

!MCDefinition methodsFor: 'installing' stamp: 'avi 2/17/2004 13:19'!
postloadOver: aDefinition
	self postload! !

!MCDefinition methodsFor: 'annotations' stamp: 'StephaneDucasse 12/30/2012 18:04'!
printAnnotations: requests on: aStream
	"Add a string for an annotation pane, trying to fulfill the browser annotationRequests."

	aStream nextPutAll: 'not yet implemented'! !

!MCDefinition methodsFor: 'printing' stamp: 'ab 7/18/2003 19:43'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: '(', self summary, ')'! !

!MCDefinition methodsFor: 'accessing' stamp: 'ab 5/24/2003 14:12'!
provisions
	^ #()! !

!MCDefinition methodsFor: 'accessing' stamp: 'ab 5/24/2003 14:12'!
requirements
	^ #()! !

!MCDefinition methodsFor: 'comparing' stamp: 'ab 7/19/2003 18:04'!
sortKey
	self subclassResponsibility ! !

!MCDefinition methodsFor: 'printing' stamp: 'ab 7/19/2003 18:23'!
summary
	self subclassResponsibility ! !

!MCDefinition methodsFor: 'printing' stamp: 'StephaneDucasse 8/17/2012 16:31'!
summarySuffixOver: previousDefinition
	^self source = previousDefinition source
		ifTrue: [ ' (source same but rev changed)' ]
		ifFalse: [ ' (changed)' ]! !

!MCDefinition methodsFor: 'installing' stamp: 'ab 11/14/2002 00:08'!
unload! !

MCDefinition subclass: #MCMethodDefinition
	instanceVariableNames: 'classIsMeta source category selector className timeStamp'
	classVariableNames: 'Definitions InitializersEnabled'
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCMethodDefinition commentStamp: '<historical>' prior: 0!
A MCMethodDefinition represents a method definition. 
It captures the following information.

Instance Variables
	category:		<Object>
	classIsMeta:		<Object>
	className:		<Object>
	selector:		<Object>
	source:		<Object>
	timeStamp:		<Object>

!

!MCMethodDefinition class methodsFor: 'class initialization' stamp: 'CamilloBruni 6/2/2012 00:13'!
cachedDefinitions
	Definitions ifNil: [ Definitions := WeakIdentityKeyDictionary new ].
	^ Definitions! !

!MCMethodDefinition class methodsFor: 'instance creation' stamp: 'ab 7/26/2003 02:05'!
className: classString
classIsMeta: metaBoolean
selector: selectorString
category: catString
timeStamp: timeString
source: sourceString
	^ self instanceLike:
		(self new initializeWithClassName: classString
					classIsMeta: metaBoolean
					selector: selectorString
					category: catString
					timeStamp: timeString
					source: sourceString)! !

!MCMethodDefinition class methodsFor: 'instance creation' stamp: 'ab 4/1/2003 01:40'!
className: classString
selector: selectorString
category: catString
timeStamp: timeString
source: sourceString
	^ self	className: classString
			classIsMeta: false
			selector: selectorString
			category: catString
			timeStamp: timeString
			source: sourceString! !

!MCMethodDefinition class methodsFor: 'cleanup' stamp: 'StephaneDucasse 3/9/2010 16:34'!
cleanUp
	"Flush caches"

	self shutDown.! !

!MCMethodDefinition class methodsFor: 'class initialization' stamp: 'StephaneDucasse 12/28/2012 21:31'!
flushMethodCache
	"We do not named this method flushCache because it would override an important class methods."
	
	Definitions := nil.! !

!MCMethodDefinition class methodsFor: 'instance creation' stamp: 'CamilloBruni 6/2/2012 00:14'!
forMethodReference: aMethodReference
	| definition |
	definition := self cachedDefinitions at: aMethodReference compiledMethod ifAbsent: [].
	(definition isNil
		or: [definition selector ~= aMethodReference selector
		or: [definition className ~= aMethodReference classSymbol
		or: [definition classIsMeta ~= aMethodReference classIsMeta
		or: [definition category ~= aMethodReference category]]]])
			ifTrue: [
				definition := self 
					className: aMethodReference classSymbol
					classIsMeta: aMethodReference classIsMeta
					selector: aMethodReference selector
					category: aMethodReference category
					timeStamp: aMethodReference timeStampString
					source: aMethodReference source.
				self cachedDefinitions at: aMethodReference compiledMethod put: definition].
	^ definition
	! !

!MCMethodDefinition class methodsFor: 'class initialization' stamp: 'ab 8/22/2003 18:14'!
initialize
	Smalltalk addToShutDownList: self! !

!MCMethodDefinition class methodsFor: 'settings' stamp: 'PavelKrivanek 6/22/2011 11:40'!
initializersEnabled

	^ InitializersEnabled ifNil: [true]! !

!MCMethodDefinition class methodsFor: 'settings' stamp: 'PavelKrivanek 6/22/2011 11:40'!
initializersEnabled: aBoolean

	InitializersEnabled := aBoolean! !

!MCMethodDefinition class methodsFor: 'class initialization' stamp: 'StephaneDucasse 12/28/2012 21:31'!
shutDown
	"Free up all cached monticello method definitions"
	self flushMethodCache ! !

!MCMethodDefinition methodsFor: 'comparing' stamp: 'CamilloBruni 2/28/2012 13:44'!
= aDefinition
	^(super = aDefinition)
		and: [aDefinition category = self category
		and: [aDefinition timeStamp = self timeStamp
		and: [aDefinition source = self source]]]! !

!MCMethodDefinition methodsFor: 'visiting' stamp: 'ab 7/18/2003 21:47'!
accept: aVisitor
	^ aVisitor visitMethodDefinition: self! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'lr 3/14/2010 21:13'!
actualClass
	^ Smalltalk globals
		at: className
		ifPresent: [ :class | 
			classIsMeta
				ifTrue: [ class classSide ]
				ifFalse: [ class ] ]! !

!MCMethodDefinition methodsFor: 'installing' stamp: 'stephaneDucasse 5/8/2010 19:58'!
addMethodAdditionTo: aCollection
	| methodAddition |
	methodAddition := MethodAddition new
		compile: source
		classified: category
		withStamp: timeStamp
		notifying: nil
		logSource: true
		inClass: self actualClass.
	"This might raise an exception and never return"
	methodAddition createCompiledMethod.
	aCollection add: methodAddition.
! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'ab 11/13/2002 01:59'!
category
	^ category! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'cwp 11/25/2002 07:26'!
classIsMeta
	^ classIsMeta! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'ab 11/15/2002 01:12'!
className
	^className! !

!MCMethodDefinition methodsFor: 'printing' stamp: 'ab 12/5/2002 21:25'!
description
	^ Array	
		with: className
		with: selector
		with: classIsMeta! !

!MCMethodDefinition methodsFor: 'printing' stamp: 'al 12/3/2005 12:15'!
fullClassName
	"Using #class selector for classes for backwards compatibility"

	^ self classIsMeta
		ifFalse: [self className]
		ifTrue: [
			(self actualClass isNil or: [ self actualClass isTrait ])
				ifFalse: [self className, ' class']
				ifTrue: [self className, ' classSide']]! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'StephaneDucasse 7/7/2010 18:47'!
fullTimeStamp
	
	^TimeStamp fromMethodTimeStamp: timeStamp! !

!MCMethodDefinition methodsFor: 'comparing' stamp: 'stephaneducasse 2/4/2006 20:47'!
hash
	| hash |
	hash := String stringHash: classIsMeta asString initialHash: 0.
	hash := String stringHash: source initialHash: hash.
	hash := String stringHash: category initialHash: hash.
	hash := String stringHash: className initialHash: hash.
	^ hash! !

!MCMethodDefinition methodsFor: 'serializing' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithClassName: classString
classIsMeta: metaBoolean
selector: selectorString
category: catString
timeStamp: timeString
source: sourceString
	className := classString asSymbol.
	selector := selectorString asSymbol.
	category := catString asSymbol.
	timeStamp := timeString.
	classIsMeta := metaBoolean.
	source := sourceString withSqueakLineEndings.
! !

!MCMethodDefinition methodsFor: 'testing' stamp: 'ab 5/24/2003 13:49'!
isCodeDefinition
	^ true! !

!MCMethodDefinition methodsFor: 'installing' stamp: 'avi 9/17/2003 22:27'!
isExtensionMethod
	^ category beginsWith: '*'! !

!MCMethodDefinition methodsFor: 'installing' stamp: 'NicoPaez 10/1/2010 10:30'!
isExternalStructureFieldDefinition
	"Really belongs in the FFI package, for now, include in base even if FFI is not loaded due to the dire consequences if its not here and an FFI package is loaded (incorrect field compilation can cause VM crashes).
	
	If in the future other external packages are found to need custom post-load blocks as well, a pluggable way of doing so should be investigated, but for now it's not worth the effort and coordination required with external package maintainers"
	^ selector = #fields
		and: [classIsMeta
			and: [
				(Smalltalk globals at: #ExternalStructure ifPresent: [:externalStructure |
					self actualClass theNonMetaClass inheritsFrom: externalStructure]) == true]]
! !

!MCMethodDefinition methodsFor: 'testing' stamp: 'ab 8/8/2003 17:05'!
isInitializer
	^ selector = #initialize and: [classIsMeta]
	! !

!MCMethodDefinition methodsFor: 'testing' stamp: 'GuillermoPolito 1/11/2012 22:53'!
isLoadable
	^self actualClass notNil! !

!MCMethodDefinition methodsFor: 'testing' stamp: 'ab 12/4/2002 21:52'!
isMethodDefinition
	^true! !

!MCMethodDefinition methodsFor: 'installing' stamp: 'avi 11/10/2003 15:45'!
isOverrideMethod
	"this oughta check the package"
	^ self isExtensionMethod and: [category endsWith: '-override']! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'stephaneDucasse 5/8/2010 19:58'!
load
	self actualClass
		compile: source
		classified: category
		withStamp: timeStamp
		notifying: nil! !

!MCMethodDefinition methodsFor: 'installing' stamp: 'PavelKrivanek 6/22/2011 09:47'!
postloadOver: aDefinition
	super postloadOver: aDefinition.
	self class initializersEnabled ifTrue: [
		(self isInitializer
			and: [ self actualClass isTrait not 
					and: [ aDefinition isNil or: [ self source ~= aDefinition source ]]]) ifTrue: [
				self actualClass theNonMetaClass initialize ] ].
	"Postloading of FFI fields."
	self isExternalStructureFieldDefinition
		ifTrue: [self actualClass theNonMetaClass compileFields].! !

!MCMethodDefinition methodsFor: 'annotations' stamp: 'StephaneDucasse 12/30/2012 18:04'!
printAnnotations: requests on: aStream
	"Add a string for an annotation pane, trying to fulfill the browser annotationRequests."
	
	requests do: [ :aRequest |
		aRequest == #timeStamp ifTrue: [ aStream nextPutAll: self timeStamp ].
		aRequest == #messageCategory ifTrue: [ aStream nextPutAll: self category ].
		aRequest == #requirements ifTrue: [
			self requirements do: [ :req |
				aStream nextPutAll: req ] separatedBy: [ aStream space ]].
	] separatedBy: [ aStream space ].! !

!MCMethodDefinition methodsFor: 'installing' stamp: 'ar 12/12/2009 17:05'!
removeSelector: aSelector fromClass: aClass
	"Safely remove the given selector from the target class.
	Be careful not to remove the selector when it has wondered
	to another package."
	| newCategory |
	newCategory := aClass organization categoryOfElement: aSelector.
	newCategory ifNotNil:[
		"If moved to and fro extension, ignore removal"
		(category beginsWith: '*') = (newCategory beginsWith: '*') ifFalse:[^self].
		"Check if moved between different extension categories"
		((category beginsWith: '*') and:[category ~= newCategory]) ifTrue:[^self]].
	aClass removeSelector: aSelector.
! !

!MCMethodDefinition methodsFor: 'comparing' stamp: 'ab 5/24/2003 14:11'!
requirements
	^ Array with: className! !

!MCMethodDefinition methodsFor: 'installing' stamp: 'StephaneDucasse 5/28/2011 13:34'!
scanForPreviousVersion
	| sourceFilesCopy method position |
	method := self actualClass compiledMethodAt: selector ifAbsent: [ ^ nil ].
	position := method filePosition.
	sourceFilesCopy := SourceFiles
		collect: [ :x | 
			x isNil
				ifTrue: [ nil ]
				ifFalse: [ x readOnlyCopy ] ].
	[ 
	| file prevPos tokens preamble methodCategory stamp prevFileIndex |
	method fileIndex = 0
		ifTrue: [ ^ nil ].
	file := sourceFilesCopy at: method fileIndex.
	[ position notNil & file notNil ]
		whileTrue: [ 
			file position: (0 max: position - 150).	"Skip back to before the preamble"
			[ file position < (position - 1) ] whileTrue: [ preamble := file nextChunk ].	"then pick it up from the front"	"Preamble is likely a linked method preamble, if we're in
			a changes file (not the sources file).  Try to parse it
			for prior source position and file index"
			prevPos := nil.
			stamp := ''.
			(preamble findString: 'methodsFor:' startingAt: 1) > 0
				ifTrue: [ tokens := Scanner new scanTokens: preamble ]
				ifFalse: [ tokens := Array new	"ie cant be back ref" ].
			((tokens size between: 7 and: 8) and: [ (tokens at: tokens size - 5) = #methodsFor: ])
				ifTrue: [ 
					(tokens at: tokens size - 3) = #stamp:
						ifTrue: [ 
							"New format gives change stamp and unified prior pointer"
							stamp := tokens at: tokens size - 2.
							prevPos := tokens last.
							prevFileIndex := sourceFilesCopy fileIndexFromSourcePointer: prevPos.
							prevPos := sourceFilesCopy filePositionFromSourcePointer: prevPos ]
						ifFalse: [ 
							"Old format gives no stamp; prior pointer in two parts"
							prevPos := tokens at: tokens size - 2.
							prevFileIndex := tokens last ].
					(prevPos = 0 or: [ prevFileIndex = 0 ])
						ifTrue: [ prevPos := nil ] ].
			((tokens size between: 5 and: 6) and: [ (tokens at: tokens size - 3) = #methodsFor: ])
				ifTrue: [ 
					(tokens at: tokens size - 1) = #stamp:
						ifTrue: [ 
							"New format gives change stamp and unified prior pointer"
							stamp := tokens at: tokens size ] ].
			methodCategory := tokens after: #methodsFor: ifAbsent: [ 'as yet unclassifed' ].
			methodCategory = category
				ifFalse: [ 
					methodCategory = (Smalltalk globals at: #Categorizer ifAbsent: [ Smalltalk globals at: #ClassOrganizer ]) default
						ifTrue: [ methodCategory := methodCategory , ' ' ].
					^ ChangeRecord new
						file: file
						position: position
						type: #method
						class: className
						category: methodCategory
						meta: classIsMeta
						stamp: stamp ].
			position := prevPos.
			prevPos notNil
				ifTrue: [ file := sourceFilesCopy at: prevFileIndex ] ].
	^ nil ]
		ensure: [ 
			sourceFilesCopy
				do: [ :x | 
					x notNil
						ifTrue: [ x close ] ] ]! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'ab 11/15/2002 01:11'!
selector
	^selector! !

!MCMethodDefinition methodsFor: 'comparing' stamp: 'ab 7/19/2003 18:01'!
sortKey
	^ self className, '.', (self classIsMeta ifTrue: ['meta'] ifFalse: ['nonmeta']), '.', self selector! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'ab 11/13/2002 01:59'!
source
	^ source! !

!MCMethodDefinition methodsFor: 'printing' stamp: 'BenjaminVanRyseghem 9/28/2011 15:26'!
summary

	^ selector! !

!MCMethodDefinition methodsFor: 'accessing' stamp: 'ab 1/15/2003 13:42'!
timeStamp
	^ timeStamp! !

!MCMethodDefinition methodsFor: 'installing' stamp: 'ar 12/12/2009 15:40'!
unload
	| previousVersion |
	self isOverrideMethod ifTrue: [previousVersion := self scanForPreviousVersion].
	previousVersion
		ifNil: [self actualClass ifNotNil:[:class| self removeSelector: selector fromClass: class]]
		ifNotNil: [previousVersion fileIn] ! !

MCDefinition subclass: #MCMockDefinition
	instanceVariableNames: 'token'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Mocks'!

!MCMockDefinition class methodsFor: 'as yet unclassified' stamp: 'ab 7/7/2003 23:21'!
token: aString

	^ self new token: aString! !

!MCMockDefinition class methodsFor: 'as yet unclassified' stamp: 'cwp 7/21/2003 19:46'!
wantsChangeSetLogging
	^ false! !

!MCMockDefinition methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 5/30/2012 17:12'!
= definition
	self == definition
		ifTrue: [ ^ true ].
	self species = definition species
		ifFalse: [ ^ false ].
	^definition token = token! !

!MCMockDefinition methodsFor: 'as yet unclassified' stamp: 'ab 7/7/2003 23:21'!
asString

	^ token! !

!MCMockDefinition methodsFor: 'as yet unclassified' stamp: 'ab 7/7/2003 23:21'!
description

	^ token first! !

!MCMockDefinition methodsFor: 'as yet unclassified' stamp: 'ab 7/7/2003 23:21'!
hash

	^ token hash! !

!MCMockDefinition methodsFor: 'as yet unclassified' stamp: 'ab 7/7/2003 23:21'!
printString

	^ token! !

!MCMockDefinition methodsFor: 'as yet unclassified' stamp: 'ab 7/7/2003 23:21'!
summary

	^ token! !

!MCMockDefinition methodsFor: 'as yet unclassified' stamp: 'ab 7/7/2003 23:21'!
token

	^ token! !

!MCMockDefinition methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
token: aString

	token := aString! !

MCDefinition subclass: #MCOrganizationDefinition
	instanceVariableNames: 'categories'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCOrganizationDefinition commentStamp: '<historical>' prior: 0!
A MCOrganizationDefinition represents a category change. !

!MCOrganizationDefinition class methodsFor: 'instance creation' stamp: 'StephaneDucasse 12/30/2012 09:59'!
categories: anArray
	^ self instanceLike: (self new categories: anArray)! !

!MCOrganizationDefinition methodsFor: 'equal/hash' stamp: 'SeanDeNigris 2/13/2013 08:41'!
= aDefinition

	^ (super = aDefinition)
		and: [ self categories size = aDefinition categories size and: [ self categories includesAllOf: aDefinition categories ] ].! !

!MCOrganizationDefinition methodsFor: 'accessing' stamp: 'ab 7/18/2003 21:47'!
accept: aVisitor
	^ aVisitor visitOrganizationDefinition: self! !

!MCOrganizationDefinition methodsFor: 'private' stamp: 'EstebanLorenzano 2/11/2013 14:38'!
basicCommonPrefix
	"Answers the minimum common denominator on package names contained in the monticello package. 
	 It can answer a package in the form X-Y-, with a minus at end..."
	| stream  |
	categories isEmpty ifTrue: [ ^ '' ].
	
	stream := String new writeStream.
	categories first withIndexDo: [:c :index |
		categories do: [:each |
			(each at: index ifAbsent: []) = c 
				ifFalse: [ ^ stream contents ] ].
		stream nextPut: c ].
	
	^stream contents! !

!MCOrganizationDefinition methodsFor: 'accessing' stamp: 'StephaneDucasse 12/30/2012 09:52'!
categories
	"ensure the categories are sorted alphabetically, so the merge don't take it as a conflict"
	^ categories! !

!MCOrganizationDefinition methodsFor: 'accessing' stamp: 'StephaneDucasse 12/30/2012 09:52'!
categories: anArray
	categories := anArray sort! !

!MCOrganizationDefinition methodsFor: 'accessing' stamp: 'EstebanLorenzano 2/11/2013 14:36'!
commonPrefix
	"Answer the package common name. 
	 In ancient times, this was answering X-Y-, with a minus at the end... this was useful for the 
	 old PackageInfo framework. RPackage does not do pattern matching, thus this is not needed anymore 
	 (and provoques an error)"
	| prefix |
	prefix := self basicCommonPrefix.
	^(prefix notEmpty and: [ prefix endsWith: '-' ])
		ifTrue: [ prefix allButLast ]
		ifFalse: [ prefix ].
	
! !

!MCOrganizationDefinition methodsFor: 'accessing' stamp: 'EstebanLorenzano 2/11/2013 14:30'!
description
	^ Array with: #organization with: self commonPrefix! !

!MCOrganizationDefinition methodsFor: 'testing' stamp: 'cwp 7/11/2003 01:33'!
isOrganizationDefinition
	^ true! !

!MCOrganizationDefinition methodsFor: 'installing' stamp: 'avi 2/22/2004 13:46'!
postloadOver: oldDefinition
	SystemOrganization categories:
		(self
			reorderCategories: SystemOrganization categories
			original: (oldDefinition ifNil: [#()] ifNotNil: [oldDefinition categories]))! !

!MCOrganizationDefinition methodsFor: 'private' stamp: 'stephaneducasse 2/4/2006 20:47'!
reorderCategories: allCategories original: oldCategories
	| first |
	first := allCategories detect: [:ea | categories includes: ea] ifNone: [^ allCategories].
	^ 	((allCategories copyUpTo: first) copyWithoutAll: oldCategories, categories),
		categories,
		((allCategories copyAfter: first) copyWithoutAll: oldCategories, categories)
! !

!MCOrganizationDefinition methodsFor: 'accessing' stamp: 'ab 7/19/2003 18:01'!
sortKey
	^ '<organization>'! !

!MCOrganizationDefinition methodsFor: 'accessing' stamp: 'ab 7/22/2003 01:14'!
source
	^ String streamContents:
		[:s |
		categories do: [:ea | s nextPutAll: ea] separatedBy: [s cr]]! !

!MCOrganizationDefinition methodsFor: 'accessing' stamp: 'ab 5/24/2003 13:55'!
summary
	^ categories asArray printString! !

!MCOrganizationDefinition methodsFor: 'unloading' stamp: 'ChristopheDemarey 6/17/2013 17:22'!
unload
  categories do: [ :category | 
    (SystemOrganization isEmptyCategoryNamed: category) 
      ifTrue: [ SystemOrganization removeCategory: category ] ]
! !

MCDefinition subclass: #MCScriptDefinition
	instanceVariableNames: 'script packageName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

MCScriptDefinition subclass: #MCPostscriptDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCPostscriptDefinition class methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 17:04'!
scriptSelector
	^ #postscript! !

!MCPostscriptDefinition methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 17:16'!
postload
	self evaluate! !

!MCPostscriptDefinition methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 17:16'!
sortKey
	^ 'zzz' "force to the end so it gets loaded late"! !

MCScriptDefinition subclass: #MCPreambleDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCPreambleDefinition class methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 17:04'!
scriptSelector
	^ #preamble! !

!MCPreambleDefinition methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 17:15'!
load
	super load.
	self evaluate! !

MCScriptDefinition subclass: #MCRemovalPostscriptDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCRemovalPostscriptDefinition class methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 17:04'!
scriptSelector
	^ #postscriptOfRemoval ! !

!MCRemovalPostscriptDefinition methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 17:15'!
unload
	super unload.
	self evaluate! !

MCScriptDefinition subclass: #MCRemovalPreambleDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCRemovalPreambleDefinition class methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 17:04'!
scriptSelector
	^ #preambleOfRemoval! !

!MCRemovalPreambleDefinition methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 17:14'!
sortKey
	^ 'zzz' "force to the end so it gets unloaded early"! !

!MCRemovalPreambleDefinition methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 17:15'!
unload
	super unload.
	self evaluate! !

!MCScriptDefinition class methodsFor: 'as yet unclassified' stamp: 'bf 4/4/2005 12:20'!
from: aPackageInfo
	^ self script: (aPackageInfo perform: self scriptSelector) contents asString packageName: aPackageInfo name! !

!MCScriptDefinition class methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 16:59'!
script: aString packageName: packageString
	^ self instanceLike: (self new initializeWithScript: aString packageName: packageString)! !

!MCScriptDefinition class methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2005 17:00'!
scriptSelector
	self subclassResponsibility! !

!MCScriptDefinition class methodsFor: 'as yet unclassified' stamp: 'bf 8/13/2009 00:24'!
scriptSelector: selectorString script: aString packageName: packageString
	^ (self subclassForScriptSelector: selectorString)
		script: aString packageName: packageString! !

!MCScriptDefinition class methodsFor: 'as yet unclassified' stamp: 'bf 8/13/2009 00:25'!
subclassForScriptSelector: selectorString
	^self allSubclasses detect: [:ea | ea scriptSelector = selectorString]! !

!MCScriptDefinition methodsFor: 'comparing' stamp: 'avi 2/28/2005 16:55'!
= aDefinition
	^ (super = aDefinition)
		and: [script = aDefinition script]! !

!MCScriptDefinition methodsFor: 'visiting' stamp: 'bf 8/12/2009 21:41'!
accept: aVisitor
	aVisitor visitScriptDefinition: self! !

!MCScriptDefinition methodsFor: 'accessing' stamp: 'avi 2/28/2005 17:12'!
description
	^ Array with: packageName with: self scriptSelector! !

!MCScriptDefinition methodsFor: 'installing' stamp: 'jb 7/1/2011 10:52'!
evaluate
	self class evaluatorClass evaluate: script! !

!MCScriptDefinition methodsFor: 'initializing' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithScript: aString packageName: packageString
	script := aString.
	packageName := packageString! !

!MCScriptDefinition methodsFor: 'installing' stamp: 'avi 2/28/2005 17:11'!
installScript
	self installScript: script! !

!MCScriptDefinition methodsFor: 'installing' stamp: 'EstebanLorenzano 9/12/2012 13:35'!
installScript: aString
	| sel pi |
	sel := (self scriptSelector, ':') asSymbol.
	pi := self packageSet.
	(pi respondsTo: sel)
		ifTrue: [pi perform: sel with: aString]! !

!MCScriptDefinition methodsFor: 'testing' stamp: 'bf 8/12/2009 22:55'!
isScriptDefinition
	^true! !

!MCScriptDefinition methodsFor: 'installing' stamp: 'avi 2/28/2005 17:12'!
load
	self installScript! !

!MCScriptDefinition methodsFor: 'accessing' stamp: 'EstebanLorenzano 9/14/2012 11:32'!
packageInfo
	self deprecated: 'Use #packageSet' on: '14 September 2012'  in: '2.0'.
	^ PackageInfo named: packageName! !

!MCScriptDefinition methodsFor: 'accessing' stamp: 'bf 8/13/2009 00:09'!
packageName
	^ packageName! !

!MCScriptDefinition methodsFor: 'accessing' stamp: 'EstebanLorenzano 9/11/2012 17:01'!
packageSet
	^ RPackageSet named: packageName! !

!MCScriptDefinition methodsFor: 'accessing' stamp: 'avi 2/28/2005 16:54'!
script
	^ script! !

!MCScriptDefinition methodsFor: 'accessing' stamp: 'avi 2/28/2005 17:10'!
scriptSelector
	^ self class scriptSelector! !

!MCScriptDefinition methodsFor: 'accessing' stamp: 'bf 10/25/2005 19:05'!
sortKey
	^ '!!', self scriptSelector "force to the front so it gets loaded first"! !

!MCScriptDefinition methodsFor: 'accessing' stamp: 'avi 2/28/2005 17:04'!
source
	^ script! !

!MCScriptDefinition methodsFor: 'accessing' stamp: 'avi 2/28/2005 17:12'!
summary
	^ packageName, ' ', self scriptSelector! !

!MCScriptDefinition methodsFor: 'installing' stamp: 'avi 2/28/2005 17:12'!
unload
	self installScript: nil! !

Object subclass: #MCDefinitionIndex
	instanceVariableNames: 'definitions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Patching'!

!MCDefinitionIndex commentStamp: 'LaurentLaffont 3/31/2011 21:06' prior: 0!
I'm a simple container of MCDefinitions which can be added or removed.!

!MCDefinitionIndex class methodsFor: 'as yet unclassified' stamp: 'ab 6/2/2003 01:29'!
definitions: aCollection
	^ self new addAll: aCollection! !

!MCDefinitionIndex methodsFor: 'as yet unclassified' stamp: 'StephaneDucasse 5/6/2010 08:52'!
add: aDefinition
	^ definitions at: aDefinition description put: aDefinition! !

!MCDefinitionIndex methodsFor: 'as yet unclassified' stamp: 'ab 6/2/2003 00:38'!
addAll: aCollection
	aCollection do: [:ea | self add: ea]! !

!MCDefinitionIndex methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
definitionLike: aDefinition ifPresent: foundBlock ifAbsent: errorBlock
	| definition |
	definition := definitions at: aDefinition description ifAbsent: [].
	^ definition
		ifNil: errorBlock
		ifNotNil: [foundBlock value: definition]! !

!MCDefinitionIndex methodsFor: 'as yet unclassified' stamp: 'ab 6/2/2003 00:42'!
definitions
	^ definitions values! !

!MCDefinitionIndex methodsFor: 'as yet unclassified' stamp: 'alain.plantec 5/28/2009 10:02'!
initialize
	super initialize.
	definitions := Dictionary new! !

!MCDefinitionIndex methodsFor: 'as yet unclassified' stamp: 'ab 6/2/2003 00:40'!
remove: aDefinition
	definitions removeKey: aDefinition description ifAbsent: []! !

Object subclass: #MCDependencySorter
	instanceVariableNames: 'required provided orderedItems'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Loading'!

!MCDependencySorter commentStamp: '<historical>' prior: 0!
A MCDependencySorter computes the dependencies to a set of entities.!

!MCDependencySorter class methodsFor: 'as yet unclassified' stamp: 'ab 5/23/2003 14:17'!
items: aCollection
	^ self new addAll: aCollection! !

!MCDependencySorter class methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
sortItems: aCollection
	| sorter |
	sorter := self items: aCollection.
	sorter externalRequirements do: [:req  | sorter addProvision: req].
	^ sorter orderedItems.! !

!MCDependencySorter methodsFor: 'building' stamp: 'StephaneDucasse 5/6/2010 08:52'!
add: anItem
	| requirements |
	requirements := self unresolvedRequirementsFor: anItem.
	requirements isEmpty
		ifTrue: [self addToOrder: anItem]
		ifFalse: [self addRequirements: requirements for: anItem].
	^ anItem! !

!MCDependencySorter methodsFor: 'building' stamp: 'bf 11/12/2004 14:50'!
addAll: aCollection
	aCollection asArray sort do: [:ea | self add: ea]! !

!MCDependencySorter methodsFor: 'building' stamp: 'avi 10/7/2004 22:47'!
addExternalProvisions: aCollection
	(aCollection intersection: self externalRequirements)
		do: [:ea | self addProvision: ea]! !

!MCDependencySorter methodsFor: 'private' stamp: 'stephaneducasse 2/4/2006 20:47'!
addProvision: anObject
	| newlySatisfied |
	provided add: anObject.
	newlySatisfied := required removeKey: anObject ifAbsent: [#()].
	self addAll: newlySatisfied.! !

!MCDependencySorter methodsFor: 'private' stamp: 'ab 5/22/2003 23:24'!
addRequirement: reqObject for: itemObject
	(self itemsRequiring: reqObject) add: itemObject! !

!MCDependencySorter methodsFor: 'private' stamp: 'ab 5/22/2003 23:23'!
addRequirements: aCollection for: anObject
	aCollection do: [:ea | self addRequirement: ea for: anObject]! !

!MCDependencySorter methodsFor: 'private' stamp: 'ab 5/22/2003 23:15'!
addToOrder: anItem
	orderedItems add: anItem.
	anItem provisions do: [:ea | self addProvision: ea].! !

!MCDependencySorter methodsFor: 'accessing' stamp: 'dvf 9/8/2004 00:49'!
externalRequirements
	| unloaded providedByUnloaded |
	unloaded := self itemsWithMissingRequirements.
	providedByUnloaded := (unloaded gather: [:e | e provisions]) asSet.
	^ required keys reject: [:ea | providedByUnloaded includes: ea ]! !

!MCDependencySorter methodsFor: 'initialization' stamp: 'alain.plantec 5/28/2009 10:02'!
initialize
	super initialize.
	provided := Set new.
	required := Dictionary new.
	orderedItems := OrderedCollection new.! !

!MCDependencySorter methodsFor: 'private' stamp: 'ab 5/22/2003 23:24'!
itemsRequiring: anObject
	^ required at: anObject ifAbsentPut: [Set new]! !

!MCDependencySorter methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
itemsWithMissingRequirements
	| items |
	items := Set new.
	required do: [:ea | items addAll: ea].
	^ items
! !

!MCDependencySorter methodsFor: 'sorting' stamp: 'ab 5/22/2003 23:25'!
orderedItems
	^ orderedItems! !

!MCDependencySorter methodsFor: 'private' stamp: 'ab 5/22/2003 23:22'!
unresolvedRequirementsFor: anItem
	^ anItem requirements difference: provided! !

Object subclass: #MCDoItParser
	instanceVariableNames: 'source'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Chunk Format'!

!MCDoItParser commentStamp: '<historical>' prior: 0!
A MCDoItParser is a simple 'parser' which understand the addDefinitionsTo: message. Each parser can use the source
and add definitions to the list of entities that is passed to them.

MCDoitParser invokes automatically its subclasses to parse the correct source.

Each Doit entities (entities which are defined as doits) extend this entry point to add specific behavior.!

MCDoItParser subclass: #MCClassTraitParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Chunk Format'!

!MCClassTraitParser commentStamp: '<historical>' prior: 0!
MCClassTraitParser identifies classTrait. !

!MCClassTraitParser class methodsFor: 'factory identification hook' stamp: 'al 10/9/2005 20:43'!
pattern
	^ '*classTrait*uses:*'! !

!MCClassTraitParser methodsFor: 'actions' stamp: 'SvenVanCaekenberghe 1/8/2012 14:45'!
addDefinitionsTo: aCollection
	| tokens  definition traitCompositionString |
	tokens := Scanner new scanTokens: source.
	traitCompositionString := (source readStream
		match: 'uses:';
		upToEnd) trimBoth.
	definition := MCClassTraitDefinition
		baseTraitName: (tokens at: 1) 
		classTraitComposition: traitCompositionString.
	aCollection add: definition
! !

!MCDoItParser class methodsFor: 'private' stamp: 'avi 3/10/2004 12:29'!
concreteSubclasses
	^ self allSubclasses reject: [:c | c isAbstract]! !

!MCDoItParser class methodsFor: 'instance creation' stamp: 'marcus.denker 11/10/2008 10:04'!
forDoit: aString
	^ (self subclassForDoit: aString) ifNotNil: [:c | c new source: aString]! !

!MCDoItParser class methodsFor: 'testing' stamp: 'avi 3/10/2004 12:51'!
isAbstract
	^ self pattern isNil! !

!MCDoItParser class methodsFor: 'factory identification hook' stamp: 'StephaneDucasse 12/30/2012 18:01'!
pattern
	"return a pattern matching expression to identify the parser corresponding to the expression.
	Subclasses should specialize it."

	^ nil! !

!MCDoItParser class methodsFor: 'instance creation' stamp: 'avi 3/10/2004 12:30'!
subclassForDoit: aString
	^ self concreteSubclasses detect: [:ea | ea pattern match: aString] ifNone: []! !

!MCDoItParser methodsFor: 'actions' stamp: 'avi 3/10/2004 12:40'!
addDefinitionsTo: aCollection
	self subclassResponsibility ! !

!MCDoItParser methodsFor: 'accessing' stamp: 'avi 3/10/2004 12:40'!
source
	^ source! !

!MCDoItParser methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
source: aString
	source := aString! !

MCDoItParser subclass: #MCScriptParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Chunk Format'!

!MCScriptParser commentStamp: '<historical>' prior: 0!
A MCScriptParser identifies script and add MCScriptDefinition. !

!MCScriptParser class methodsFor: 'factory identification hook' stamp: 'bf 8/13/2009 00:07'!
pattern
	^'(PackageInfo named: *'! !

!MCScriptParser methodsFor: 'actions' stamp: 'bf 8/13/2009 00:13'!
addDefinitionsTo: aCollection
	| tokens  definition |
	tokens := Scanner new scanTokens: source.
	definition := MCScriptDefinition
		scriptSelector: tokens second allButLast
		script: tokens third
		packageName: tokens first third.
	aCollection add: definition.! !

MCDoItParser subclass: #MCSystemCategoryParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Chunk Format'!

!MCSystemCategoryParser commentStamp: '<historical>' prior: 0!
A MCSystemCategoryParser extracts or set a category/protocol to the corresponding MCOrganizationDefinition.!

!MCSystemCategoryParser class methodsFor: 'factory identification hook' stamp: 'avi 3/10/2004 12:41'!
pattern
	^ 'SystemOrganization*'! !

!MCSystemCategoryParser methodsFor: 'actions' stamp: 'StephaneDucasse 12/30/2012 17:34'!
addDefinitionsTo: aCollection
	| definition |
	definition := aCollection 
					detect: [:ea | ea isOrganizationDefinition ] 
					ifNone: [aCollection add: (MCOrganizationDefinition categories: #())].
	definition categories: (definition categories copyWith: self category).! !

!MCSystemCategoryParser methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
category
	| tokens  |
	tokens := Scanner new scanTokens: source.
	tokens size = 3 ifFalse: [self error: 'Unrecognized category definition'].
	^ tokens at: 3! !

MCDoItParser subclass: #MCTraitParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Chunk Format'!

!MCTraitParser commentStamp: '<historical>' prior: 0!
A MCTraitParser extracts an MCTraitDefinition from the source.!

!MCTraitParser class methodsFor: 'factory identification hook' stamp: 'al 10/9/2005 21:09'!
pattern
	^ 'Trait named:*'! !

!MCTraitParser methodsFor: 'actions' stamp: 'SvenVanCaekenberghe 1/8/2012 14:45'!
addDefinitionsTo: aCollection
	| tokens  definition traitCompositionString |
	tokens := Scanner new scanTokens: source.
	traitCompositionString := (source readStream
		match: 'uses:';
		upToAll: 'category:') trimBoth.
	definition := MCTraitDefinition
		name: (tokens at: 3) 
		traitComposition: traitCompositionString
		category:  tokens last
		comment:  ''  
		commentStamp:   ''.
	aCollection add: definition.! !

Object subclass: #MCFrontier
	instanceVariableNames: 'frontier bag'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCFrontier class methodsFor: 'instance creation' stamp: 'avi 9/17/2005 22:07'!
frontierOn: aVersionInfo
	^ self frontierOnAll: (Array with: aVersionInfo)! !

!MCFrontier class methodsFor: 'instance creation' stamp: 'avi 9/17/2005 22:07'!
frontierOn: aVersionInfo and: otherVersionInfo
	^ self frontierOnAll: (Array with: aVersionInfo with: otherVersionInfo)! !

!MCFrontier class methodsFor: 'instance creation' stamp: 'stephaneducasse 2/4/2006 20:47'!
frontierOnAll: aCollection
	| remaining  allVersions |
	remaining := Bag new.
	allVersions := (aCollection gather: [:ea | ea withBreadthFirstAncestors]) asSet.
	allVersions do: [:ea | remaining addAll: ea ancestors].
	^self new frontier: aCollection bag: remaining! !

!MCFrontier methodsFor: 'accessing' stamp: 'avi 9/17/2005 22:02'!
frontier
	^frontier! !

!MCFrontier methodsFor: 'initialization' stamp: 'avi 9/17/2005 22:11'!
frontier: f bag: remaining
	frontier := f asOrderedCollection.
	bag := remaining! !

!MCFrontier methodsFor: 'advancing' stamp: 'avi 9/17/2005 22:13'!
remove: aVersionInfo
	frontier remove: aVersionInfo.
	aVersionInfo ancestors  do:
		[ :ancestor |
			bag remove: ancestor.
			(bag occurrencesOf: ancestor) = 0
				ifTrue: [frontier add: ancestor]].
	^aVersionInfo! !

!MCFrontier methodsFor: 'advancing' stamp: 'avi 9/17/2005 22:02'!
removeAll: collection
	collection do: [ :n | self remove: n]! !

Object subclass: #MCMergeRecord
	instanceVariableNames: 'version packageSnapshot ancestorInfo ancestor ancestorSnapshot imagePatch mergePatch'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCMergeRecord class methodsFor: 'as yet unclassified' stamp: 'abc 2/13/2004 15:52'!
version: aVersion
	^ self basicNew initializeWithVersion: aVersion! !

!MCMergeRecord methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
ancestorInfo
	^ ancestorInfo ifNil: [ancestorInfo := version info commonAncestorWith: version workingCopy ancestry]! !

!MCMergeRecord methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
ancestorSnapshot
	^ ancestorSnapshot ifNil: [ancestorSnapshot := version workingCopy findSnapshotWithVersionInfo: self ancestorInfo]! !

!MCMergeRecord methodsFor: 'as yet unclassified' stamp: 'StephaneDucasse 4/27/2010 11:49'!
imageIsClean
	| ancestors |
	ancestors := version workingCopy ancestors.
	^ ancestors size = 1
		and: [(ancestors first = self ancestorInfo)
				and: [self imagePatch isEmpty]]! !

!MCMergeRecord methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
imagePatch
	^ imagePatch ifNil: [imagePatch := self packageSnapshot patchRelativeToBase: self ancestorSnapshot]! !

!MCMergeRecord methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithVersion: aVersion
	version := aVersion! !

!MCMergeRecord methodsFor: 'as yet unclassified' stamp: 'abc 2/13/2004 17:14'!
isAncestorMerge
	^ version workingCopy ancestry hasAncestor: version info! !

!MCMergeRecord methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
mergePatch
	^ mergePatch ifNil: [mergePatch := version snapshot patchRelativeToBase: self ancestorSnapshot]! !

!MCMergeRecord methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
packageSnapshot
	^ packageSnapshot ifNil: [packageSnapshot := version package snapshot]! !

!MCMergeRecord methodsFor: 'as yet unclassified' stamp: 'abc 2/13/2004 17:14'!
updateWorkingCopy
	self isAncestorMerge ifFalse:
		[self imageIsClean
			ifTrue: [version workingCopy loaded: version]
			ifFalse: [version workingCopy merged: version]]! !

!MCMergeRecord methodsFor: 'as yet unclassified' stamp: 'abc 2/13/2004 15:52'!
version
	^ version! !

Object subclass: #MCMerger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Merging'!

!MCMerger commentStamp: '<historical>' prior: 0!
A MCMerger is an abstract responsible for performing merge operations and detecting conflicts.!

!MCMerger methodsFor: 'operations' stamp: 'StephaneDucasse 6/24/2011 15:18'!
addConflictWithOperation: anOperation
	self operations add: anOperation beConflict! !

!MCMerger methodsFor: 'operations' stamp: 'StephaneDucasse 6/24/2011 15:05'!
applyTo: anObject
	self isMerged ifFalse: [self error: 'You must resolve all the conflicts first'].
	self operations do: [:ea | ea applyTo: anObject]! !

!MCMerger methodsFor: 'accessing' stamp: 'StephaneDucasse 6/24/2011 15:14'!
conflicts	
	^ self operations select: #isConflict
! !

!MCMerger methodsFor: 'testing' stamp: 'ab 6/5/2003 19:09'!
isMerged
	^ self conflicts allSatisfy: [:ea | ea isResolved]! !

!MCMerger methodsFor: 'operations' stamp: 'stephaneducasse 2/4/2006 20:47'!
load
	| loader |
	loader := MCPackageLoader new.
	loader provisions addAll: self provisions.
	self applyTo: loader.
	loader load! !

!MCMerger methodsFor: 'operations' stamp: 'stephaneducasse 2/4/2006 20:47'!
loadWithNameLike: baseName
	| loader |
	loader := MCPackageLoader new.
	loader provisions addAll: self provisions.
	self applyTo: loader.
	loader loadWithNameLike: baseName! !

!MCMerger methodsFor: 'accessing' stamp: 'ab 6/2/2003 01:11'!
mergedSnapshot
	^ MCPatcher apply: self to: self baseSnapshot! !

!MCMerger methodsFor: 'accessing' stamp: 'ab 7/18/2003 16:34'!
operations
	^ #()! !

!MCMerger methodsFor: 'accessing' stamp: 'avi 10/6/2004 15:19'!
provisions
	^ #()! !

MCMerger subclass: #MCThreeWayMerger
	instanceVariableNames: 'index operations provisions redundantAdds'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Merging'!

!MCThreeWayMerger commentStamp: '<historical>' prior: 0!
A MCThreeWayMerger is more advanced merge operation.
Operations are subclasses of MCPatchOperation:  addition (MCAddition), modification of an entity (MCModification ) and removal (MCRemoval).
!

!MCThreeWayMerger class methodsFor: 'as yet unclassified' stamp: 'avi 2/13/2004 01:53'!
base: aSnapshot patch: aPatch
	aPatch isEmpty ifTrue: [MCNoChangesException signal].
	^ self new
		addBaseSnapshot: aSnapshot;
		applyPatch: aPatch;
		yourself
		! !

!MCThreeWayMerger class methodsFor: 'as yet unclassified' stamp: 'ab 6/2/2003 01:09'!
base: aSnapshot target: targetSnapshot ancestor: ancestorSnapshot
	^ self base: aSnapshot patch: (targetSnapshot patchRelativeToBase: ancestorSnapshot)! !

!MCThreeWayMerger methodsFor: 'operations' stamp: 'avi 10/6/2004 15:18'!
addBaseSnapshot: aSnapshot
	aSnapshot definitions do:
		[:ea |
		index add: ea.
		provisions addAll: ea provisions]! !

!MCThreeWayMerger methodsFor: 'operations' stamp: 'marcus.denker 11/10/2008 10:04'!
addDefinition: aDefinition
	index
		definitionLike: aDefinition
		ifPresent: [:other |
			(self removalForDefinition: aDefinition)
				ifNotNil:
					[:op |
					self addOperation: (MCModification of: other to: aDefinition).
					self removeOperation: op.
					^ self].
			other = aDefinition
				ifFalse: [self addConflictWithOperation: (MCModification of: other to: aDefinition)]
				ifTrue: [self redundantAdds add: aDefinition]]
		ifAbsent: [self addOperation: (MCAddition of: aDefinition)]! !

!MCThreeWayMerger methodsFor: 'operations' stamp: 'ab 6/2/2003 01:30'!
addOperation: anOperation
	self operations add: anOperation! !

!MCThreeWayMerger methodsFor: 'operations' stamp: 'avi 2/13/2004 01:52'!
applyPatch: aPatch
	aPatch applyTo: self! !

!MCThreeWayMerger methodsFor: 'operations' stamp: 'avi 2/13/2004 01:49'!
baseSnapshot
	^ (MCSnapshot fromDefinitions: index definitions)! !

!MCThreeWayMerger methodsFor: 'initialize-release' stamp: 'alain.plantec 5/28/2009 10:04'!
initialize
	super initialize.
	index := MCDefinitionIndex new.
	provisions := Set new! !

!MCThreeWayMerger methodsFor: 'operations' stamp: 'StephaneDucasse 6/24/2011 15:10'!
modificationConflictForDefinition: aDefinition
	^ self conflicts 
		detect: [:ea |
			(ea definition isRevisionOf: aDefinition) and:
				[ea operation isModification]] 
		ifNone: []! !

!MCThreeWayMerger methodsFor: 'operations' stamp: 'StephaneDucasse 6/24/2011 14:40'!
modifyDefinition: baseDefinition to: targetDefinition
	index
		definitionLike: baseDefinition
		ifPresent: [:other | 
			other = baseDefinition
				ifTrue: [self addOperation: (MCModification of:  baseDefinition to: targetDefinition)]
				ifFalse: [other = targetDefinition
							ifFalse: [self addConflictWithOperation:
											(MCModification of: other to: targetDefinition)]]]
		ifAbsent: [self addConflictWithOperation: (MCAddition of: targetDefinition)]! !

!MCThreeWayMerger methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
operations
	^ operations ifNil: [operations := OrderedCollection new]! !

!MCThreeWayMerger methodsFor: 'accessing' stamp: 'avi 10/6/2004 15:19'!
provisions
	^ provisions! !

!MCThreeWayMerger methodsFor: 'operations' stamp: 'stephaneducasse 2/4/2006 20:47'!
redundantAdds
	^ redundantAdds ifNil: [redundantAdds := Set new]! !

!MCThreeWayMerger methodsFor: 'operations' stamp: 'avi 9/19/2005 02:40'!
removalForDefinition: aDefinition
	^ operations ifNotNil:
		[operations
			detect: [:ea | (ea definition isRevisionOf: aDefinition) and: [ea isRemoval]]
			ifNone: []]! !

!MCThreeWayMerger methodsFor: 'operations' stamp: 'StephaneDucasse 6/24/2011 15:12'!
removeDefinition: aDefinition
	index
		definitionLike: aDefinition
		ifPresent: [:other | 
			other = aDefinition
				ifTrue: [(self modificationConflictForDefinition: aDefinition)
							ifNotNil: [:c |
									c beNonConflict.
									^ self]. 
						(self redundantAdds includes: aDefinition)
							ifFalse: [self addOperation: (MCRemoval of: aDefinition)]]
				ifFalse: [self addConflictWithOperation: (MCRemoval of: other)]]
		ifAbsent: []! !

!MCThreeWayMerger methodsFor: 'operations' stamp: 'avi 9/19/2005 02:40'!
removeOperation: anOperation
	operations remove: anOperation! !

Object subclass: #MCMock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Mocks'!

!MCMock class methodsFor: 'as yet unclassified' stamp: 'cwp 7/21/2003 19:40'!
wantsChangeSetLogging
	^ false! !

MCMock subclass: #MCMockDependentItem
	instanceVariableNames: 'name provides requires'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Mocks'!

!MCMockDependentItem methodsFor: 'as yet unclassified' stamp: 'bf 5/20/2005 16:15'!
<= other
	^ self name <= other name! !

!MCMockDependentItem methodsFor: 'as yet unclassified' stamp: 'ab 7/7/2003 23:21'!
name

	^ name! !

!MCMockDependentItem methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
name: aString

	name := aString! !

!MCMockDependentItem methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
provides: anArray

	provides := anArray! !

!MCMockDependentItem methodsFor: 'as yet unclassified' stamp: 'ab 7/7/2003 23:21'!
provisions

	^ provides ifNil: [#()]! !

!MCMockDependentItem methodsFor: 'as yet unclassified' stamp: 'ab 7/7/2003 23:21'!
requirements

	^ requires ifNil: [#()]! !

!MCMockDependentItem methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
requires: anArray

	requires := anArray! !

Object subclass: #MCPackage
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Base'!

!MCPackage commentStamp: '<historical>' prior: 0!
MCPackage represents a package. It is merely a wrapper on top of a package set or packageInfo.

Strangely enough it does not inherit from MCDefinition.

Its most important method is snapshot which returns a snapshot with all the entities that should be saved. !

!MCPackage class methodsFor: 'as yet unclassified' stamp: 'ab 7/10/2003 01:17'!
named: aString
	^ self new name: aString! !

!MCPackage class methodsFor: 'renaming' stamp: ''!
renamePackage: oldPackageName to: newPackageName
	"self renamePackage: 'Seaside-Squeak-Development-Core' to: 'Seaside-Pharo-Development-Core'"
	
	| oldPackage newPackage oldWorkingCopy newWorkingCopy |
	oldPackage := PackageInfo named: oldPackageName.
	newPackage := PackageInfo named: newPackageName.
	
	" rename system categories "
	oldPackage systemCategories do: [ :oldCategory |
		| newCategory |
		newCategory := oldCategory allButFirst: oldPackage systemCategoryPrefix size.
		Smalltalk organization renameCategory: oldCategory toBe: newPackage systemCategoryPrefix , newCategory ].
	
	" rename method categories "
	oldPackage extensionClasses do: [ :extensionClass |
		(oldPackage extensionCategoriesForClass: extensionClass) do: [ :oldProtocol |
			| newProtocol |
			newProtocol := oldProtocol allButFirst: oldPackage methodCategoryPrefix size.
			extensionClass organization renameCategory: oldProtocol toBe: newPackage methodCategoryPrefix , newProtocol ] ].
	
	" update monticello packages "
	oldWorkingCopy := MCWorkingCopy forPackage: (MCPackage named: oldPackageName).
	newWorkingCopy := MCWorkingCopy forPackage: (MCPackage named: newPackageName).
	newWorkingCopy repositoryGroup: oldWorkingCopy repositoryGroup; modified: true.
	
	" test is all methos have been caught "
	oldPackage methods isEmpty
		ifTrue: [ oldWorkingCopy unload. PackageOrganizer default unregisterPackage: oldPackage ]
		ifFalse: [ self error: 'Some code entities remain in the old package, please migrate manually.' ]! !

!MCPackage methodsFor: 'comparing' stamp: 'ar 4/26/2005 21:57'!
= other
	^ other species = self species and: [other name sameAs: name]! !

!MCPackage methodsFor: 'working copies' stamp: 'bf 4/19/2005 16:26'!
hasWorkingCopy
	^ MCWorkingCopy registry includesKey: self! !

!MCPackage methodsFor: 'comparing' stamp: 'ar 4/26/2005 21:57'!
hash
	^ name asLowercase hash! !

!MCPackage methodsFor: 'accessing' stamp: 'ab 7/7/2003 00:57'!
name
	^ name! !

!MCPackage methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
name: aString
	name := aString! !

!MCPackage methodsFor: 'accessing' stamp: 'EstebanLorenzano 9/14/2012 11:32'!
packageInfo
	self deprecated: 'Use #packageSet' on: '14 September 2012'  in: '2.0'.
	^ PackageInfo named: name! !

!MCPackage methodsFor: 'accessing' stamp: 'EstebanLorenzano 9/11/2012 17:01'!
packageSet
	^ RPackageSet named: name! !

!MCPackage methodsFor: 'printing' stamp: 'nk 7/28/2003 13:30'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		nextPutAll: name;
		nextPut: $)! !

!MCPackage methodsFor: 'accessing' stamp: 'GuillermoPolito 2/8/2013 13:02'!
snapshot
	| packageInfo definitions categories |
	packageInfo := self packageSet.
	definitions := OrderedCollection new.
	categories := (packageInfo packages collect: #packageName) asArray.
	categories isEmpty 
		ifFalse: [ definitions add: (MCOrganizationDefinition categories: categories) ].
		
	packageInfo methods 
		do: [:ea | definitions add: ea asMCMethodDefinition] 
		displayingProgress: [ :ea| 'Snapshotting methods...' ].
		
	packageInfo overriddenMethods
		do: [:ea | definitions add:
					(packageInfo changeRecordForOverriddenMethod: ea) asMCMethodDefinition]
		displayingProgress: [ :ea| 'Searching for overrides in ', ea asString ].
		
	packageInfo definedClasses 
		do: [:ea | definitions addAll: ea classDefinitions] 
		displayingProgress: [ :ea| 'Snapshotting class ', ea asString ].
		
	(packageInfo respondsTo: #hasPreamble) ifTrue: [
		packageInfo hasPreamble 
			ifTrue: [definitions add: (MCPreambleDefinition from: packageInfo)].
		packageInfo hasPostscript 
			ifTrue: [definitions add: (MCPostscriptDefinition from: packageInfo)].
		packageInfo hasPreambleOfRemoval 
			ifTrue: [definitions add: (MCRemovalPreambleDefinition from: packageInfo)].
		packageInfo hasPostscriptOfRemoval 
			ifTrue: [definitions add: (MCRemovalPostscriptDefinition from: packageInfo)]]. 
	^ MCSnapshot fromDefinitions: definitions
! !

!MCPackage methodsFor: 'printing' stamp: 'ab 7/10/2003 01:13'!
storeOn: aStream
	aStream
		nextPutAll: 'MCPackage';
		space; nextPutAll: 'named: '; store: name.! !

!MCPackage methodsFor: 'working copies' stamp: 'cwp 11/13/2003 13:32'!
unload
	^ self workingCopy unload! !

!MCPackage methodsFor: 'working copies' stamp: 'cwp 11/13/2003 13:33'!
workingCopy
	^ MCWorkingCopy forPackage: self.! !

Object subclass: #MCPackageCache
	instanceVariableNames: 'sorter fileNames'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCPackageCache commentStamp: 'LaurentLaffont 3/31/2011 21:06' prior: 0!
I'm a kind of cache for versions and filenames of packages.!

!MCPackageCache methodsFor: 'as yet unclassified' stamp: 'alain.plantec 5/28/2009 10:03'!
initialize
	super initialize.
	sorter := MCVersionSorter new.
	fileNames := Dictionary new.! !

!MCPackageCache methodsFor: 'as yet unclassified' stamp: 'AdrianLienhard 1/21/2010 22:14'!
recordVersionInfo: aVersionInfo forFileNamed: aString
	fileNames at: aVersionInfo put: aString.
	sorter addVersionInfo: aVersionInfo! !

!MCPackageCache methodsFor: 'as yet unclassified' stamp: 'avi 1/22/2004 18:21'!
versionInfos
	^ sorter sortedVersionInfos ! !

Object subclass: #MCPackageLoader
	instanceVariableNames: 'requirements unloadableDefinitions obsoletions additions removals errorDefinitions provisions methodAdditions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Loading'!

!MCPackageLoader commentStamp: '<historical>' prior: 0!
A MCPackageLoader is responsible for loading packages.  It gets used by VersionLoader, so it is eventually responsible for loading everything.

Instance Variables
	additions:		<Definitions>  Definitions that need to be added
	errorDefinitions:		<Object>
	obsoletions:		<Object>
	provisions:		<Object>
	removals:		<Object>
	requirements:		<Object>
	unloadableDefinitions:		<Object>
	methodAdditions  <MethodAdditions> MethodDefinitions corresponding to the Definitions in "additions" that have been added so far.
!

MCPackageLoader subclass: #MCMultiPackageLoader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Loading'!

!MCMultiPackageLoader commentStamp: '<historical>' prior: 0!
A PackageLoader doing some additional cross-package checks!

!MCMultiPackageLoader methodsFor: 'private' stamp: 'bf 3/17/2006 15:51'!
analyze
	| index |
	index := MCDefinitionIndex definitions: additions.
	removals removeAllSuchThat: [:removal |
		(index definitionLike: removal
			ifPresent: [:addition | obsoletions at: addition put: removal]
			ifAbsent: []) notNil].
	super analyze! !

!MCPackageLoader class methodsFor: 'as yet unclassified' stamp: 'ab 7/6/2003 23:30'!
installSnapshot: aSnapshot
	self new
		installSnapshot: aSnapshot;
		load! !

!MCPackageLoader class methodsFor: 'as yet unclassified' stamp: 'bf 12/5/2004 12:00'!
unloadPackage: aPackage
	self new
		unloadPackage: aPackage;
		loadWithNameLike: aPackage name, '-unload'! !

!MCPackageLoader class methodsFor: 'as yet unclassified' stamp: 'ab 7/7/2003 12:11'!
updatePackage: aPackage withSnapshot: aSnapshot
	self new
		updatePackage: aPackage withSnapshot: aSnapshot;
		load! !

!MCPackageLoader methodsFor: 'patch ops' stamp: 'ab 5/24/2003 16:13'!
addDefinition: aDefinition
	additions add: aDefinition! !

!MCPackageLoader methodsFor: 'private' stamp: 'stephaneducasse 2/4/2006 20:47'!
analyze
	| sorter |
	sorter := self sorterForItems: additions.
	additions := sorter orderedItems.
	requirements := sorter externalRequirements.
	unloadableDefinitions := sorter itemsWithMissingRequirements asSortedCollection.
	
	sorter := self sorterForItems: removals.
	removals := sorter orderedItems reversed.! !

!MCPackageLoader methodsFor: 'private' stamp: 'StephaneDucasse 9/13/2009 18:17'!
basicLoad
	errorDefinitions := OrderedCollection new.
	[[
	
	"FIXME. Do a separate pass on loading class definitions as the very first thing.
	This is a workaround for a problem with the so-called 'atomic' loading (you wish!!)
	which isn't atomic at all but mixes compilation of methods with reshapes of classes.

	Since the method is not installed until later, any class reshape in the middle *will*
	affect methods in subclasses that have been compiled before. There is probably
	a better way of dealing with this by ensuring that the sort order of the definition lists
	superclass definitions before methods for subclasses but I need this NOW, and adding
	an extra pass ensures that methods are compiled against their new class definitions."

	additions do: [:ea | self loadClassDefinition: ea] displayingProgress: 'Loading classes...'.
	
	additions do: [:ea | self tryToLoad: ea] displayingProgress: 'Compiling methods...'.
	removals do: [:ea | ea unload] displayingProgress: 'Cleaning up...'.
	self shouldWarnAboutErrors ifTrue: [self warnAboutErrors].
	errorDefinitions do: [:ea | ea addMethodAdditionTo: methodAdditions] displayingProgress: 'Reloading...'.
	methodAdditions do: [:each | each installMethod].
	methodAdditions do: [:each | each notifyObservers].
	additions do: [:ea | ea postloadOver: (self obsoletionFor: ea)] displayingProgress: 'Initializing...']
		on: InMidstOfFileinNotification 
		do: [:n | n resume: true]]
			ensure: [self flushChangesFile]! !

!MCPackageLoader methodsFor: 'private' stamp: 'ab 5/25/2003 01:24'!
dependencyWarning
	^ String streamContents:
		[:s |
		s nextPutAll: 'This package depends on the following classes:'; cr.
		requirements do: [:ea | s space; space; nextPutAll: ea; cr].
		s nextPutAll: 'You must resolve these dependencies before you will be able to load these definitions: '; cr.
		unloadableDefinitions do: [:ea | s space; space; nextPutAll: ea summary; cr]] ! !

!MCPackageLoader methodsFor: 'private' stamp: 'avi 1/24/2004 17:44'!
errorDefinitionWarning
	^ String streamContents:
		[:s |
		s nextPutAll: 'The following definitions had errors while loading.  Press Proceed to try to load them again (they may work on a second pass):'; cr.
		errorDefinitions do: [:ea | s space; space; nextPutAll: ea summary; cr]] ! !

!MCPackageLoader methodsFor: 'private' stamp: 'BertFreudenberg 12/26/2010 13:31'!
flushChangesFile
	"The changes file is second in the SourceFiles array"

	(SourceFiles at: 2) ifNotNil: [:f | f flush]! !

!MCPackageLoader methodsFor: 'private' stamp: 'StephaneDucasse 9/13/2009 18:15'!
initialize
	super initialize.
	additions := OrderedCollection new.
	removals := OrderedCollection new.
	obsoletions := Dictionary new.
	methodAdditions := OrderedCollection new. 
! !

!MCPackageLoader methodsFor: 'public' stamp: 'stephaneducasse 2/4/2006 20:47'!
installSnapshot: aSnapshot
	| patch |
	patch := aSnapshot patchRelativeToBase: MCSnapshot empty.
	patch applyTo: self.
! !

!MCPackageLoader methodsFor: 'public' stamp: 'Igor.Stasenko 10/1/2010 20:32'!
load
	self analyze.
	unloadableDefinitions isEmpty ifFalse: [self warnAboutDependencies].
	self useNewChangeSetDuring: [self basicLoad].
	MCMethodDefinition cachedDefinitions finalizeValues.! !

!MCPackageLoader methodsFor: 'private' stamp: 'StephaneDucasse 9/13/2009 18:17'!
loadClassDefinition: aDefinition

	[aDefinition isClassDefinition ifTrue:[aDefinition load]] on: Error do: [errorDefinitions add: aDefinition].! !

!MCPackageLoader methodsFor: 'public' stamp: 'nk 2/23/2005 07:51'!
loadWithName: baseName
	self analyze.
	unloadableDefinitions isEmpty ifFalse: [self warnAboutDependencies].
	self useChangeSetNamed: baseName during: [self basicLoad]! !

!MCPackageLoader methodsFor: 'public' stamp: 'nk 8/30/2004 08:39'!
loadWithNameLike: baseName
	self analyze.
	unloadableDefinitions isEmpty ifFalse: [self warnAboutDependencies].
	self useNewChangeSetNamedLike: baseName during: [self basicLoad]! !

!MCPackageLoader methodsFor: 'patch ops' stamp: 'avi 2/17/2004 13:14'!
modifyDefinition: old to: new
	self addDefinition: new.
	obsoletions at: new put: old.! !

!MCPackageLoader methodsFor: 'private' stamp: 'avi 2/17/2004 13:15'!
obsoletionFor: aDefinition
	^ obsoletions at: aDefinition ifAbsent: [nil]! !

!MCPackageLoader methodsFor: 'private' stamp: 'ab 5/25/2003 01:19'!
orderDefinitionsForLoading: aCollection
	^ (self sorterForItems: aCollection) orderedItems! !

!MCPackageLoader methodsFor: 'private' stamp: 'ab 5/24/2003 16:52'!
orderedAdditions
	^ additions! !

!MCPackageLoader methodsFor: 'private' stamp: 'NicoPaez 10/1/2010 10:33'!
provisions
	^ provisions ifNil: [provisions := Set withAll: Smalltalk globals keys]! !

!MCPackageLoader methodsFor: 'patch ops' stamp: 'ab 5/24/2003 16:14'!
removeDefinition: aDefinition
	removals add: aDefinition! !

!MCPackageLoader methodsFor: 'private' stamp: 'AlainPlantec 1/7/2010 22:20'!
shouldWarnAboutErrors
	^ errorDefinitions isEmpty not and: [false "should make this a setting ?"]! !

!MCPackageLoader methodsFor: 'private' stamp: 'stephaneducasse 2/4/2006 20:47'!
sorterForItems: aCollection
	| sorter |
	sorter := MCDependencySorter items: aCollection.
	sorter addExternalProvisions: self provisions.
	^ sorter! !

!MCPackageLoader methodsFor: 'private' stamp: 'StephaneDucasse 9/13/2009 18:16'!
tryToLoad: aDefinition
	[aDefinition addMethodAdditionTo: methodAdditions] on: Error do: [errorDefinitions add: aDefinition].! !

!MCPackageLoader methodsFor: 'public' stamp: 'EstebanLorenzano 10/17/2012 16:54'!
unloadPackage: aPackage
	self updatePackage: aPackage withSnapshot: MCSnapshot empty.
	MCMethodDefinition cachedDefinitions finalizeValues.! !

!MCPackageLoader methodsFor: 'public' stamp: 'stephaneducasse 2/4/2006 20:47'!
updatePackage: aPackage withSnapshot: aSnapshot
	|  patch packageSnap |
	packageSnap := aPackage snapshot.
	patch := aSnapshot patchRelativeToBase: packageSnap.
	patch applyTo: self.
	packageSnap definitions do: [:ea | self provisions addAll: ea provisions]
! !

!MCPackageLoader methodsFor: 'private' stamp: 'MarcusDenker 3/28/2011 18:13'!
useChangeSetNamed: baseName during: aBlock
	"Use the named change set, or create one with the given name."
	| oldChanges newChanges |
	oldChanges :=ChangeSet current.
	newChanges := (ChangeSet named: baseName) ifNil: [ ChangeSet new name: baseName ].
	ChangeSet newChanges: newChanges.
	[aBlock value] ensure: [ChangeSet newChanges: oldChanges].
! !

!MCPackageLoader methodsFor: 'private' stamp: 'nk 8/30/2004 08:38'!
useNewChangeSetDuring: aBlock
	^self useNewChangeSetNamedLike: 'MC' during: aBlock! !

!MCPackageLoader methodsFor: 'private' stamp: 'nk 2/23/2005 07:50'!
useNewChangeSetNamedLike: baseName during: aBlock
	^self useChangeSetNamed: (ChangeSet uniqueNameLike: baseName) during: aBlock! !

!MCPackageLoader methodsFor: 'private' stamp: 'ab 5/25/2003 01:22'!
warnAboutDependencies 
	self notify: self dependencyWarning! !

!MCPackageLoader methodsFor: 'private' stamp: 'avi 1/24/2004 17:42'!
warnAboutErrors
	self notify: self errorDefinitionWarning.
! !

Object subclass: #MCPatch
	instanceVariableNames: 'operations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Patching'!

!MCPatch commentStamp: 'StephaneDucasse 6/24/2011 14:11' prior: 0!
I represent a set of patch operations which can be applied by sending message applyTo: to my instances. !

!MCPatch class methodsFor: 'as yet unclassified' stamp: 'avi 9/11/2004 15:49'!
fromBase: baseSnapshot target: targetSnapshot
	^ self new initializeWithBase: baseSnapshot target: targetSnapshot! !

!MCPatch class methodsFor: 'as yet unclassified' stamp: 'avi 9/11/2004 15:50'!
operations: aCollection
	^ self basicNew initializeWithOperations: aCollection! !

!MCPatch methodsFor: 'applying' stamp: 'ab 5/24/2003 16:12'!
applyTo: anObject
	operations do: [:ea | ea applyTo: anObject].
! !

!MCPatch methodsFor: 'accessing' stamp: 'CamilloBruni 9/28/2011 17:27'!
hasConflict
	^ self operations anySatisfy: [ :change| change isConflict ]! !

!MCPatch methodsFor: 'intializing' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithBase: baseSnapshot target: targetSnapshot
	| base target |	
	operations := OrderedCollection new.
	base := MCDefinitionIndex definitions: baseSnapshot definitions.
	target := MCDefinitionIndex definitions: targetSnapshot definitions.
	
	target definitions do:
		[:t |
		base
			definitionLike: t
			ifPresent: [:b | (b isSameRevisionAs: t) ifFalse: [operations add: (MCModification of: b to: t)]]
			ifAbsent: [operations add: (MCAddition of: t)]]
		displayingProgress: 'Diffing...'.
		
	base definitions do:
		[:b |
		target
			definitionLike: b
			ifPresent: [:t]
			ifAbsent: [operations add: (MCRemoval of: b)]]		! !

!MCPatch methodsFor: 'intializing' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithOperations: aCollection
	operations := aCollection! !

!MCPatch methodsFor: 'querying' stamp: 'cwp 6/9/2003 11:53'!
isEmpty
	^ operations isEmpty! !

!MCPatch methodsFor: 'accessing' stamp: 'ab 5/13/2003 12:18'!
operations
	^ operations! !

Object subclass: #MCPatchOperation
	instanceVariableNames: 'isApplicable isConflict'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Patching'!

!MCPatchOperation commentStamp: 'StephaneDucasse 6/24/2011 14:15' prior: 0!
I'm an abstract class that represent an single delta element between two packages: removal, addition or modification.!

MCPatchOperation subclass: #MCAddition
	instanceVariableNames: 'definition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Patching'!

!MCAddition commentStamp: '<historical>' prior: 0!
A MCAddition represents the operation to add an entity to a snapshot. !

!MCAddition class methodsFor: 'as yet unclassified' stamp: 'cwp 11/27/2002 10:03'!
of: aDefinition
	^ self new intializeWithDefinition: aDefinition! !

!MCAddition methodsFor: 'accessing' stamp: 'ab 6/1/2003 13:09'!
baseDefinition
	^ nil! !

!MCAddition methodsFor: 'accessing' stamp: 'StephaneDucasse 6/24/2011 14:53'!
basicApplyTo: anObject
	anObject addDefinition: definition! !

!MCAddition methodsFor: 'accessing' stamp: 'cwp 11/27/2002 09:32'!
definition
	^ definition! !

!MCAddition methodsFor: 'accessing' stamp: 'ab 7/18/2003 16:45'!
fromSource
	^ ''! !

!MCAddition methodsFor: 'initializing' stamp: 'stephaneducasse 2/4/2006 20:47'!
intializeWithDefinition: aDefinition
	definition := aDefinition! !

!MCAddition methodsFor: 'accessing' stamp: 'ab 8/22/2003 02:26'!
inverse
	^ MCRemoval of: definition! !

!MCAddition methodsFor: 'testing' stamp: 'cwp 11/28/2002 07:22'!
isAddition
	^ true! !

!MCAddition methodsFor: 'testing' stamp: 'nk 2/25/2005 17:28'!
isClassPatch
	^definition isClassDefinition! !

!MCAddition methodsFor: 'accessing' stamp: 'nk 2/23/2005 18:39'!
sourceString
	^(self toSource asText)
		addAttribute: TextColor red;
		yourself! !

!MCAddition methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 9/28/2011 15:25'!
summary

	^ definition summary! !

!MCAddition methodsFor: 'accessing' stamp: 'nk 2/23/2005 18:17'!
targetClass
	^definition actualClass ! !

!MCAddition methodsFor: 'accessing' stamp: 'ab 6/1/2003 13:09'!
targetDefinition
	^ definition! !

!MCAddition methodsFor: 'accessing' stamp: 'ab 7/18/2003 16:45'!
toSource
	^ definition source! !

MCPatchOperation subclass: #MCModification
	instanceVariableNames: 'obsoletion modification'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Patching'!

!MCModification commentStamp: '<historical>' prior: 0!
A MCModification represents the operation to modify an entity to a snapshot. !

!MCModification class methodsFor: 'as yet unclassified' stamp: 'cwp 11/28/2002 07:19'!
of: base to: target
	^ self new initializeWithBase: base target: target! !

!MCModification methodsFor: 'accessing' stamp: 'ab 6/1/2003 13:10'!
baseDefinition
	^ obsoletion! !

!MCModification methodsFor: 'accessing' stamp: 'StephaneDucasse 6/24/2011 14:53'!
basicApplyTo: anObject
	anObject modifyDefinition: obsoletion to: modification! !

!MCModification methodsFor: 'accessing' stamp: 'cwp 11/28/2002 06:55'!
definition
	^ modification! !

!MCModification methodsFor: 'accessing' stamp: 'ab 7/18/2003 16:44'!
fromSource
	^ obsoletion source! !

!MCModification methodsFor: 'initializing' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithBase: base target: target
	obsoletion := base.
	modification := target.! !

!MCModification methodsFor: 'accessing' stamp: 'ab 8/22/2003 02:27'!
inverse
	^ MCModification of: modification to: obsoletion! !

!MCModification methodsFor: 'testing' stamp: 'nk 2/25/2005 17:29'!
isClassPatch
	^obsoletion isClassDefinition! !

!MCModification methodsFor: 'testing' stamp: 'cwp 11/28/2002 07:23'!
isModification
	^ true! !

!MCModification methodsFor: 'accessing' stamp: 'cwp 11/27/2002 09:46'!
modification
	^ modification! !

!MCModification methodsFor: 'accessing' stamp: 'cwp 11/27/2002 09:48'!
obsoletion
	^ obsoletion! !

!MCModification methodsFor: 'accessing' stamp: 'nk 11/10/2003 21:44'!
printAnnotations: request on: aStream
	aStream nextPutAll: 'old: '.
	obsoletion printAnnotations: request on: aStream.
	aStream cr.
	aStream nextPutAll: 'new: '.
	modification printAnnotations: request on: aStream.! !

!MCModification methodsFor: 'accessing' stamp: 'StephaneDucasse 8/17/2012 16:30'!
summarySuffix
	^ modification summarySuffixOver: obsoletion
! !

!MCModification methodsFor: 'as yet unclassified' stamp: 'StephaneDucasse 8/17/2012 16:30'!
summarySuffixOver: previousDefinition
	| sourceChanged categoryChanged timeStampChanged |
	sourceChanged := self source ~= previousDefinition source.
	timeStampChanged := self timeStamp ~= previousDefinition timeStamp.
	categoryChanged := self category ~= previousDefinition category.
	sourceChanged | timeStampChanged | categoryChanged
		ifFalse: [ ^super summarySuffixOver: previousDefinition ].
	sourceChanged ifTrue: [
		^categoryChanged
			ifTrue: [ ' (changed and recategorized)' ]
			ifFalse: [ ' (changed)' ] ].
	timeStampChanged & categoryChanged
		ifTrue: [^ ' (recategorized and different time stamp)' ].
	^categoryChanged
		ifTrue: [ ' (only recategorized)' ]
		ifFalse: [ ' (only different time stamp)' ]
! !

!MCModification methodsFor: 'accessing' stamp: 'nk 2/23/2005 18:18'!
targetClass
	^ obsoletion actualClass! !

!MCModification methodsFor: 'accessing' stamp: 'ab 6/1/2003 13:10'!
targetDefinition
	^ modification! !

!MCModification methodsFor: 'accessing' stamp: 'ab 7/18/2003 16:44'!
toSource
	^ modification source! !

!MCPatchOperation methodsFor: 'comparing' stamp: 'ab 7/19/2003 18:11'!
<= other
	^ self definition <= other definition! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'StephaneDucasse 12/30/2012 17:53'!
annotations
	^self annotations: Annotation annotationRequests! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'AlainPlantec 1/7/2010 22:21'!
annotations: requests
	"Answer a string for an annotation pane, trying to fulfill the CodeHolder annotationRequests"
	^String streamContents: [ :s | self printAnnotations: requests on: s ].! !

!MCPatchOperation methodsFor: 'applying operation' stamp: 'IgorStasenko 6/24/2011 17:29'!
applyTo: anObject

	(self isConflict and: [self isResolved not] ) ifTrue: [
		self error: 'Cannot continue until this conflict has been resolved' ].
	
	self remoteChosen ifTrue: [ self basicApplyTo: anObject ].! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'StephaneDucasse 6/24/2011 15:00'!
beConflict
	isConflict := true.
	isApplicable := nil. "not resolved"! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'StephaneDucasse 6/24/2011 15:12'!
beNonConflict
	isConflict := false.
	isApplicable := true.! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'StephaneDucasse 6/24/2011 14:54'!
chooseLocal
	isApplicable := false! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'StephaneDucasse 6/24/2011 14:54'!
chooseNewer
	self isLocalNewer 
		ifTrue: [ self chooseLocal ]
		ifFalse: [ self isRemoteNewer ifTrue: [ self chooseRemote ]]! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'StephaneDucasse 6/24/2011 14:55'!
chooseOlder
	self isRemoteNewer 
		ifTrue: [ self chooseLocal ]
		ifFalse: [ self isLocalNewer ifTrue: [ self chooseRemote ]]! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'StephaneDucasse 6/24/2011 14:55'!
chooseRemote
	isApplicable := true! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'StephaneDucasse 6/24/2011 14:55'!
clearChoice
	isApplicable := nil! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'cwp 11/28/2002 06:59'!
definition
	^ self subclassResponsibility ! !

!MCPatchOperation methodsFor: 'initialize-release' stamp: 'StephaneDucasse 6/24/2011 15:13'!
initialize
	super initialize.
	
	"set default values"
	
	self beNonConflict! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'cwp 11/27/2002 09:26'!
inverse
	self subclassResponsibility! !

!MCPatchOperation methodsFor: 'testing' stamp: 'cwp 11/27/2002 09:30'!
isAddition
	^ false! !

!MCPatchOperation methodsFor: 'as yet unclassified' stamp: 'nk 2/25/2005 17:28'!
isClassPatch
	^false! !

!MCPatchOperation methodsFor: 'testing' stamp: 'StephaneDucasse 6/24/2011 14:44'!
isConflict
	^ isConflict == true! !

!MCPatchOperation methodsFor: 'testing' stamp: 'StephaneDucasse 6/24/2011 14:56'!
isLocalNewer
	^ self localDefinition fullTimeStamp > self remoteDefinition fullTimeStamp! !

!MCPatchOperation methodsFor: 'testing' stamp: 'cwp 11/27/2002 09:30'!
isModification
	^ false! !

!MCPatchOperation methodsFor: 'testing' stamp: 'StephaneDucasse 6/24/2011 14:56'!
isRemoteNewer
	^ self localDefinition fullTimeStamp < self remoteDefinition fullTimeStamp! !

!MCPatchOperation methodsFor: 'testing' stamp: 'cwp 11/27/2002 09:30'!
isRemoval
	^ false! !

!MCPatchOperation methodsFor: 'testing' stamp: 'StephaneDucasse 6/24/2011 15:07'!
isResolved
	^ isConflict and: [ isApplicable notNil ]! !

!MCPatchOperation methodsFor: 'testing' stamp: 'StephaneDucasse 6/24/2011 14:57'!
localChosen
	^ isApplicable == false
! !

!MCPatchOperation methodsFor: 'testing' stamp: 'StephaneDucasse 6/24/2011 14:57'!
localDefinition
	^ self baseDefinition! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'StephaneDucasse 6/24/2011 14:57'!
operation
	"for compatibility with MCConflict interface"
	^ self! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'gvc 10/30/2006 11:15'!
patchWrapper
	"Answer a wrapper for a patch tree for the receiver."

	^PSMCPatchOperationWrapper with: self! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'avi 8/31/2003 17:53'!
prefixForOperation: aSymbol
	aSymbol == #insert ifTrue: [^ '+'].
	aSymbol == #remove ifTrue: [^ '-'].
	^ ' '! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'AlainPlantec 1/7/2010 22:21'!
printAnnotations: requests on: aStream
	"Add a string for an annotation pane, trying to fulfill the CodeHolder annotationRequests"

	self definition printAnnotations: requests on: aStream.! !

!MCPatchOperation methodsFor: 'testing' stamp: 'IgorStasenko 6/24/2011 17:28'!
remoteChosen
	^ isApplicable ~~ false! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'StephaneDucasse 6/24/2011 14:58'!
remoteDefinition
	^ self targetDefinition! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'CamilloBruni 8/4/2011 15:16'!
source
	^ self localChosen
		ifTrue: [self fromSource]
		ifFalse: [self sourceText]! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'nk 2/25/2005 17:26'!
sourceString
	^self sourceText asString! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'AlainPlantec 12/1/2009 22:09'!
sourceText
	^ self isClassPatch 
		ifTrue: [(TextDiffBuilder from: self fromSource to: self toSource) buildDisplayPatch]
		ifFalse: [TextDiffBuilder buildDisplayPatchFrom:  self fromSource to: self toSource inClass: self targetClass]
! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'StephaneDucasse 6/24/2011 14:59'!
status
	^ self isResolved
		ifFalse: ['']
		ifTrue: [self remoteChosen
					ifFalse: ['L']
					ifTrue: ['R']]! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'CamilloBruni 8/4/2011 15:15'!
summary
	| attribute |
	attribute := 
		self isResolved
			ifTrue: [self remoteChosen ifTrue: [#underlined] ifFalse: [#struckOut]]
			ifFalse: [#bold].
	^ Text string:( self definition summary, self summarySuffix) attribute: (TextEmphasis perform: attribute)! !

!MCPatchOperation methodsFor: 'accessing' stamp: 'ab 7/6/2003 00:06'!
summarySuffix
	^ ''! !

!MCPatchOperation methodsFor: 'as yet unclassified' stamp: 'nk 2/23/2005 18:17'!
targetClass
	self subclassResponsibility.! !

MCPatchOperation subclass: #MCRemoval
	instanceVariableNames: 'definition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Patching'!

!MCRemoval commentStamp: '<historical>' prior: 0!
A MCRemoval represents the removal of an entity of a given snapshot.!

!MCRemoval class methodsFor: 'as yet unclassified' stamp: 'cwp 11/27/2002 10:03'!
of: aDefinition
	^ self new intializeWithDefinition: aDefinition! !

!MCRemoval methodsFor: 'accessing' stamp: 'ab 6/1/2003 13:10'!
baseDefinition
	^ definition! !

!MCRemoval methodsFor: 'accessing' stamp: 'StephaneDucasse 6/24/2011 14:53'!
basicApplyTo: anObject
	anObject removeDefinition: definition! !

!MCRemoval methodsFor: 'accessing' stamp: 'cwp 11/27/2002 10:02'!
definition
	^ definition! !

!MCRemoval methodsFor: 'accessing' stamp: 'ab 7/18/2003 16:44'!
fromSource
	^ definition source! !

!MCRemoval methodsFor: 'initializing' stamp: 'stephaneducasse 2/4/2006 20:47'!
intializeWithDefinition: aDefinition
	definition := aDefinition! !

!MCRemoval methodsFor: 'accessing' stamp: 'ab 8/22/2003 02:26'!
inverse
	^ MCAddition of: definition! !

!MCRemoval methodsFor: 'testing' stamp: 'nk 2/25/2005 17:28'!
isClassPatch
	^definition isClassDefinition! !

!MCRemoval methodsFor: 'testing' stamp: 'cwp 11/28/2002 07:24'!
isRemoval
	^ true! !

!MCRemoval methodsFor: 'accessing' stamp: 'nk 2/23/2005 18:38'!
sourceString
	^self fromSource asText
		addAttribute: TextEmphasis struckOut;
		addAttribute: TextColor blue;
		yourself! !

!MCRemoval methodsFor: 'accessing' stamp: 'ab 7/6/2003 00:05'!
summarySuffix
	^ ' (removed)'! !

!MCRemoval methodsFor: 'accessing' stamp: 'nk 2/25/2005 17:23'!
targetClass
	^ definition actualClass! !

!MCRemoval methodsFor: 'accessing' stamp: 'ab 6/1/2003 13:10'!
targetDefinition
	^ nil! !

!MCRemoval methodsFor: 'accessing' stamp: 'ab 7/18/2003 16:44'!
toSource
	^ ''! !

Object subclass: #MCPatcher
	instanceVariableNames: 'definitions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Patching'!

!MCPatcher commentStamp: 'LaurentLaffont 3/31/2011 21:05' prior: 0!
I can add or remove or modify definitions. My main purpose is to generate monticello snapshot out of multiple snapshots or definitions by using my most used method #apply:to:.!

!MCPatcher class methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
apply: aPatch to: aSnapshot
	| loader |
	loader := self snapshot: aSnapshot.
	aPatch applyTo: loader.
	^ loader patchedSnapshot! !

!MCPatcher class methodsFor: 'as yet unclassified' stamp: 'ab 6/1/2003 14:22'!
snapshot: aSnapshot
	^ self new initializeWithSnapshot: aSnapshot! !

!MCPatcher methodsFor: 'as yet unclassified' stamp: 'ab 6/2/2003 00:46'!
addDefinition: aDefinition
	definitions add: aDefinition! !

!MCPatcher methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithSnapshot: aSnapshot
	definitions := MCDefinitionIndex definitions: aSnapshot definitions! !

!MCPatcher methodsFor: 'as yet unclassified' stamp: 'ab 6/1/2003 14:23'!
modifyDefinition: baseDefinition to: targetDefinition
	self addDefinition: targetDefinition! !

!MCPatcher methodsFor: 'as yet unclassified' stamp: 'ab 7/6/2003 23:48'!
patchedSnapshot
	^ MCSnapshot fromDefinitions: definitions definitions! !

!MCPatcher methodsFor: 'as yet unclassified' stamp: 'ab 6/2/2003 00:46'!
removeDefinition: aDefinition
	definitions remove: aDefinition! !

Object subclass: #MCReader
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Storing'!

!MCReader class methodsFor: 'testing' stamp: 'avi 1/21/2004 19:00'!
canReadFileNamed: fileName
	^ (fileName endsWith: self extension)! !

!MCReader class methodsFor: 'testing' stamp: 'avi 1/21/2004 19:01'!
concreteSubclasses
	^ self allSubclasses reject: [:c | c isAbstract]! !

!MCReader class methodsFor: 'testing' stamp: 'avi 1/21/2004 19:01'!
isAbstract
	^ (self respondsTo: #extension) not! !

!MCReader class methodsFor: 'instance creation' stamp: 'avi 1/21/2004 19:02'!
on: aStream
	^ self new stream: aStream! !

!MCReader class methodsFor: 'instance creation' stamp: 'stephaneducasse 2/4/2006 20:47'!
on: aStream name: aFileName
	| class |
	class := self readerClassForFileNamed: aFileName.
	^ class
		ifNil: [self error: 'Unsupported format: ', aFileName]
		ifNotNil: [class on: aStream]! !

!MCReader class methodsFor: 'testing' stamp: 'avi 1/21/2004 19:03'!
readerClassForFileNamed: fileName
	^ self concreteSubclasses
		detect: [:c | c canReadFileNamed: fileName]
		ifNone: [nil]! !

!MCReader methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 4/24/2012 15:49'!
stream
	stream closed ifTrue: [ stream open ].
	^ stream
	! !

!MCReader methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
stream: aStream
	stream := aStream! !

MCReader subclass: #MCSnapshotReader
	instanceVariableNames: 'definitions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Storing'!

!MCSnapshotReader commentStamp: 'LaurentLaffont 2/23/2011 20:21' prior: 0!
I am an abstract class defining the interface for reading sources containing class and method definitions.
I should define loadDefinitions as a subclassResponsibility, but don't.
MCVersionReader could be a subclass of me for reading sources containing additional metadata, but isn't. 

I'm useless and can be safely removed. I'm only referenced in MCMczReader>>extractDefinitionsFrom: and it can be replaced by MCReader.!

!MCSnapshotReader class methodsFor: 'as yet unclassified' stamp: 'avi 1/21/2004 22:56'!
snapshotFromStream: aStream
	^ (self on: aStream) snapshot! !

!MCSnapshotReader methodsFor: 'as yet unclassified' stamp: 'avi 1/21/2004 23:09'!
definitions
	definitions ifNil: [self loadDefinitions].
	^ definitions! !

!MCSnapshotReader methodsFor: 'as yet unclassified' stamp: 'avi 1/21/2004 23:10'!
snapshot
	^ MCSnapshot fromDefinitions: self definitions! !

MCSnapshotReader subclass: #MCStReader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Storing'!

!MCStReader commentStamp: 'LaurentLaffont 3/31/2011 21:07' prior: 0!
I read Category/Classes/Methodes/.... definitions from Monticello source.st file format.

See also MCStWriter.

Example:

|source|
source := String streamContents: [:aStream| |writer|
    writer := MCStWriter on: aStream.
    writer writeDefinitions: {True asClassDefinition. False asClassDefinition}.
].

(MCStReader on: source readStream) definitions explore.!

!MCStReader class methodsFor: 'as yet unclassified' stamp: 'avi 1/20/2004 00:17'!
extension
	^ 'st'! !

!MCStReader methodsFor: 'as yet unclassified' stamp: 'marcus.denker 11/10/2008 10:04'!
addDefinitionsFromDoit: aString
	(MCDoItParser forDoit: aString) ifNotNil:
		[:parser |
		parser addDefinitionsTo: definitions]! !

!MCStReader methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
categoryFromDoIt: aString
	| tokens  |
	tokens := Scanner new scanTokens: aString.
	tokens size = 3 ifFalse: [self error: 'Unrecognized category definition'].
	^ tokens at: 3! !

!MCStReader methodsFor: 'as yet unclassified' stamp: 'SvenVanCaekenberghe 1/8/2012 14:45'!
classDefinitionFrom: aPseudoClass
	| tokens traitCompositionString lastIndex classTraitCompositionString |
	tokens := Scanner new scanTokens: aPseudoClass definition.
	traitCompositionString := (aPseudoClass definition readStream
		match: 'uses:';
		upToAll: 'instanceVariableNames:') trimBoth.
	classTraitCompositionString := (aPseudoClass metaClass definition asString readStream
		match: 'uses:';
		upToAll: 'instanceVariableNames:') trimBoth.
	traitCompositionString isEmpty ifTrue: [traitCompositionString := '{}'].
	classTraitCompositionString isEmpty ifTrue: [classTraitCompositionString := '{}'].
	lastIndex := tokens size.
	^ MCClassDefinition
		name: (tokens at: 3)
		superclassName: (tokens at: 1)
		traitComposition: traitCompositionString
		classTraitComposition: classTraitCompositionString
		category: (tokens at: lastIndex)
		instVarNames: ((tokens at: lastIndex - 6) findTokens: ' ')
		classVarNames: ((tokens at: lastIndex - 4) findTokens: ' ')
		poolDictionaryNames: ((tokens at: lastIndex - 2) findTokens: ' ')
		classInstVarNames: (self classInstVarNamesFor: aPseudoClass)
		type: (self typeOfSubclass: (tokens at: 2))
		comment: (self commentFor: aPseudoClass)
		commentStamp: (self commentStampFor: aPseudoClass)! !

!MCStReader methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
classInstVarNamesFor: aPseudoClass
	| tokens |
	
	self flag: #traits.
	aPseudoClass metaClass hasDefinition ifFalse: [^ #()].
	tokens := Scanner new scanTokens: aPseudoClass metaClass definition.
	"tokens size = 4 ifFalse: [self error: 'Unrecognized metaclass definition']."
	^ tokens last findTokens: ' '! !

!MCStReader methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
commentFor: aPseudoClass
	| comment |
	comment := aPseudoClass organization classComment.
	^ comment asString = ''
		ifTrue: [comment]
		ifFalse: [comment string]! !

!MCStReader methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
commentStampFor: aPseudoClass
	| comment |
	comment := aPseudoClass organization classComment.
	^  [comment stamp] on: MessageNotUnderstood do: [nil]! !

!MCStReader methodsFor: 'evaluating' stamp: 'stephaneducasse 2/4/2006 20:47'!
loadDefinitions
	| filePackage |
	filePackage :=
		FilePackage new
			fullName: 'ReadStream';
			fileInFrom: self readStream.
	definitions := OrderedCollection new.
	filePackage classes do:
		[:pseudoClass |
		pseudoClass hasDefinition
			ifTrue: [definitions add:
					(self classDefinitionFrom: pseudoClass)].
		definitions addAll: (self methodDefinitionsFor: pseudoClass).
		definitions addAll: (self methodDefinitionsFor: pseudoClass metaClass)].
	filePackage doIts do:
		[:ea |
		self addDefinitionsFromDoit: ea string].
	! !

!MCStReader methodsFor: 'as yet unclassified' stamp: 'avi 3/3/2004 15:23'!
methodDefinitionsFor: aPseudoClass
	^ aPseudoClass selectors collect: 
		[:ea |
		 MCMethodDefinition
			className: aPseudoClass name
			classIsMeta: aPseudoClass isMeta
			selector: ea
			category: (aPseudoClass organization categoryOfElement: ea)
			timeStamp: (aPseudoClass stampAt: ea)
			source: (aPseudoClass sourceCodeAt: ea)]! !

!MCStReader methodsFor: 'evaluating' stamp: 'avi 1/21/2004 14:21'!
readStream
	^ ('!!!!

', stream contents) readStream! !

!MCStReader methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
systemOrganizationFromRecords: changeRecords
	| categories |
	categories := changeRecords
					select: [:ea | 'SystemOrganization*' match: ea string]
					thenCollect: [:ea | (self categoryFromDoIt: ea string)].
	^ categories isEmpty ifFalse: [MCOrganizationDefinition categories: categories asArray]! !

!MCStReader methodsFor: 'as yet unclassified' stamp: 'avi 1/19/2004 21:56'!
typeOfSubclass: aSymbol
	#(
		(subclass: normal)
		(variableSubclass: variable)
		(variableByteSubclass: bytes)
		(variableWordSubclass: words)
		(weakSubclass: weak)
		) do: [:ea | ea first = aSymbol ifTrue: [^ ea second]].
	self error: 'Unrecognized class definition'! !

MCReader subclass: #MCVersionReader
	instanceVariableNames: 'package info definitions dependencies'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Storing'!

MCVersionReader subclass: #MCMczReader
	instanceVariableNames: 'zip infoCache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Storing'!

MCMczReader subclass: #MCMcdReader
	instanceVariableNames: 'baseInfo patch'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Storing'!

!MCMcdReader class methodsFor: 'as yet unclassified' stamp: 'avi 2/13/2004 23:09'!
extension
	^ 'mcd'! !

!MCMcdReader methodsFor: 'as yet unclassified' stamp: 'avi 2/14/2004 21:33'!
baseInfo
	^ baseInfo ifNil: [self loadBaseInfo]! !

!MCMcdReader methodsFor: 'as yet unclassified' stamp: 'avi 2/13/2004 23:41'!
basicVersion
	^ MCDiffyVersion
		package: self package
		info: self info
		dependencies: self dependencies
		baseInfo: self baseInfo
		patch: self patch! !

!MCMcdReader methodsFor: 'as yet unclassified' stamp: 'avi 2/14/2004 21:37'!
buildPatchFrom: oldDefinitions to: newDefinitions
	^ MCPatch
		fromBase: (MCSnapshot fromDefinitions: oldDefinitions)
		target: (MCSnapshot fromDefinitions: newDefinitions)! !

!MCMcdReader methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
loadBaseInfo
	^ baseInfo := self extractInfoFrom: (self parseMember: 'base')! !

!MCMcdReader methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 6/17/2012 12:37'!
loadPatch
	| old new |
	(self zip memberNamed: 'patch.bin') ifNotNil:
		[:m | [^ patch := (MCDataStream on: m contentStream) next ]
			on: Error do: [:fallThrough ]].
	definitions := OrderedCollection new.
	(self zip membersMatching: 'old/*')
		do: [:m | self extractDefinitionsFrom: m].
	old := definitions asArray.
	definitions := OrderedCollection new.
	(self zip membersMatching: 'new/*')
		do: [:m | self extractDefinitionsFrom: m].
	new := definitions asArray.
	^ patch := self buildPatchFrom: old to: new.
	! !

!MCMcdReader methodsFor: 'as yet unclassified' stamp: 'avi 2/14/2004 21:34'!
patch
	^ patch ifNil: [self loadPatch]! !

!MCMczReader class methodsFor: 'accessing' stamp: 'cwp 8/1/2003 14:59'!
extension
	^ 'mcz'! !

!MCMczReader class methodsFor: 'testing' stamp: 'avi 1/19/2004 14:48'!
supportsDependencies
	^ true! !

!MCMczReader class methodsFor: 'testing' stamp: 'cwp 8/1/2003 12:19'!
supportsVersions
	^ true! !

!MCMczReader methodsFor: 'as yet unclassified' stamp: 'stephane.ducasse 3/31/2009 21:23'!
associate: tokens
	| result |
	result := Dictionary new.
	tokens pairsDo: [:key :value | 
					| tmp |
					tmp := value.
					value isString ifFalse: [tmp := value collect: [:ea | self associate: ea]].
					value = 'nil' ifTrue: [tmp := ''].
					result at: key put: tmp].
	^ result! !

!MCMczReader methodsFor: 'as yet unclassified' stamp: 'marcus.denker 11/10/2008 10:04'!
extractDefinitionsFrom: member
	| reader |
	(MCSnapshotReader readerClassForFileNamed: member fileName)
		ifNotNil: [:rc | reader := rc on: member contentStream text.
					definitions addAll: reader definitions]
! !

!MCMczReader methodsFor: 'as yet unclassified' stamp: 'avi 1/19/2004 16:11'!
extractDependencyFrom: zipMember
	^ MCVersionDependency
		package: (MCPackage named: (zipMember fileName copyAfterLast: $/))
		info: (self extractInfoFrom: (self parseMember: zipMember fileName))! !

!MCMczReader methodsFor: 'as yet unclassified' stamp: 'StephaneDucasse 8/17/2012 15:50'!
extractInfoFrom: dict
	^MCWorkingCopy infoFromDictionary: dict cache: self infoCache! !

!MCMczReader methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
infoCache
	^ infoCache ifNil: [infoCache := Dictionary new]! !

!MCMczReader methodsFor: 'loading' stamp: 'MarianoMartinezPeck 6/17/2012 12:37'!
loadDefinitions
	definitions := OrderedCollection new.
	(self zip memberNamed: 'snapshot.bin') ifNotNil:
		[:m | [^ definitions := (MCDataStream on: m contentStream) next definitions]
			on: Error do: [:fallThrough ]].
	"otherwise"
	(self zip membersMatching: 'snapshot/*')
		do: [:m | self extractDefinitionsFrom: m].
! !

!MCMczReader methodsFor: 'loading' stamp: 'stephaneducasse 2/4/2006 20:47'!
loadDependencies
	dependencies := (self zip membersMatching: 'dependencies/*') collect: [:m | self extractDependencyFrom: m].
	dependencies := dependencies asArray.
! !

!MCMczReader methodsFor: 'loading' stamp: 'stephaneducasse 2/4/2006 20:47'!
loadPackage
	| dict |
	dict := self parseMember: 'package'.
	package := MCPackage named: (dict at: #name)! !

!MCMczReader methodsFor: 'loading' stamp: 'BenjaminVanRyseghem 8/31/2012 15:10'!
loadVersionInfo

	info := self extractInfoFrom: (self parseMember: 'version')! !

!MCMczReader methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
parseMember: fileName
	| tokens |
	tokens := (self scanner scanTokens: (self zip contentsOf: fileName)) first.
	^ self associate: tokens! !

!MCMczReader methodsFor: 'as yet unclassified' stamp: 'avi 1/22/2004 20:33'!
scanner
	^ MCScanner! !

!MCMczReader methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 4/24/2012 15:47'!
zip
	stream closed ifTrue: [ zip := nil].
	zip ifNil:
		[zip := ZipArchive new.
		zip readFrom: self stream].
	^ zip! !

!MCVersionReader class methodsFor: 'reading' stamp: 'StephaneDucasse 2/2/2010 22:07'!
file: fileName streamDo: aBlock
	
	^ FileStream 
			readOnlyFileNamed: fileName 
			do: [:file | aBlock value: file]! !

!MCVersionReader class methodsFor: 'System-FileRegistry' stamp: 'CamilloBruni 2/3/2013 17:21'!
fileReaderServicesForFile: fullName suffix: suffix
	<fileService>
	self isAbstract ifTrue: [^ #()].
	^ ((suffix = self extension) or: [ suffix = '*' ])
		ifTrue: [self services]
		ifFalse: [Array new: 0]
		! !

!MCVersionReader class methodsFor: 'System-FileRegistry' stamp: 'CamilloBruni 2/3/2013 17:22'!
initialize
	"MCVersionReader initialize"

	Smalltalk globals at: #MczInstaller ifPresent: [ :installer | 
		FileServices unregisterFileReader: installer ].
	
	self concreteSubclasses do: [ :aClass | 
		"get rid of AnObsoleteMCMcReader and AnObsoleteMCMcvReader"
		FileServices registerFileReader: aClass ].
	
	FileServices registeredFileReaderClasses 
		select: [ :ea | ea isObsolete ]
		thenDo: [ :ea | FileServices unregisterFileReader: ea ]! !

!MCVersionReader class methodsFor: 'System-FileRegistry' stamp: 'CamilloBruni 5/4/2012 21:38'!
loadVersionFile: fileName
	| version |
	version := self versionFromFile: fileName.
	version workingCopy repositoryGroup addRepository:
		(MCDirectoryRepository new directory: fileName asFileReference).
	version load.
! !

!MCVersionReader class methodsFor: '*monticello-file services' stamp: 'SeanDeNigris 6/18/2012 16:56'!
loadVersionStream: stream fromDirectory: directory
	| version repository |
	repository := MCDirectoryRepository new directory: directory.
	version := self versionFromStream: stream.
	version workingCopy repositoryGroup addRepository: repository.
	version load.
! !

!MCVersionReader class methodsFor: 'System-FileRegistry' stamp: 'cwp 8/1/2003 14:46'!
mergeVersionFile: fileName
	(self versionFromFile: fileName) merge! !

!MCVersionReader class methodsFor: '*monticello-file services' stamp: 'nk 2/25/2005 11:17'!
mergeVersionStream: stream
	(self versionFromStream: stream) merge! !

!MCVersionReader class methodsFor: 'reading' stamp: 'bf 3/23/2005 01:20'!
on: s fileName: f
	^ self on: s! !

!MCVersionReader class methodsFor: 'System-FileRegistry' stamp: 'cwp 8/1/2003 14:46'!
openVersionFile: fileName
	(self versionFromFile: fileName) open! !

!MCVersionReader class methodsFor: '*monticello-file services' stamp: 'nk 2/25/2005 11:12'!
openVersionFromStream: stream
	(self versionFromStream: stream) open! !

!MCVersionReader class methodsFor: '*monticello-file services' stamp: 'tbn 8/11/2010 10:19'!
serviceLoadVersion
	^ (SimpleServiceEntry
		provider: self
		label: 'Load version'
		selector: #loadVersionStream:fromDirectory:
		description: 'Load a package version'
		buttonLabel: 'Load')
		argumentGetter: [ :fileList | { fileList readOnlyStream . fileList directory } ]! !

!MCVersionReader class methodsFor: '*monticello-file services' stamp: 'tbn 8/11/2010 10:19'!
serviceMergeVersion
	^ (SimpleServiceEntry
		provider: self
		label: 'Merge version'
		selector: #mergeVersionStream:
		description: 'Merge a package version into the image'
		buttonLabel: 'Merge')
		argumentGetter: [ :fileList | fileList readOnlyStream ]! !

!MCVersionReader class methodsFor: '*monticello-file services' stamp: 'tbn 8/11/2010 10:20'!
serviceOpenVersion
	^ (SimpleServiceEntry
		provider: self
		label: 'Open version'
		selector: #openVersionFromStream:
		description: 'Open a package version'
		buttonLabel: 'Open')
		argumentGetter: [ :fileList | fileList readOnlyStream ]! !

!MCVersionReader class methodsFor: 'System-FileRegistry' stamp: 'avi 1/21/2004 22:55'!
services
	^ Array 
		with: self serviceLoadVersion
		with: self serviceMergeVersion
		with: self serviceOpenVersion! !

!MCVersionReader class methodsFor: 'System-FileRegistry' stamp: 'pavel.krivanek 10/24/2010 19:38'!
unload
	FileServices unregisterFileReader: self ! !

!MCVersionReader class methodsFor: 'reading' stamp: 'cwp 7/31/2003 23:03'!
versionFromFile: fileName
	^ self file: fileName streamDo: [:stream | self versionFromStream: stream]! !

!MCVersionReader class methodsFor: 'reading' stamp: 'avi 1/21/2004 22:58'!
versionFromStream: aStream
	^ (self on: aStream) version! !

!MCVersionReader class methodsFor: 'reading' stamp: 'avi 1/21/2004 22:59'!
versionInfoFromStream: aStream
	^ (self on: aStream) info! !

!MCVersionReader methodsFor: 'accessing' stamp: 'CamilloBruni 4/20/2012 18:18'!
basicVersion
	^ MCVersion
		package: self package
		info: self info
		snapshot: [ self snapshot ]
		dependencies: self dependencies! !

!MCVersionReader methodsFor: 'accessing' stamp: 'avi 1/21/2004 23:10'!
definitions
	definitions ifNil: [self loadDefinitions].
	^ definitions! !

!MCVersionReader methodsFor: 'accessing' stamp: 'avi 1/19/2004 14:50'!
dependencies
	dependencies ifNil: [self loadDependencies].
	^ dependencies! !

!MCVersionReader methodsFor: 'accessing' stamp: 'ab 8/20/2003 19:53'!
info
	info ifNil: [self loadVersionInfo].
	^ info! !

!MCVersionReader methodsFor: 'loading' stamp: 'ab 8/20/2003 19:54'!
loadDefinitions
	self subclassResponsibility ! !

!MCVersionReader methodsFor: 'loading' stamp: 'avi 1/19/2004 14:50'!
loadDependencies
	self subclassResponsibility ! !

!MCVersionReader methodsFor: 'loading' stamp: 'ab 8/20/2003 19:54'!
loadPackage
	self subclassResponsibility ! !

!MCVersionReader methodsFor: 'loading' stamp: 'ab 8/20/2003 19:54'!
loadVersionInfo
	self subclassResponsibility! !

!MCVersionReader methodsFor: 'accessing' stamp: 'ab 8/20/2003 19:53'!
package
	package ifNil: [self loadPackage].
	^ package! !

!MCVersionReader methodsFor: 'accessing' stamp: 'ab 8/20/2003 19:54'!
snapshot
	^ MCSnapshot fromDefinitions: self definitions! !

!MCVersionReader methodsFor: 'accessing' stamp: 'avi 10/9/2003 12:38'!
version
	^ self basicVersion! !

Object subclass: #MCRepository
	instanceVariableNames: 'creationTemplate storeDiffs'
	classVariableNames: 'Settings'
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

MCRepository subclass: #MCDictionaryRepository
	instanceVariableNames: 'description dict'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCDictionaryRepository methodsFor: 'as yet unclassified' stamp: 'ab 8/21/2003 12:56'!
= other
	^ self == other! !

!MCDictionaryRepository methodsFor: 'as yet unclassified' stamp: 'ab 8/20/2003 21:04'!
allVersionInfos
	^ dict values collect: [:ea | ea info]! !

!MCDictionaryRepository methodsFor: 'as yet unclassified' stamp: 'avi 8/26/2004 14:20'!
basicStoreVersion: aVersion
	dict at: aVersion info put: aVersion! !

!MCDictionaryRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
closestAncestorVersionFor: anAncestry ifNone: errorBlock
	| info |
	info := anAncestry breadthFirstAncestors
			detect: [:ea | self includesVersionWithInfo: ea]
			ifNone: [^ errorBlock value].
	^ self versionWithInfo: info! !

!MCDictionaryRepository methodsFor: 'as yet unclassified' stamp: 'ab 7/26/2003 02:47'!
description

	^ description ifNil: ['cache']! !

!MCDictionaryRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
description: aString

	description := aString ! !

!MCDictionaryRepository methodsFor: 'as yet unclassified' stamp: 'ab 7/26/2003 02:47'!
dictionary

	^ dict! !

!MCDictionaryRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
dictionary: aDictionary

	dict := aDictionary! !

!MCDictionaryRepository methodsFor: 'as yet unclassified' stamp: 'ab 7/21/2003 23:39'!
includesVersionNamed: aString
	^ dict anySatisfy: [:ea | ea info name = aString]! !

!MCDictionaryRepository methodsFor: 'as yet unclassified' stamp: 'ab 8/21/2003 19:49'!
includesVersionWithInfo: aVersionInfo
	^ dict includesKey: aVersionInfo! !

!MCDictionaryRepository methodsFor: 'as yet unclassified' stamp: 'alain.plantec 5/28/2009 10:02'!
initialize

	super initialize.
	dict := Dictionary new.
! !

!MCDictionaryRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
sortedVersionInfos
	| sorter |
	sorter := MCVersionSorter new.
	self allVersionInfos do: [:ea | sorter addVersionInfo: ea].
	^ sorter sortedVersionInfos
! !

!MCDictionaryRepository methodsFor: 'as yet unclassified' stamp: 'ab 8/16/2003 18:22'!
versionWithInfo: aVersionInfo ifAbsent: errorBlock
	^ dict at: aVersionInfo ifAbsent: errorBlock! !

MCRepository subclass: #MCFileBasedRepository
	instanceVariableNames: 'cache allFileNames cacheFileNames'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

MCFileBasedRepository subclass: #MCDirectoryRepository
	instanceVariableNames: 'directory'
	classVariableNames: 'DefaultDirectoryName'
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

MCDirectoryRepository subclass: #MCCacheRepository
	instanceVariableNames: 'packageCaches seenFiles cacheEnabled'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

MCCacheRepository class
	instanceVariableNames: 'default'!

MCCacheRepository class
	instanceVariableNames: 'default'!

!MCCacheRepository class methodsFor: 'accessing settings' stamp: 'SeanDeNigris 7/17/2012 15:34'!
cacheDirectory
	^self uniqueInstance directory asAbsolute
	! !

!MCCacheRepository class methodsFor: 'accessing settings' stamp: 'SeanDeNigris 7/17/2012 15:20'!
cacheDirectory: directory
	"directory - can be an absolutePathString (to interface with the settings framework), or aFileReference"
 
	default := self new directory: directory asFileReference.! !

!MCCacheRepository class methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 7/24/2012 10:50'!
default
	"This should be deleted as soon as Metacello is fixed"
	self flag: #toBeDeleted.			
	^ self uniqueInstance ! !

!MCCacheRepository class methodsFor: 'defaults' stamp: 'SeanDeNigris 7/17/2012 15:38'!
defaultDirectory
	^ 'package-cache' asFileReference 
		ensureDirectory;
		yourself! !

!MCCacheRepository class methodsFor: 'accessing' stamp: 'avi 10/9/2003 12:56'!
description
	^ nil! !

!MCCacheRepository class methodsFor: 'utility' stamp: 'SeanDeNigris 7/17/2012 15:34'!
disableCacheDuring: aBlock
	self uniqueInstance disableCacheDuring: aBlock! !

!MCCacheRepository class methodsFor: 'initialize-release' stamp: 'SeanDeNigris 7/17/2012 15:36'!
initialize
	self resetIfInvalid! !

!MCCacheRepository class methodsFor: 'private' stamp: 'SeanDeNigris 7/17/2012 15:36'!
resetIfInvalid
	"Reset if invalid"

	default notNil and: [default directory exists ifFalse: [default := nil]]! !

!MCCacheRepository class methodsFor: 'accessing' stamp: 'SeanDeNigris 7/17/2012 15:38'!
uniqueInstance
	self resetIfInvalid.
	^ default ifNil: [default := self new directory: self defaultDirectory]! !

!MCCacheRepository methodsFor: 'as yet unclassified' stamp: 'StephaneDucasse 8/17/2012 16:22'!
basicStoreVersion: aVersion
	"we store a version if it is cacheable and we don't have the file already"
	
	(aVersion isCacheable and: [(self includesFileNamed: aVersion fileName) not])
		ifTrue: [super basicStoreVersion: aVersion]
! !

!MCCacheRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
cacheForPackage: aPackage
	packageCaches ifNil: [packageCaches := Dictionary new].
	^ packageCaches at: aPackage ifAbsentPut: [MCPackageCache new]! !

!MCCacheRepository methodsFor: 'utility' stamp: 'CamilloBruni 7/6/2012 16:20'!
disable
	cacheEnabled := false.! !

!MCCacheRepository methodsFor: 'utility' stamp: 'CamilloBruni 7/6/2012 16:20'!
disableCacheDuring: aBlock
	self disable.
	aBlock ensure: [ self enable ].! !

!MCCacheRepository methodsFor: 'utility' stamp: 'CamilloBruni 7/6/2012 16:20'!
enable
	cacheEnabled := true! !

!MCCacheRepository methodsFor: 'initialize-release' stamp: 'CamilloBruni 7/6/2012 16:20'!
initialize
	cacheEnabled := true.
	super initialize.! !

!MCCacheRepository methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 2/28/2012 20:10'!
loadVersionFromFileNamed: aString
	
	^ self versionReaderForFileNamed: aString do: [:r | r version]! !

!MCCacheRepository methodsFor: 'as yet unclassified' stamp: 'avi 1/22/2004 18:15'!
newFileNames 
	^ self allFileNames difference: self seenFileNames! !

!MCCacheRepository methodsFor: 'as yet unclassified' stamp: 'avi 1/22/2004 15:13'!
packageForFileNamed: aString
	^ self packageCache at: aString ifAbsentPut: [self versionReaderForFileNamed: aString do: [:r | r package]]! !

!MCCacheRepository methodsFor: 'as yet unclassified' stamp: 'avi 1/22/2004 18:29'!
rescan
	self newFileNames do:
		[:ea |
		self versionReaderForFileNamed: ea do:
			[:reader |
			(self cacheForPackage: reader package)
				recordVersionInfo: reader info
				forFileNamed: ea.
			self seenFileNames add: ea]]
		displayingProgress: 'Scanning cache...'! !

!MCCacheRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
seenFileNames
	^ seenFiles ifNil: [seenFiles := OrderedCollection new]! !

!MCCacheRepository methodsFor: 'as yet unclassified' stamp: 'MarcusDenker 7/6/2012 16:59'!
storeVersion: aVersion
	cacheEnabled == true ifFalse: [ ^ self ].
	(self includesFileNamed: aVersion fileName) ifTrue: [ ^ self ].
	^ super storeVersion: aVersion.! !

!MCCacheRepository methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 7/6/2012 16:13'!
versionInfoForFileNamed: aString
	^ self infoCache 
		at: aString 
		ifAbsentPut: [ self versionReaderForFileNamed: aString do: [:r | r info]]! !

!MCCacheRepository methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 4/20/2012 18:10'!
versionWithInfo: aVersionInfo ifAbsent: errorBlock
	(self allFileNamesForVersionNamed: aVersionInfo name) do: [:fileName | | version |
			version := self versionFromFileNamed: fileName.
			version info = aVersionInfo ifTrue: [^ version]].
	^ errorBlock value! !

!MCDirectoryRepository class methodsFor: 'instance creation' stamp: 'CamilloBruni 5/4/2012 21:35'!
defaultDirectoryName
	^ DefaultDirectoryName ifNil: [
		DefaultDirectoryName :=  FileSystem workingDirectory fullName].! !

!MCDirectoryRepository class methodsFor: 'instance creation' stamp: 'SeanDeNigris 4/21/2010 14:38'!
defaultDirectoryName: aDirectoryName
	DefaultDirectoryName := aDirectoryName.! !

!MCDirectoryRepository class methodsFor: 'instance creation' stamp: 'ab 7/24/2003 21:20'!
description
	^ 'directory'! !

!MCDirectoryRepository class methodsFor: 'instance creation' stamp: 'SeanDeNigris 7/12/2012 08:45'!
morphicConfigure
	^ (UIManager default chooseDirectoryFrom: self defaultDirectoryName asFileReference) ifNotNil:
		[:directory |
		self new directory: directory]! !

!MCDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 2/28/2012 20:08'!
allFileNamesForVersionNamed: aString
	"avoid slow default implementation and directly check for existing files"
	|extensions|
	extensions := MCReader concreteSubclasses collect: [ :class| class extension ].
	^ extensions 
		collect: [ :extension| aString , '.', extension ]
		thenSelect: [ :fileName| self includesFileNamed: fileName ]! !

!MCDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 5/4/2012 19:04'!
description
	^ directory fullName! !

!MCDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'ab 7/6/2003 17:49'!
directory
	^ directory! !

!MCDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
directory: aDirectory
	directory := aDirectory! !

!MCDirectoryRepository methodsFor: 'comparing' stamp: 'CamilloBruni 5/24/2012 09:36'!
hash
	^ directory hash! !

!MCDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'SeanDeNigris 6/18/2012 15:58'!
includesFileNamed: aString
	"HACK: speed up the cache hits"
	^ (directory / aString) exists.! !

!MCDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 2/28/2012 20:08'!
includesVersionNamed: aString
	"avoid slow default implementation and directly check for existing files"
	^ (self allFileNamesForVersionNamed: aString) isEmpty not! !

!MCDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 5/4/2012 21:35'!
initialize
	super initialize.
	directory := FileSystem workingDirectory! !

!MCDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'nk 11/2/2003 10:55'!
isValid
	^directory exists! !

!MCDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'StephaneDucasse 6/17/2012 18:51'!
loadAllFileNames
	^ (directory entries sort: [:a :b | a modificationTime >= b modificationTime]) collect: [:ea | ea basename]! !

!MCDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'SeanDeNigris 6/29/2012 15:49'!
readStreamForFileNamed: aString do: aBlock
	| val |
	directory / aString readStreamDo: [ :stream|
		val := aBlock value: stream ].
	^ val! !

!MCDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'SeanDeNigris 6/29/2012 15:50'!
writeStreamForFileNamed: aString replace: shouldReplace do: aBlock

	| file |
	file := directory / aString.

	shouldReplace ifTrue: [ file delete ].

	file writeStreamDo:  [ :stream | 
		aBlock value: stream ].! !

MCDirectoryRepository subclass: #MCSubDirectoryRepository
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCSubDirectoryRepository commentStamp: 'nk 6/11/2004 18:56' prior: 0!
A MCDirectoryRepository that looks in subdirectories too.!

!MCSubDirectoryRepository class methodsFor: 'user interface' stamp: 'nk 6/11/2004 18:48'!
description
	^ 'directory with subdirectories'! !

!MCSubDirectoryRepository methodsFor: 'enumeration' stamp: 'EstebanLorenzano 8/17/2012 13:12'!
allDirectories
	^directory allDirectories ! !

!MCSubDirectoryRepository methodsFor: 'user interface' stamp: 'CamilloBruni 5/4/2012 19:04'!
description
	^ (directory / '*') fullName
! !

!MCSubDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'EstebanLorenzano 8/17/2012 13:43'!
findFullNameForReading: aBaseName
	"Answer the latest version of aBaseName"
	^((directory glob: [ :each | each basename = aBaseName])
		sorted: [ :a :b | a modificationTime < b modificationTime ])
		first.! !

!MCSubDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'SeanDeNigris 2/5/2013 15:21'!
findFullNameForWriting: aBaseName
	| possible split prefix fpattern now |
	split := self splitNameVersionExtensionFor: aBaseName.
	fpattern := split first, '*'.
	possible := SortedCollection sortBlock: [ :a :b |
		a first = b first
			ifTrue: [ a second = b second
					ifFalse: [ a second < b second ]
					ifTrue: [ a third fullName size < b third fullName size ]]
			ifFalse: [ a first > b first ] ].
	now := Time totalSeconds.
	prefix := directory pathSegments size.
	self allDirectories do: [:dir | | dirScore fileScore parts |
		parts := dir pathSegments allButFirst: prefix.
		dirScore := (parts select: [ :part | fpattern match: part ]) size.
		fileScore := (dir entries collect: [ :ent |
			(ent isDirectory not and: [ fpattern match: ent name ])
				ifFalse: [ SmallInteger maxVal ]
				ifTrue: [ now - ent modificationTime ]]).	"minimum age"
		fileScore := fileScore isEmpty ifTrue: [ SmallInteger maxVal  ]
			ifFalse: [ fileScore min ].
		possible add: { dirScore. fileScore. dir } ].
	
	^ (possible first third) / aBaseName! !

!MCSubDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'StephaneDucasse 6/17/2012 18:52'!
loadAllFileNames
	"sorting {entry. dirName. name}"

	| sorted |
	sorted := SortedCollection sortBlock: [:a :b |
		a first modificationTime >= b first modificationTime ].
	self allDirectories
		do: [:dir | dir entries
				do: [:ent | ent isDirectory
						ifFalse: [sorted add: {ent. dir fullName. ent basename}]]].
	^ sorted
		collect: [:ea | ea third ]! !

!MCSubDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
readStreamForFileNamed: aString do: aBlock
	| file val |
	file := FileStream readOnlyFileNamed: (self findFullNameForReading: aString).
	val := aBlock value: file.
	file close.
	^ val! !

!MCSubDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'SeanDeNigris 2/5/2013 15:35'!
splitNameVersionExtensionFor: aBaseName

	| file version |
	file := aBaseName asFileReference.
	version := (file extensions at: file extensions size - 1) asNumber.
	^ { file base. version. file extension }.
	
	
	! !

!MCSubDirectoryRepository methodsFor: 'as yet unclassified' stamp: 'nk 6/11/2004 20:34'!
writeStreamForFileNamed: aString replace: aBoolean do: aBlock 
	| file |
	file := aBoolean
				ifTrue: [FileStream
						forceNewFileNamed: (self findFullNameForReading: aString)]
				ifFalse: [FileStream
						newFileNamed: (self findFullNameForWriting: aString)].
	aBlock value: file.
	file close! !

!MCFileBasedRepository class methodsFor: 'cleanup' stamp: 'StephaneDucasse 3/9/2010 16:33'!
cleanUp
	"Flush caches"

	self flushAllCaches.! !

!MCFileBasedRepository class methodsFor: 'as yet unclassified' stamp: 'avi 2/3/2005 00:43'!
flushAllCaches
	self allSubInstancesDo: [:ea | ea flushCache]! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 4/20/2012 17:57'!
allFileNames
	cacheFileNames == true ifFalse: [ ^ self loadAllFileNames ].
	^ allFileNames ifNil: [
			allFileNames := self loadAllFileNames]! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'ab 8/21/2003 20:01'!
allFileNamesForVersionNamed: aString
	^ self filterFileNames: self readableFileNames forVersionNamed: aString! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 3/2/2012 13:01'!
allFileNamesOrCache
	cacheFileNames ifFalse: [ ^ self allFileNames ].
	^ allFileNames ifNil: [
			allFileNames := self allFileNames]! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'ab 8/21/2003 00:40'!
allVersionNames
	^ self readableFileNames collect: [:ea | self versionNameFromFileName: ea]! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
basicStoreVersion: aVersion
	self
		writeStreamForFileNamed: aVersion fileName
		do: [:s | aVersion fileOutOn: s].
	aVersion isCacheable ifTrue: [
		cache ifNil: [cache := Dictionary new].
		cache at: aVersion fileName put: aVersion].
! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
cache
	^ cache ifNil: [cache := Dictionary new]! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 3/2/2012 13:01'!
cacheAllFileNamesDuring: aBlock
	cacheFileNames == true ifTrue: [ ^ aBlock value ].
	allFileNames := nil.
	cacheFileNames := true.	
	^ aBlock ensure: [
		allFileNames := nil.
		cacheFileNames := false]! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'bf 6/9/2005 15:47'!
cachedFileNames
	^cache == nil
		ifTrue: [#()]
		ifFalse: [cache keys]! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
canReadFileNamed: aString
	| reader |
	reader := MCVersionReader readerClassForFileNamed: aString.
	^ reader notNil! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'avi 9/17/2005 16:40'!
closestAncestorVersionFor: anAncestry ifNone: errorBlock
	^ self cacheAllFileNamesDuring:
		[super closestAncestorVersionFor: anAncestry ifNone: errorBlock]! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'ab 8/21/2003 20:01'!
filterFileNames: aCollection forVersionNamed: aString
	^ aCollection select: [:ea | (self versionNameFromFileName: ea) = aString] ! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
flushCache
	cache := nil! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 1/26/2012 19:49'!
includesFileNamed: aString
	"slow default implementation"
	^ self allFileNames includes: aString! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'ab 8/21/2003 00:36'!
includesVersionNamed: aString
	^ self allVersionNames includes: aString! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 3/2/2012 13:03'!
loadAllFileNames
	self subclassResponsibility! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'SeanDeNigris 7/17/2012 15:50'!
loadVersionFromFileNamed: aString

	(MCCacheRepository uniqueInstance includesFileNamed: aString)
		ifTrue: [ ^ MCCacheRepository uniqueInstance loadVersionFromFileNamed: aString].
	
	^ self versionReaderForFileNamed: aString do: [:r | r version]! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'avi 9/17/2005 18:37'!
loadVersionInfoFromFileNamed: aString
	^ self versionReaderForFileNamed: aString do: [:r | r info]
	! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 4/20/2012 18:14'!
maxCacheSize
	^ 512! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 1/26/2012 19:50'!
notifyList
	(self includesFileNamed: 'notify') ifFalse: [^ #()].
	^ self readStreamForFileNamed: 'notify' do:
		[:s | s upToEnd  lines]! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'bf 3/11/2005 18:01'!
possiblyNewerVersionsOfAnyOf: someVersions
	| pkgs |
	pkgs := Dictionary new.

	someVersions do: [:aVersionInfo |
		pkgs at: (aVersionInfo name copyUpToLast: $-)
			put: (aVersionInfo name copyAfterLast: $.) asNumber].

	^[self allVersionNames select: [:each |
		(pkgs at: (each copyUpToLast: $-) ifPresent: [:verNumber |
			verNumber < (each copyAfterLast: $.) asNumber
				or: [verNumber = (each copyAfterLast: $.) asNumber
					and: [someVersions noneSatisfy: [:v | v name = each]]]]) == true]
	] on: Error do: [:ex | ex return: #()]! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'StephaneDucasse 1/24/2013 12:06'!
readableFileNames
	| all cached new emptyFilenamelength |
	"<hyphenated-package-name>.<dotted.branch.tag>-<initials>.<count>.mcz"
	emptyFilenamelength := 'P-i.c.mcz' size.
	all := self allFileNames.	"from repository"
	all := all reject: [ :each |  each size < emptyFilenamelength].
		"first stupid way to filter first level broken files. Ideally we should 
		remove any files not following the naming pattern: PackageName-author.number[(branch)].mcz"
	cached := self cachedFileNames.	"in memory"
	new := all difference: cached.
	^ (cached asArray, new)
		select: [:ea | self canReadFileNamed: ea]! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'avi 9/18/2005 22:43'!
resizeCache: aDictionary
	[aDictionary size <= self maxCacheSize] whileFalse:
		[aDictionary removeKey: aDictionary keys atRandom]! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 4/21/2012 18:01'!
versionFromFileNamed: aString
	| v |
	v := self cache
			 at: aString 
			 ifAbsent: [ self loadVersionFromFileNamed: aString ].
	self resizeCache: cache.
	(v notNil and: [v isCacheable]) ifTrue: [cache at: aString put: v].
	^ v! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'avi 9/17/2005 18:37'!
versionInfoFromFileNamed: aString
	self cache at: aString ifPresent: [:v | ^ v info].
	^ self loadVersionInfoFromFileNamed: aString! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'bf 5/30/2005 22:52'!
versionNameFromFileName: aString
	^ (aString copyUpToLast: $.) copyUpTo: $(! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'marcus.denker 11/10/2008 10:04'!
versionReaderForFileNamed: aString do: aBlock
	^ self
		readStreamForFileNamed: aString
		do: [:s |
			(MCVersionReader readerClassForFileNamed: aString) ifNotNil:
				[:class | aBlock value: (class on: s fileName: aString)]]
! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'SeanDeNigris 7/17/2012 15:40'!
versionWithInfo: aVersionInfo ifAbsent: errorBlock
	"get a version for the given versionInfo. always query first the packageCache and only then try to load the version from the remote location"
	^ MCCacheRepository uniqueInstance
		versionWithInfo: aVersionInfo 
		ifAbsent: [
			(self allFileNamesForVersionNamed: aVersionInfo name) do:
				[:fileName | | version |
				version := self versionFromFileNamed: fileName.
				version info = aVersionInfo ifTrue: [^ version]].
			^ errorBlock value].! !

!MCFileBasedRepository methodsFor: 'as yet unclassified' stamp: 'avi 10/31/2003 14:32'!
writeStreamForFileNamed: aString do: aBlock
	^ self writeStreamForFileNamed: aString replace: false do: aBlock! !

MCFileBasedRepository subclass: #MCFtpRepository
	instanceVariableNames: 'host directory user password connection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCFtpRepository class methodsFor: 'as yet unclassified' stamp: 'avi 9/16/2003 13:57'!
creationTemplate
	^
'MCFtpRepository
	host: ''modules.squeakfoundation.org''
	directory: ''mc''
	user: ''squeak''
	password: ''squeak'''
	! !

!MCFtpRepository class methodsFor: 'as yet unclassified' stamp: 'avi 9/16/2003 13:57'!
description
	^ 'FTP'! !

!MCFtpRepository class methodsFor: 'as yet unclassified' stamp: 'avi 9/16/2003 13:57'!
host: host directory: directory user: user password: password
	^ self new
		host: host;
		directory: directory;
		user: user;
		password: password! !

!MCFtpRepository class methodsFor: 'as yet unclassified' stamp: 'bkv 2/18/2004 20:38'!
templateCreationSelector
	^ #host:directory:user:password: ! !

!MCFtpRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
clientDo: aBlock
	| client |
	client := FTPClient openOnHostNamed: host.
	client loginUser: user password: password.
	directory isEmpty ifFalse: [client changeDirectoryTo: directory].
	^ [aBlock value: client] ensure: [client close]! !

!MCFtpRepository methodsFor: 'required' stamp: 'avi 9/17/2003 12:52'!
description
	^ 'ftp://', user, '@', host, '/', directory! !

!MCFtpRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
directory: dirPath
	directory := dirPath! !

!MCFtpRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
host: hostname
	host := hostname! !

!MCFtpRepository methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 3/2/2012 13:02'!
loadAllFileNames
	^ self clientDo:
		[:client |
		self parseDirectoryListing: client getDirectory]! !

!MCFtpRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
parseDirectoryListing: aString
	| stream files line tokens |
	stream := aString readStream.
	files := OrderedCollection new.
	[stream atEnd] whileFalse:
		[line := stream nextLine.
		tokens := line findTokens: ' '.
		tokens size > 2 ifTrue: [files add: tokens last]].
	^ files! !

!MCFtpRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
password: passwordString
	password := passwordString! !

!MCFtpRepository methodsFor: 'required' stamp: 'nice 1/5/2010 15:59'!
readStreamForFileNamed: aString do: aBlock
	
	^ self clientDo:
		[:client | | stream |
		client binary.
		stream := RWBinaryOrTextStream on: String new.
		stream nextPutAll: (client getFileNamed: aString).
		aBlock value: stream reset]! !

!MCFtpRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
user: userString
	user := userString! !

!MCFtpRepository methodsFor: 'required' stamp: 'stephaneducasse 2/4/2006 20:47'!
writeStreamForFileNamed: aString replace: ignoreBoolean do: aBlock
	| stream |
	stream := RWBinaryOrTextStream on: String new.
	aBlock value: stream.
	self clientDo:
		[:client |
		client binary.
		client putFileStreamContents: stream reset as: aString]! !

MCFileBasedRepository subclass: #MCHttpRepository
	instanceVariableNames: 'location user password readerCache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

MCHttpRepository subclass: #MCGemstoneRepository
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCGemstoneRepository commentStamp: 'CamilloBruni 2/8/2012 18:10' prior: 0!
An explicit subclass of MCHttpRepository to provide a template for gemstone repositories.!

!MCGemstoneRepository class methodsFor: 'accessing' stamp: 'CamilloBruni 10/21/2012 13:24'!
baseURL
	^ 'http://ss3.gemstone.com/ss/'! !

!MCGemstoneRepository class methodsFor: 'accessing' stamp: 'CamilloBruni 2/8/2012 18:12'!
description
	^ 'ss3.gemstone.com'! !

!MCGemstoneRepository methodsFor: 'interface' stamp: 'CamilloBruni 4/23/2012 20:22'!
includesVersionNamed: aString
	"directly do a filename check since squeaksource only stores mcz"
	^ self includesFileNamed: aString, '.mcz'! !

!MCHttpRepository class methodsFor: 'accessing' stamp: 'CamilloBruni 10/21/2012 13:25'!
baseURL
	^ ''! !

!MCHttpRepository class methodsFor: 'initialization' stamp: 'bf 7/28/2005 19:44'!
clearPasswords
	self allSubInstancesDo: [:ea | ea password: ''].
! !

!MCHttpRepository class methodsFor: 'creation template' stamp: 'CamilloBruni 10/21/2012 13:24'!
creationTemplate
	^self creationTemplateLocation: self baseURL
		user: ''
		password: ''
! !

!MCHttpRepository class methodsFor: 'creation template' stamp: 'CamilloBruni 2/15/2012 15:21'!
creationTemplateLocation: location user: user password: password
	^
self name, '
	location: {1}
	user: {2}
	password: {3}' format: {location printString. user printString. password printString}! !

!MCHttpRepository class methodsFor: 'accessing' stamp: 'ab 7/24/2003 21:20'!
description
	^ 'HTTP'! !

!MCHttpRepository class methodsFor: 'instance creation' stamp: 'SeanDeNigris 8/26/2012 15:50'!
inboxRepository
	^ self location: 'http://ss3.gemstone.com/ss/PharoInbox/'.! !

!MCHttpRepository class methodsFor: 'instance creation' stamp: 'SeanDeNigris 8/26/2012 18:20'!
location: location

	^ MCServerRegistry uniqueInstance repositoryAt: location credentialsDo: [ :username :password |
			self new
				location: location;
				user: username;
				password: password ].! !

!MCHttpRepository class methodsFor: 'instance creation' stamp: 'SeanDeNigris 8/26/2012 18:22'!
location: aUrl user: user password: password

	| result |
	result := self location: aUrl.
	user ifNotEmpty: [
		result
			user: user;
			password: password ].
	^ result.! !

!MCHttpRepository class methodsFor: 'instance creation' stamp: 'CamilloBruni 10/21/2012 13:26'!
project: aProjectIdentifier
	^ self location: self baseURL, aProjectIdentifier! !

!MCHttpRepository methodsFor: 'converting' stamp: 'CamilloBruni 9/14/2012 17:24'!
asCreationTemplate
	^self class creationTemplateLocation: self location user: user password: password! !

!MCHttpRepository methodsFor: 'private' stamp: 'SeanDeNigris 8/27/2012 12:09'!
assertBinaryResponse: response 

	response contentType isBinary ifFalse: [ 
		MCRepositoryError signal: 'Expected a binary response instead of ', response contentType printString ].! !

!MCHttpRepository methodsFor: 'private' stamp: 'SeanDeNigris 8/27/2012 12:05'!
assertNonBinaryResponse: response 
	
	response contentType isBinary ifTrue: [ MCRepositoryError signal: 'Did not expect a binary response but got ', response contentType printString ].! !

!MCHttpRepository methodsFor: 'i/o' stamp: 'SeanDeNigris 7/17/2012 15:50'!
cacheRawVersionNamed: aString stream: contents
	"directly forward the contents to the cache repository. 
	this avoids and unnecessary serialization step"
	MCCacheRepository uniqueInstance 
		writeStreamForFileNamed: aString 
		replace: true 
		do: [ :s| s nextPutAll: contents ]! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'CamilloBruni 9/14/2012 17:24'!
description
	^ self location! !

!MCHttpRepository methodsFor: 'i/o' stamp: 'SeanDeNigris 6/20/2012 23:28'!
displayProgress: label during: workBlock
	| nextUpdateTime |
	nextUpdateTime := 0.
	^UIManager default displayProgress: label 
		from: 0.0 to: 1.0 during:[:bar|
			[workBlock value] on: HTTPProgress do:[:ex|
				(ex total == nil or: [ex amount == nil]) ifFalse:[
					(nextUpdateTime < Time millisecondClockValue 
						or:[ex total = ex amount]) ifTrue:[
							bar current: ex amount asFloat / ex total asFloat.
							nextUpdateTime := Time millisecondClockValue + 100.
					].
				].
				ex resume.
			]
		].
! !

!MCHttpRepository methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/5/2012 21:54'!
entityStreamContents: aBlock
	"Generate output in a buffer because we need the length"
	
	| stream |
	stream := RWBinaryOrTextStream on: String new.
	aBlock value: stream.
	stream reset.
	^ (ZnStreamingEntity type: ZnMimeType applicationOctetStream)
		stream: stream;
		contentLength: stream size;
		yourself! !

!MCHttpRepository methodsFor: 'actions' stamp: 'al 12/12/2005 11:06'!
flushCache
	super flushCache.
	readerCache := nil.! !

!MCHttpRepository methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/15/2013 14:59'!
httpClient
	"Return a new, specifically configured instance of the HTTP client for internal use.
	Note how we request GZIP compression and will signal progress."

	^ ZnClient new
		systemPolicy;
		beOneShot;
		username: self user password: self password;
		setAcceptEncodingGzip;
		signalProgress: true;
		yourself! !

!MCHttpRepository methodsFor: 'i/o' stamp: 'CamilloBruni 9/27/2012 20:59'!
loadAllFileNames
	| client |
	self displayProgress: 'Loading all file names from ', self description during: [
		(client := self httpClient)
			ifFail: [ :exception | 
				(exception className beginsWith: 'Zn')
					ifTrue: [ MCRepositoryError signal: 'Could not access ', self location, ': ', exception printString ]
					ifFalse: [ exception pass ] ];
			url: self locationWithTrailingSlash;
			queryAt: 'C' put: 'M;O=D';	"legacy that some servers maybe expect"
			get.
		self assertNonBinaryResponse: client response ].
	^ self parseFileNamesFromStream: client contents readStream! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'CamilloBruni 9/14/2012 17:24'!
location
	^ location! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
location: aUrlString
	location := aUrlString! !

!MCHttpRepository methodsFor: 'actions' stamp: 'CamilloBruni 9/14/2012 17:24'!
locationWithTrailingSlash
	^ (self location endsWith: '/')
		ifTrue: [self location]
		ifFalse: [self location, '/']! !

!MCHttpRepository methodsFor: 'actions' stamp: 'stephaneducasse 2/4/2006 20:47'!
parseFileNamesFromStream: aStream
	| names fullName |
	names := OrderedCollection new.
	[aStream atEnd] whileFalse:
		[[aStream upTo: $<. {$a. $A. nil} includes: aStream next] whileFalse.
		aStream upTo: $".
		aStream atEnd ifFalse: [
			fullName := aStream upTo: $".
			names add: fullName unescapePercents]].
	^ names! !

!MCHttpRepository methodsFor: 'actions' stamp: 'DamienCassou 2/14/2013 11:40'!
password
	self userAndPasswordFromSettingsDo: [:usr :pwd | ^pwd].

	self user isEmpty ifTrue: [^password ifNil: ['']].

	password isEmptyOrNil ifTrue: [ 
		password := (UsersManager default currentUser userNamePasswordFor: self keyChainGroupSelector) 							ifNil: [ '' ]
							ifNotNil: [:usr | usr password ]].


	password isEmptyOrNil ifTrue: [
		| answer |
		user isEmptyOrNil ifTrue: [
			"Give the user a chance to change the login"
			answer := UIManager default request: 'User name for ', String cr, location
				initialAnswer: ''.
			answer isEmptyOrNil
				ifTrue: [^password]
				ifFalse: [self user: answer] ].
		
		password := UIManager default requestPassword: 'Password for "', self user, '" at ', String cr, self location.
		"The user doesn't care about logging into this repo. Clear the username so they will not be prompted every time"
		password isEmptyOrNil ifTrue: [ user := '' ].
	].

	^ password! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
password: passwordString
	password := passwordString! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'CamilloBruni 9/18/2012 18:35'!
project
	"Return a project name"
	^ (self location splitOn: $/) last! !

!MCHttpRepository methodsFor: 'i/o' stamp: 'EstebanLorenzano 4/25/2013 10:54'!
readStreamForFileNamed: aString do: aBlock
	| client |
	self displayProgress: 'Downloading ', aString during: [
		client := self httpClient.
		client
			ifFail: [ :exception | self error: 'Could not load ', aString, ': ', exception printString ];
			get: (self urlForFileNamed: aString).
		self assertBinaryResponse: client response.
		"immediately cache the version and avoid an unnecessary serialization"
		self cacheRawVersionNamed: aString stream: client contents ].
	^ aBlock value: client contents readStream! !

!MCHttpRepository methodsFor: 'actions' stamp: 'ab 8/21/2003 13:08'!
urlForFileNamed: aString
	^ self locationWithTrailingSlash, aString encodeForHTTP! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'PavelKrivanek 2/8/2013 13:21'!
user
	self userAndPasswordFromSettingsDo: [:usr :pwd | ^usr].
	"not in settings"
	user isEmptyOrNil ifFalse: [ ^user ].
	
	^ Smalltalk globals at: #UsersManager 
		ifPresent: [:usersManager |
			(usersManager default currentUser userNamePasswordFor: self keyChainGroupSelector)
				ifNotNil: [ :usr | usr username ]		
				ifNil: [ '' ]]
		ifAbsent: [ '' ]! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
user: userString
	user := userString! !

!MCHttpRepository methodsFor: 'actions' stamp: 'CamilloBruni 9/14/2012 17:25'!
userAndPasswordFromSettingsDo: aBlock
	"The mcSettings file in ExternalSettings preferenceDirectory should contain entries for each account:
	
		account1: *myhost.mydomain* user:password
		account2: *otherhost.mydomain/somerep* dXNlcjpwYXNzd29yZA==

	That is it must start with 'account', followed by anything to distinguish accounts, and a colon. Then comes a match expression for the repository url, and after a space the user:password string.
	
	To not have the clear text password on your disc, you can base64 encode it:
			(Base64MimeConverter mimeEncode: 'user:password' readStream) contents
	"

	
	Settings ifNotNil: [
		Settings keysAndValuesDo: [:key :value | | entry userAndPassword |
			(key asLowercase beginsWith: 'account') ifTrue: [
				entry := value findTokens: '	 '.
				(entry first match: self location) ifTrue: [
					userAndPassword := entry second.
					(userAndPassword includes: $:) ifFalse: [
						userAndPassword := (Base64MimeConverter mimeDecodeToChars: userAndPassword readStream) contents].
					userAndPassword := userAndPassword findTokens: $:.
					^aBlock value: userAndPassword first 
						value: userAndPassword second 
					]
			]
		]
	].
	^nil! !

!MCHttpRepository methodsFor: 'actions' stamp: 'stephaneducasse 2/4/2006 20:47'!
versionReaderForFileNamed: aString
	readerCache ifNil: [readerCache := Dictionary new].
	^ readerCache at: aString ifAbsent:
		[self resizeCache: readerCache.
		super versionReaderForFileNamed: aString do:
			[:r |
			r ifNotNil: [readerCache at: aString put: r]]]
	! !

!MCHttpRepository methodsFor: 'actions' stamp: 'marcus.denker 11/10/2008 10:04'!
versionReaderForFileNamed: aString do: aBlock
	^ (self versionReaderForFileNamed: aString) ifNotNil: aBlock! !

!MCHttpRepository methodsFor: 'i/o' stamp: 'SvenVanCaekenberghe 6/5/2012 21:52'!
writeStreamForFileNamed: aString replace: ignoreBoolean do: aBlock
	| entity |
	entity := self entityStreamContents: aBlock.
	self displayProgress: 'Uploading ', aString during: [
		self httpClient
			entity: entity;
			ifFail: [ :exception | self error: 'Could not save ', aString, ': ', exception printString ];
			url: (self urlForFileNamed: aString);
			put ]! !

MCHttpRepository subclass: #MCSmalltalkhubRepository
	instanceVariableNames: 'owner project'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCSmalltalkhubRepository class methodsFor: 'accessing' stamp: 'CamilloBruni 10/21/2012 13:25'!
baseURL
	^ 'http://smalltalkhub.com/mc/'! !

!MCSmalltalkhubRepository class methodsFor: 'creation template' stamp: 'CamilloBruni 9/14/2012 17:15'!
creationTemplate
	^self creationTemplateOwner: ''
		project: ''
		user: ''
		password: ''
! !

!MCSmalltalkhubRepository class methodsFor: 'creation template' stamp: 'CamilloBruni 9/14/2012 17:23'!
creationTemplateOwner: owner project: project user: user password: password
	^ String streamContents: [ :s|
		s 
			nextPutAll: self name; cr;
			tab; nextPutAll: 'owner: '; print: owner; cr;
			tab; nextPutAll: 'project: '; print: project; cr;
			tab; nextPutAll: 'user: '; print: user; cr;
			tab; nextPutAll: 'password: '; print: password ].! !

!MCSmalltalkhubRepository class methodsFor: 'accessing' stamp: 'CamilloBruni 9/14/2012 16:38'!
description
	^ 'smalltalkhub.com'! !

!MCSmalltalkhubRepository class methodsFor: 'instance creation' stamp: 'CamilloBruni 10/9/2012 14:54'!
owner: owner project: project 
	^ self
		owner: owner
		project: project
		user: String empty
		password: String empty! !

!MCSmalltalkhubRepository class methodsFor: 'instance creation' stamp: 'CamilloBruni 9/14/2012 17:22'!
owner: owner project: project user: user password: password
	^ self new
		owner: owner;
		project: project;
		user: user;
		password: password;
		yourself! !

!MCSmalltalkhubRepository methodsFor: 'converting' stamp: 'CamilloBruni 9/14/2012 17:20'!
asCreationTemplate
	^self class 
		creationTemplateOwner: self owner project: self project user: user password: password! !

!MCSmalltalkhubRepository methodsFor: 'interface' stamp: 'CamilloBruni 9/14/2012 17:11'!
includesFileNamed: aString
	"avoid the slower default method and simply do a head request "
	self httpClient
		numberOfRetries: 0;
		ifFail: [ :exception| 
			exception response code = 404
				ifTrue: [ ^ false ].
			exception pass];
		head: (self urlForFileNamed: aString).
	^ true! !

!MCSmalltalkhubRepository methodsFor: 'interface' stamp: 'CamilloBruni 9/14/2012 16:37'!
includesVersionNamed: aString
	"directly do a filename check since squeaksource only stores mcz"
	^ self includesFileNamed: aString, '.mcz'! !

!MCSmalltalkhubRepository methodsFor: 'interface' stamp: 'CamilloBruni 9/14/2012 17:25'!
loadAllFileNames
	| client |
	(client := self httpClient)
		ifFail: [ :exception | self error: 'Could not access ', self location, ': ', exception printString ];
		url: self locationWithTrailingSlash;
		queryAt: 'format' put: 'raw';
		get.
	self assertNonBinaryResponse: client response.
	^ self parseFileNamesFromStream: client contents! !

!MCSmalltalkhubRepository methodsFor: 'accessing' stamp: 'CamilloBruni 9/14/2012 17:25'!
location
	^ 'http://smalltalkhub.com/mc/', self owner, '/', self project, '/main/' ! !

!MCSmalltalkhubRepository methodsFor: 'accessing' stamp: 'CamilloBruni 9/14/2012 17:24'!
locationWithTrailingSlash
	^ self location! !

!MCSmalltalkhubRepository methodsFor: 'accessing' stamp: 'CamilloBruni 9/14/2012 17:21'!
owner
	
	^ owner! !

!MCSmalltalkhubRepository methodsFor: 'accessing' stamp: 'CamilloBruni 9/18/2012 18:36'!
owner: aString
	
	owner := aString! !

!MCSmalltalkhubRepository methodsFor: 'interface' stamp: 'CamilloBruni 10/15/2012 13:36'!
parseFileNamesFromStream: aNewLineDelimitedString
	^ aNewLineDelimitedString 
		ifNil: [ ^ OrderedCollection new ]
		ifNotNil: [ aNewLineDelimitedString subStrings: String crlf ]! !

!MCSmalltalkhubRepository methodsFor: 'accessing' stamp: 'CamilloBruni 9/14/2012 17:21'!
project
	
	^ project! !

!MCSmalltalkhubRepository methodsFor: 'accessing' stamp: 'CamilloBruni 9/18/2012 18:36'!
project: aString
	
	project := aString! !

MCHttpRepository subclass: #MCSqueaksourceRepository
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCSqueaksourceRepository class methodsFor: 'creation template' stamp: 'CamilloBruni 10/21/2012 13:25'!
baseURL
	^ 'http://squeaksource.com/'! !

!MCSqueaksourceRepository class methodsFor: 'accessing' stamp: 'CamilloBruni 2/8/2012 18:13'!
description
	^ 'squeaksource.com'! !

!MCSqueaksourceRepository methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 9/14/2012 17:25'!
asCreationTemplate
	^self class creationTemplateLocation: self location user: user password: password! !

!MCSqueaksourceRepository methodsFor: 'interface' stamp: 'CamilloBruni 9/14/2012 17:10'!
includesFileNamed: aString
	
	"avoid the slower default method and simply do a head request "
	self httpClient
		numberOfRetries: 0;
		ifFail: [ :exception| 
			exception response code = 404
				ifTrue: [ ^ false ].
			exception pass];
		head: (self urlForFileNamed: aString).
		
	^ true! !

!MCSqueaksourceRepository methodsFor: 'interface' stamp: 'CamilloBruni 4/23/2012 20:22'!
includesVersionNamed: aString
	"directly do a filename check since squeaksource only stores mcz"
	^ self includesFileNamed: aString, '.mcz'! !

MCRepository subclass: #MCGOODSRepository
	instanceVariableNames: 'hostname port connection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCGOODSRepository class methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2004 20:36'!
creationTemplate
	^
'MCGOODSRepository
	host: ''localhost''
	port: 6100'! !

!MCGOODSRepository class methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2004 14:33'!
description
	^ 'GOODS'! !

!MCGOODSRepository class methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2004 20:36'!
host: hostname port: portNumber
	^ self new
		host: hostname;
		port: portNumber! !

!MCGOODSRepository methodsFor: 'as yet unclassified' stamp: 'avi 8/26/2004 14:20'!
basicStoreVersion: aVersion
	self root at: aVersion info put: aVersion.
	self db commit.! !

!MCGOODSRepository methodsFor: 'as yet unclassified' stamp: 'lr 3/14/2010 21:13'!
db
	(connection isNil or: [ connection isConnected not ])
		ifTrue: [ connection := Smalltalk globals at: #KKDatabase ifPresent: [ :cl | cl onHost: hostname port: port ] ].
	^ connection! !

!MCGOODSRepository methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2004 20:35'!
description
	^ 'goods://', hostname asString, ':', port asString! !

!MCGOODSRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
host: aString
	hostname := aString! !

!MCGOODSRepository methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2004 20:17'!
packages
	^ (self root collect: [:ea | ea package]) asSet asSortedCollection! !

!MCGOODSRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
port: aNumber
	port := aNumber! !

!MCGOODSRepository methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2004 14:35'!
root
	self db root ifNil: [self db root: Dictionary new].
	^ self db root! !

!MCGOODSRepository methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2004 20:21'!
versionWithInfo: aVersionInfo ifAbsent: errorBlock
	^ self root at: aVersionInfo ifAbsent: errorBlock! !

!MCGOODSRepository methodsFor: 'as yet unclassified' stamp: 'avi 2/28/2004 20:18'!
versionsAvailableForPackage: aPackage
	^ self root asArray select: [:ea | ea package = aPackage] thenCollect: [:ea | ea info]! !

!MCRepository class methodsFor: 'as yet unclassified' stamp: 'ab 8/21/2003 00:30'!
allConcreteSubclasses
	^ self withAllSubclasses reject: [:ea | ea isAbstract]! !

!MCRepository class methodsFor: 'as yet unclassified' stamp: 'bkv 2/18/2004 20:59'!
creationTemplate
	self subclassResponsibility.! !

!MCRepository class methodsFor: 'as yet unclassified' stamp: 'ab 8/21/2003 00:29'!
description
	^ nil! !

!MCRepository class methodsFor: 'as yet unclassified' stamp: 'bkv 2/18/2004 20:58'!
fillInTheBlankRequest
	self subclassResponsibility.! !

!MCRepository class methodsFor: 'as yet unclassified' stamp: 'ab 8/21/2003 12:59'!
isAbstract
	^ self description isNil! !

!MCRepository class methodsFor: 'as yet unclassified' stamp: 'ab 7/24/2003 21:01'!
morphicConfigure
	^ self new! !

!MCRepository methodsFor: 'as yet unclassified' stamp: 'ab 8/21/2003 12:36'!
= other
	^ other species = self species and: [other description = self description]! !

!MCRepository methodsFor: 'as yet unclassified' stamp: 'avi 8/31/2004 01:08'!
alwaysStoreDiffs
	^ storeDiffs ifNil: [false]! !

!MCRepository methodsFor: 'as yet unclassified' stamp: 'bkv 2/18/2004 20:48'!
asCreationTemplate
	^ self creationTemplate! !

!MCRepository methodsFor: 'as yet unclassified' stamp: 'avi 8/26/2004 14:20'!
basicStoreVersion: aVersion
	self subclassResponsibility! !

!MCRepository methodsFor: 'as yet unclassified' stamp: 'marcus.denker 11/10/2008 10:04'!
closestAncestorVersionFor: anAncestry ifNone: errorBlock
	anAncestry breadthFirstAncestorsDo:
		[:ancestorInfo |
		(self versionWithInfo: ancestorInfo) ifNotNil: [:v | ^ v]].
	^ errorBlock value! !

!MCRepository methodsFor: 'as yet unclassified' stamp: 'bkv 2/18/2004 20:46'!
creationTemplate
	^ creationTemplate! !

!MCRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
creationTemplate: aString
	self creationTemplate ifNotNil: [ self error: 'Creation template already set for this MCRepository instance.' ].
	
	creationTemplate := aString.! !

!MCRepository methodsFor: 'as yet unclassified' stamp: 'avi 10/9/2003 12:53'!
description
	^ self class name! !

!MCRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
doAlwaysStoreDiffs
	storeDiffs := true! !

!MCRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
doNotAlwaysStoreDiffs
	storeDiffs := false! !

!MCRepository methodsFor: 'as yet unclassified' stamp: 'ab 8/21/2003 12:36'!
hash
	^ self description hash! !

!MCRepository methodsFor: 'interface' stamp: 'ab 8/21/2003 12:40'!
includesVersionNamed: aString
	self subclassResponsibility! !

!MCRepository methodsFor: 'testing' stamp: 'nk 11/2/2003 10:55'!
isValid
	^true! !

!MCRepository methodsFor: 'as yet unclassified' stamp: 'avi 8/26/2004 14:27'!
notificationForVersion: aVersion
	^ MCVersionNotification version: aVersion repository: self! !

!MCRepository methodsFor: 'as yet unclassified' stamp: 'avi 8/26/2004 14:23'!
notifyList
	^ #()! !

!MCRepository methodsFor: 'as yet unclassified' stamp: 'bf 3/10/2005 23:01'!
possiblyNewerVersionsOfAnyOf: someVersions
	^#()! !

!MCRepository methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 4/21/2012 21:12'!
prepareVersionForStorage: aVersion
	^ self alwaysStoreDiffs
		ifTrue: [
			aVersion asDiffAgainst:
				(self closestAncestorVersionFor: aVersion info ifNone: [^ aVersion])]
		ifFalse: [ aVersion ]! !

!MCRepository methodsFor: 'as yet unclassified' stamp: 'mas 9/24/2003 04:21'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		nextPutAll: self description;
		nextPut: $).! !

!MCRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
sendNotificationsForVersion: aVersion
	| notification notifyList |
	notifyList := self notifyList.
	notifyList isEmpty ifFalse:
		[notification := self notificationForVersion: aVersion.
		notifyList do: [:ea | notification notify: ea]]! !

!MCRepository methodsFor: 'as yet unclassified' stamp: 'SeanDeNigris 7/17/2012 15:49'!
storeDependencies: aVersion
	MCCacheRepository uniqueInstance cacheAllFileNamesDuring: 
		[self cacheAllFileNamesDuring: 
			[aVersion allAvailableDependenciesDo:
				[:dep |
					(self includesVersionNamed: dep info name)
						ifFalse: [self storeVersion: dep]]]]! !

!MCRepository methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 4/22/2012 21:11'!
storeVersion: aVersion
	self basicStoreVersion: (self prepareVersionForStorage: aVersion).
	self sendNotificationsForVersion: aVersion! !

!MCRepository methodsFor: 'interface' stamp: 'avi 10/9/2003 12:42'!
versionWithInfo: aVersionInfo
	^ self versionWithInfo: aVersionInfo ifAbsent: [nil]! !

!MCRepository methodsFor: 'interface' stamp: 'ab 8/16/2003 18:22'!
versionWithInfo: aVersionInfo ifAbsent: aBlock
	self subclassResponsibility ! !

MCRepository subclass: #MCWriteOnlyRepository
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

MCWriteOnlyRepository subclass: #MCSmtpRepository
	instanceVariableNames: 'email'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCSmtpRepository class methodsFor: 'as yet unclassified' stamp: 'avi 10/9/2003 12:56'!
description
	^ 'SMTP'! !

!MCSmtpRepository class methodsFor: 'as yet unclassified' stamp: 'alain.plantec 2/6/2009 17:52'!
morphicConfigure
	| address |
	address := UIManager default request: 'Email address:' translated.
	^ address isEmpty ifFalse: [self new emailAddress: address]! !

!MCSmtpRepository methodsFor: 'as yet unclassified' stamp: 'avi 8/26/2004 14:21'!
basicStoreVersion: aVersion
	MailSender sendMessage: (self messageForVersion: aVersion)! !

!MCSmtpRepository methodsFor: 'as yet unclassified' stamp: 'avi 10/9/2003 13:11'!
bodyForVersion: aVersion
	^ String streamContents:
		[ :s |
		s nextPutAll: 'from version info:'; cr; cr.
		s nextPutAll:  aVersion info summary]! !

!MCSmtpRepository methodsFor: 'as yet unclassified' stamp: 'avi 10/9/2003 12:56'!
description
	^ 'mailto://', email! !

!MCSmtpRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
emailAddress: aString
	email := aString	! !

!MCSmtpRepository methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
messageForVersion: aVersion
	| message data |
	message := MailMessage empty.
	message setField: 'from' toString: MailSender userName.
	message setField: 'to' toString: email.
	message setField: 'subject' toString: (self subjectForVersion: aVersion). 

	message body:
		(MIMEDocument
			contentType: 'text/plain'
			content: (self bodyForVersion: aVersion)).

	"Prepare the gzipped data"
	data := RWBinaryOrTextStream on: String new.
	aVersion fileOutOn: data.
	message addAttachmentFrom: data reset withName: aVersion fileName.
	^ message! !

!MCSmtpRepository methodsFor: 'as yet unclassified' stamp: 'avi 10/9/2003 13:14'!
subjectForVersion: aVersion
	^ '[Package] ', aVersion info name! !

!MCWriteOnlyRepository methodsFor: 'as yet unclassified' stamp: 'avi 10/9/2003 12:52'!
includesVersionNamed: aString
	^ false! !

!MCWriteOnlyRepository methodsFor: 'as yet unclassified' stamp: 'avi 10/9/2003 12:52'!
versionWithInfo: aVersionInfo ifAbsent: aBlock
	^ aBlock value! !

Object subclass: #MCRepositoryGroup
	instanceVariableNames: 'repositories useCache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

MCRepositoryGroup class
	instanceVariableNames: 'default'!

!MCRepositoryGroup commentStamp: '<historical>' prior: 0!
A singleton class, holds the list of repositories. Can look for a requested VersionInfo among its repositories.!

MCRepositoryGroup class
	instanceVariableNames: 'default'!

!MCRepositoryGroup class methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
default
	^ default ifNil: [default := self new]! !

!MCRepositoryGroup methodsFor: 'as yet unclassified' stamp: 'SeanDeNigris 7/17/2012 15:42'!
addRepository: aRepository
	((repositories includes: aRepository) or: [aRepository == MCCacheRepository uniqueInstance])
		ifFalse: [repositories add: aRepository.
				self class default addRepository: aRepository].
	self changed: (Array with: #added with: aRepository).
	^ aRepository! !

!MCRepositoryGroup methodsFor: 'as yet unclassified' stamp: 'NorbertHartl 6/20/2008 10:11'!
disableCache
	useCache := false! !

!MCRepositoryGroup methodsFor: 'as yet unclassified' stamp: 'avi 11/7/2003 00:20'!
includes: aRepository
	^ self repositories includes: aRepository! !

!MCRepositoryGroup methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 4/22/2012 21:07'!
includesVersionNamed: aString
	" check for existing version name in parallel over all repositories "
	| resultSemaphore results |
	resultSemaphore := Semaphore new.
	results := Array new: self repositories size.
	
	self repositories doWithIndex: [:repository :index | 
		" fork of test for each repository "
		[[ results 
			at: index 
			put: (repository includesVersionNamed: aString)
		] ensure: [ resultSemaphore signal ]] fork ].

	" wait for all requests to finish "
	self repositories size timesRepeat: [ resultSemaphore wait ].
	" check if any repository included the given versionName already"
	^ results anySatisfy: [:result| result = true ]	! !

!MCRepositoryGroup methodsFor: 'as yet unclassified' stamp: 'alain.plantec 5/28/2009 10:03'!
initialize
	super initialize.
	repositories := OrderedCollection new! !

!MCRepositoryGroup methodsFor: 'as yet unclassified' stamp: 'sd 3/15/2008 17:33'!
removeHTTPRepositoryLocationNamed: aRepositoryString
	
	| httpRepo others |
	httpRepo := repositories select: [:each | each isKindOf: MCHttpRepository]. 
	others := repositories reject: [:each | each isKindOf: MCHttpRepository].
	repositories := others, (httpRepo reject: [:each | each locationWithTrailingSlash = aRepositoryString]).
	self changed: #repositories! !

!MCRepositoryGroup methodsFor: 'as yet unclassified' stamp: 'lr 5/9/2010 19:04'!
removeRepository: aRepository
	repositories remove: aRepository ifAbsent: [].
	self changed: (Array with: #removed with: aRepository)! !

!MCRepositoryGroup methodsFor: 'as yet unclassified' stamp: 'SeanDeNigris 7/17/2012 15:42'!
repositories
	^ (self useCache 
		ifTrue: [Array with: MCCacheRepository uniqueInstance] 
		ifFalse: [Array new]) , repositories select: #isValid! !

!MCRepositoryGroup methodsFor: 'as yet unclassified' stamp: 'avi 11/7/2003 00:51'!
repositoriesDo: aBlock
	self repositories do: [:ea | [aBlock value: ea] on: Error do: []]! !

!MCRepositoryGroup methodsFor: 'as yet unclassified' stamp: 'abc 6/20/2008 10:02'!
useCache
	^ useCache ifNil: [
		useCache := true
	]! !

!MCRepositoryGroup methodsFor: 'as yet unclassified' stamp: 'dvf 8/10/2004 23:02'!
versionWithInfo: aVersionInfo
	^self versionWithInfo: aVersionInfo ifNone: [ self error: 'Could not find version ', aVersionInfo name printString,'. Maybe you need to add a repository?' ]! !

!MCRepositoryGroup methodsFor: 'as yet unclassified' stamp: 'marcus.denker 11/10/2008 10:04'!
versionWithInfo: aVersionInfo ifNone: aBlock
	self repositoriesDo: [:ea | (ea versionWithInfo: aVersionInfo) ifNotNil: [:v | ^ v]].
	^aBlock value! !

Object subclass: #MCScanner
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Chunk Format'!

!MCScanner commentStamp: 'LaurentLaffont 3/31/2011 21:08' prior: 0!
I scan / tokenize metadata  (package name, version info, ancestry, dependencies, ...) found in .mcz files.

For example, try:

MCScanner scan:  '(name ''MyPackage-ll.6'' message ''Fix bug xxx'' id ''b21dbd73-f1c3-2746-a3cc-92f1d4edea28'')' readStream!

!MCScanner class methodsFor: 'public api' stamp: 'avi 1/22/2004 20:14'!
scan: aStream
	^ (self new stream: aStream) next! !

!MCScanner class methodsFor: 'public api' stamp: 'avi 1/22/2004 20:32'!
scanTokens: aString
	"compatibility"
	^ Array with: (self scan: aString readStream)! !

!MCScanner methodsFor: 'actions' stamp: 'stephaneducasse 2/4/2006 20:47'!
next
	| c |
	stream skipSeparators.
	c := stream peek.
	c = $# ifTrue: [c := stream next; peek].
	c = $' ifTrue: [^ self nextString].
	c = $( ifTrue: [^ self nextArray].
	c isAlphaNumeric ifTrue: [^ self nextSymbol].
	self error: 'Unknown token type'.	! !

!MCScanner methodsFor: 'actions' stamp: 'CamilloBruni 7/6/2012 15:43'!
nextArray
	stream next. "("
	^ Array streamContents: [:s |
		[	stream skipSeparators.
			(stream peek = $)) or: [stream atEnd]
		] whileFalse: [ s nextPut: self next ].
		stream next = $) 
			ifFalse: [ self error: 'Unclosed array' ]]! !

!MCScanner methodsFor: 'actions' stamp: 'avi 1/22/2004 20:09'!
nextString
	^ stream nextDelimited: $'! !

!MCScanner methodsFor: 'actions' stamp: 'avi 1/22/2004 20:16'!
nextSymbol
	^ (String streamContents:
		[:s |
		[stream peek isAlphaNumeric] whileTrue: [s nextPut: stream next]]) asSymbol
			! !

!MCScanner methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
stream: aStream
	stream := aStream! !

Object subclass: #MCServerCredentials
	instanceVariableNames: 'username password'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCServerCredentials class methodsFor: 'instance creation' stamp: 'SeanDeNigris 8/26/2012 18:24'!
user: nameString hasPassword: passwordString

	^ self new
			username: nameString;
			password: passwordString;
			yourself.! !

!MCServerCredentials methodsFor: 'accessing' stamp: 'SeanDeNigris 8/26/2012 18:33'!
password

	^ password.! !

!MCServerCredentials methodsFor: 'private' stamp: 'SeanDeNigris 8/26/2012 18:25'!
password: aString 

	password := aString.! !

!MCServerCredentials methodsFor: 'accessing' stamp: 'SeanDeNigris 8/26/2012 18:33'!
username

	^ username.! !

!MCServerCredentials methodsFor: 'private' stamp: 'SeanDeNigris 8/26/2012 18:25'!
username: aString 

	username := aString.! !

Object subclass: #MCServerRegistry
	instanceVariableNames: 'registry'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

MCServerRegistry class
	instanceVariableNames: 'uniqueInstance'!

MCServerRegistry class
	instanceVariableNames: 'uniqueInstance'!

!MCServerRegistry class methodsFor: 'instance creation' stamp: 'SeanDeNigris 8/26/2012 11:11'!
uniqueInstance

	^ uniqueInstance ifNil: [ uniqueInstance := self new ].! !

!MCServerRegistry methodsFor: 'private' stamp: 'SeanDeNigris 8/26/2012 18:18'!
initialize

	super initialize.
	registry := Dictionary new.! !

!MCServerRegistry methodsFor: 'as yet unclassified' stamp: 'SeanDeNigris 8/26/2012 11:24'!
on: repositoryUrl beUser: nameString withPassword: passwordString

	| credentials |
	credentials := MCServerCredentials user: nameString hasPassword: passwordString.
	registry at: repositoryUrl put: credentials.! !

!MCServerRegistry methodsFor: 'private' stamp: 'CamilloBruni 2/19/2013 18:30'!
repositoryAt: urlString credentialsDo: aBlock


	| possibleMatches bestMatch |
	possibleMatches := registry associations select: [ :e | urlString beginsWith: e key ].
	possibleMatches isEmpty ifTrue: [ ^ aBlock value: '' value: '' ].
	bestMatch := possibleMatches inject: possibleMatches anyOne into: [ :last :new | 
		((new key asUrl path size > last key asUrl path size) 
		"this is ugly, but URL always returns a path element, even an empty one..."
		or: [ last key asUrl path size = 1 and: [ last key asUrl path last isEmpty ]]) 
			ifTrue: [ new ]
			ifFalse: [ last ]].
	^ aBlock value: bestMatch value username value: bestMatch value password.! !

Object subclass: #MCSnapshot
	instanceVariableNames: 'definitions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Base'!

!MCSnapshot commentStamp: '<historical>' prior: 0!
MCSnapshot represents a list of entities compared to a given base entities. 

It holds a list of definitions (instances of MCDefinition subclasses). It can install these entities or update a package.!

!MCSnapshot class methodsFor: 'as yet unclassified' stamp: 'ab 7/6/2003 23:48'!
empty
	^ self fromDefinitions: #()! !

!MCSnapshot class methodsFor: 'as yet unclassified' stamp: 'ab 7/6/2003 23:48'!
fromDefinitions: aCollection
	^ self new initializeWithDefinitions: aCollection! !

!MCSnapshot methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 5/30/2012 17:12'!
= other
	self == other
		ifTrue: [ ^ true ].
	self species = other species
		ifFalse: [ ^ false ].
	^ definitions asArray = other definitions asArray! !

!MCSnapshot methodsFor: 'accessing' stamp: 'ab 12/4/2002 18:09'!
definitions
	^ definitions! !

!MCSnapshot methodsFor: 'accessing' stamp: 'ab 7/10/2003 01:05'!
hash
	^ definitions asArray hash! !

!MCSnapshot methodsFor: 'initializing' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithDefinitions: aCollection
	definitions := aCollection.! !

!MCSnapshot methodsFor: 'loading' stamp: 'ab 7/6/2003 23:31'!
install
	MCPackageLoader installSnapshot: self! !

!MCSnapshot methodsFor: 'patching' stamp: 'ab 7/7/2003 00:37'!
patchRelativeToBase: aSnapshot
	^ MCPatch fromBase: aSnapshot target: self! !

!MCSnapshot methodsFor: 'loading' stamp: 'ab 7/7/2003 12:11'!
updatePackage: aPackage
	MCPackageLoader updatePackage: aPackage withSnapshot: self! !

Object subclass: #MCVariableDefinition
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCVariableDefinition commentStamp: '<historical>' prior: 0!
A MCVariableDefinition represents a variable.!

MCVariableDefinition subclass: #MCClassInstanceVariableDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCClassInstanceVariableDefinition commentStamp: '<historical>' prior: 0!
A MCClassInstanceVariableDefinition represents a class instance variable.!

!MCClassInstanceVariableDefinition class methodsFor: 'as yet unclassified' stamp: 'cwp 7/7/2003 22:59'!
type
	^ #classInstance! !

!MCClassInstanceVariableDefinition methodsFor: 'testing' stamp: 'cwp 7/7/2003 23:31'!
isClassInstanceVariable
	^ true! !

MCVariableDefinition subclass: #MCClassVariableDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCClassVariableDefinition commentStamp: '<historical>' prior: 0!
A MCClassVariableDefinition represent a class variable.!

!MCClassVariableDefinition class methodsFor: 'as yet unclassified' stamp: 'cwp 7/7/2003 22:58'!
type
	^ #class! !

!MCClassVariableDefinition methodsFor: 'testing' stamp: 'cwp 7/7/2003 23:32'!
isClassVariable
	^ true! !

!MCClassVariableDefinition methodsFor: 'testing' stamp: 'bf 8/29/2006 11:41'!
isOrderDependend
	^false! !

MCVariableDefinition subclass: #MCInstanceVariableDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCInstanceVariableDefinition commentStamp: '<historical>' prior: 0!
A MCInstanceVariableDefinition represents an instance variable definition.!

!MCInstanceVariableDefinition class methodsFor: 'as yet unclassified' stamp: 'cwp 7/7/2003 22:59'!
type
	^ #instance! !

!MCInstanceVariableDefinition methodsFor: 'testing' stamp: 'cwp 7/7/2003 23:32'!
isInstanceVariable
	^ true! !

MCVariableDefinition subclass: #MCPoolImportDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Modeling'!

!MCPoolImportDefinition commentStamp: '<historical>' prior: 0!
A MCPoolImportDefinition represents a pool definition.!

!MCPoolImportDefinition class methodsFor: 'as yet unclassified' stamp: 'cwp 7/7/2003 22:59'!
type
	^ #pool! !

!MCPoolImportDefinition methodsFor: 'testing' stamp: 'bf 8/29/2006 11:41'!
isOrderDependend
	^false! !

!MCPoolImportDefinition methodsFor: 'testing' stamp: 'cwp 7/7/2003 23:51'!
isPoolImport
	^ true! !

!MCVariableDefinition class methodsFor: 'as yet unclassified' stamp: 'cwp 7/7/2003 23:18'!
name: aString
	^ self new name: aString
	! !

!MCVariableDefinition methodsFor: 'comparing' stamp: 'cwp 7/7/2003 23:02'!
= other
	^ (self species = other species)
		and: [self name = other name]! !

!MCVariableDefinition methodsFor: 'comparing' stamp: 'cwp 7/7/2003 23:02'!
hash
	^ name hash! !

!MCVariableDefinition methodsFor: 'testing' stamp: 'cwp 7/7/2003 23:31'!
isClassInstanceVariable
	^ false! !

!MCVariableDefinition methodsFor: 'testing' stamp: 'cwp 7/7/2003 23:08'!
isClassInstanceVariableDefinition
	^ false! !

!MCVariableDefinition methodsFor: 'testing' stamp: 'cwp 7/7/2003 23:30'!
isClassVariable
	^ false! !

!MCVariableDefinition methodsFor: 'testing' stamp: 'cwp 7/7/2003 23:31'!
isInstanceVariable
	^ false! !

!MCVariableDefinition methodsFor: 'testing' stamp: 'cwp 7/7/2003 23:10'!
isInstanceVariableDefinition
	^ false! !

!MCVariableDefinition methodsFor: 'testing' stamp: 'bf 8/29/2006 11:41'!
isOrderDependend
	^true! !

!MCVariableDefinition methodsFor: 'testing' stamp: 'cwp 7/7/2003 23:51'!
isPoolImport
	^ false! !

!MCVariableDefinition methodsFor: 'accessing' stamp: 'cwp 11/25/2002 05:57'!
name
	^ name! !

!MCVariableDefinition methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
name: aString
	name := aString! !

!MCVariableDefinition methodsFor: 'as yet unclassified' stamp: 'nk 7/24/2003 14:56'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(; nextPutAll: self name; nextPut: $)! !

Object subclass: #MCVersion
	instanceVariableNames: 'package info snapshot dependencies completeSnapshot completePackageSnapshot'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

MCVersion subclass: #MCDiffyVersion
	instanceVariableNames: 'base patch'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCDiffyVersion class methodsFor: 'name utilities' stamp: 'bf 5/30/2005 18:45'!
baseNameFrom: diffName
	| baseId verName |
	baseId := (diffName copyAfter: $() copyUpTo: $).
	baseId ifEmpty: [^baseId].
	(baseId beginsWith: '@')
		ifTrue: [^baseId copyAfter: $@].
	verName := self verNameFrom: diffName.
	^(baseId includes: $.)
		ifTrue: [(verName copyUpToLast: $-), '-', baseId]
		ifFalse: [(verName copyUpToLast: $.), '.', baseId]
! !

!MCDiffyVersion class methodsFor: 'name utilities' stamp: 'bf 5/30/2005 18:58'!
canonicalNameFor: aFileName
	^(self nameForVer: (self verNameFrom: aFileName)
		base: (self baseNameFrom: aFileName))
			, '.', MCMcdReader extension
! !

!MCDiffyVersion class methodsFor: 'name utilities' stamp: 'bf 5/30/2005 17:39'!
nameForVer: versionName base: baseName
	| baseId |
	baseId := (versionName copyUpToLast: $.) = (baseName copyUpToLast: $.)
		ifTrue: [baseName copyAfterLast: $.]
		ifFalse: [(versionName copyUpToLast: $-) = (baseName copyUpToLast: $-)
			ifTrue: [baseName copyAfterLast: $-]
			ifFalse: ['@', baseName]].
	^ versionName, '(', baseId, ')'! !

!MCDiffyVersion class methodsFor: 'instance creation' stamp: 'avi 2/13/2004 23:07'!
package: aPackage info: aVersionInfo dependencies: aCollection baseInfo: baseVersionInfo patch:
aPatch
	^ self basicNew initializeWithPackage: aPackage info: aVersionInfo dependencies: aCollection baseInfo: baseVersionInfo patch:
aPatch! !

!MCDiffyVersion class methodsFor: 'instance creation' stamp: 'avi 2/13/2004 23:06'!
package: aPackage info: aVersionInfo snapshot: aSnapshot dependencies: aCollection baseVersion: aVersion
	^ self 
		package: aPackage
		info: aVersionInfo
		dependencies: aCollection
		baseInfo: aVersion info
		patch: (aSnapshot patchRelativeToBase: aVersion snapshot)! !

!MCDiffyVersion class methodsFor: 'name utilities' stamp: 'bf 5/30/2005 18:19'!
verNameFrom: diffName
	^diffName copyUpTo: $(! !

!MCDiffyVersion methodsFor: 'as yet unclassified' stamp: 'avi 2/13/2004 23:17'!
baseInfo
	^ base! !

!MCDiffyVersion methodsFor: 'as yet unclassified' stamp: 'CamilloBrui 7/8/2011 12:28'!
baseSnapshot
       | baseVersion |
       baseVersion := self workingCopy repositoryGroup versionWithInfo: base.
       baseVersion ifNil: [ self error: 'Missing snapshot: ', self baseInfo name].
       ^ baseVersion snapshot! !

!MCDiffyVersion methodsFor: 'as yet unclassified' stamp: 'bf 5/23/2005 15:42'!
canOptimizeLoading
	"Answer wether I can provide a patch for the working copy without the usual diff pass"
	^ package hasWorkingCopy
		and: [package workingCopy modified not
			and: [package workingCopy ancestors includes: self baseInfo]]! !

!MCDiffyVersion methodsFor: 'as yet unclassified' stamp: 'bf 5/30/2005 17:39'!
fileName
	^ (self class nameForVer: info name base: base name), '.', self writerClass extension! !

!MCDiffyVersion methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithPackage: aPackage info: aVersionInfo dependencies: aCollection baseInfo: baseVersionInfo patch: aPatch
	patch := aPatch.
	base := baseVersionInfo.
	super initializeWithPackage: aPackage info: aVersionInfo snapshot: nil dependencies: aCollection.
! !

!MCDiffyVersion methodsFor: 'as yet unclassified' stamp: 'avi 2/13/2004 23:24'!
isDiffy
	^ true! !

!MCDiffyVersion methodsFor: 'as yet unclassified' stamp: 'avi 2/13/2004 23:17'!
patch
	^ patch! !

!MCDiffyVersion methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
snapshot
	^ snapshot ifNil: [snapshot := MCPatcher apply: patch to: self baseSnapshot]! !

!MCDiffyVersion methodsFor: 'as yet unclassified' stamp: 'avi 2/19/2004 22:03'!
summary
	^ '(Diff against ', self baseInfo name, ')', String cr, super summary! !

!MCDiffyVersion methodsFor: 'as yet unclassified' stamp: 'avi 2/13/2004 23:17'!
writerClass
	^ MCMcdWriter ! !

!MCVersion class methodsFor: 'instance creation' stamp: 'ab 7/7/2003 16:13'!
package: aPackage
	^ self package: aPackage info: MCVersionInfo new! !

!MCVersion class methodsFor: 'instance creation' stamp: 'ab 7/7/2003 16:13'!
package: aPackage info: aVersionInfo
	^ self package: aPackage info: aVersionInfo snapshot: aPackage snapshot! !

!MCVersion class methodsFor: 'instance creation' stamp: 'cwp 11/7/2004 13:02'!
package: aPackage info: aVersionInfo snapshot: aSnapshot
	^ self package: aPackage info: aVersionInfo snapshot: aSnapshot dependencies: #()! !

!MCVersion class methodsFor: 'instance creation' stamp: 'avi 1/19/2004 13:11'!
package: aPackage info: aVersionInfo snapshot: aSnapshot dependencies: aCollection
	^ self new initializeWithPackage: aPackage info: aVersionInfo snapshot: aSnapshot dependencies: aCollection! !

!MCVersion methodsFor: 'actions' stamp: 'SeanDeNigris 7/17/2012 15:40'!
addToCache
	MCCacheRepository uniqueInstance storeVersion: self! !

!MCVersion methodsFor: 'actions' stamp: 'avi 2/12/2004 19:37'!
adopt
	self workingCopy adopt: self! !

!MCVersion methodsFor: 'enumerating' stamp: 'nice 1/5/2010 15:59'!
allAvailableDependenciesDo: aBlock
	
	self dependencies do:
		[:ea |
		[ | version |version := ea resolve.
		version allAvailableDependenciesDo: aBlock.
		aBlock value: version]
			on: Error do: []]! !

!MCVersion methodsFor: 'enumerating' stamp: 'cwp 11/7/2004 11:58'!
allDependenciesDo: aBlock
	self allDependenciesDo: aBlock ifUnresolved: [:ignored | true]! !

!MCVersion methodsFor: 'enumerating' stamp: 'stephaneducasse 2/4/2006 20:47'!
allDependenciesDo: aBlock ifUnresolved: failBlock
	| dict |
	dict := Dictionary new.
	self allDependenciesNotIn: dict do: aBlock ifUnresolved: failBlock! !

!MCVersion methodsFor: 'enumerating' stamp: 'nice 1/5/2010 15:59'!
allDependenciesNotIn: aDictionary do: aBlock ifUnresolved: failBlock
	
	self dependencies do: 
		[:ea | | version | 
		version := aDictionary at: ea ifAbsent: [ea resolve].
		version 
			ifNil: [failBlock value: ea]
			ifNotNil: [(aDictionary includes: version) ifFalse:
						[aDictionary at: ea put: version.
						version 
							allDependenciesNotIn: aDictionary 
							do: aBlock
							ifUnresolved: failBlock.
						aBlock value: version]]]! !

!MCVersion methodsFor: 'converting' stamp: 'avi 2/19/2004 21:00'!
asDiffAgainst: aVersion
	aVersion info = self info ifTrue: [self error: 'Cannot diff against self!!'].
	^ MCDiffyVersion
		package: self package
		info: self info
		snapshot: self snapshot
		dependencies: self dependencies
		baseVersion: aVersion! !

!MCVersion methodsFor: 'testing' stamp: 'bf 5/23/2005 15:43'!
canOptimizeLoading
	"Answer wether I can provide a patch for the working copy without the usual diff pass"
	^false! !

!MCVersion methodsFor: 'accessing' stamp: 'CamilloBruni 9/30/2011 16:18'!
changes
	^ self completeSnapshot patchRelativeToBase: self completePackageSnapshot! !

!MCVersion methodsFor: 'accessing' stamp: 'CamilloBruni 9/30/2011 16:18'!
completePackageSnapshot
	^ completePackageSnapshot 
		ifNil: [ completePackageSnapshot := self loadCompletePackageSnapshot]! !

!MCVersion methodsFor: 'accessing' stamp: 'CamilloBruni 9/30/2011 16:08'!
completeSnapshot
	^ completeSnapshot ifNil: [ completeSnapshot := self loadCompleteSnapshot ]! !

!MCVersion methodsFor: 'accessing' stamp: 'avi 1/22/2004 00:24'!
dependencies
	^ dependencies ifNil: [#()]! !

!MCVersion methodsFor: 'accessing' stamp: 'avi 1/22/2004 12:44'!
fileName
	^ info name, '.', self writerClass extension! !

!MCVersion methodsFor: 'actions' stamp: 'avi 1/22/2004 12:44'!
fileOutOn: aStream
	self writerClass fileOut: self on: aStream! !

!MCVersion methodsFor: 'accessing' stamp: 'ab 7/7/2003 14:28'!
info
	^ info! !

!MCVersion methodsFor: 'initialization' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithPackage: aPackage info: aVersionInfo snapshot: aSnapshot dependencies: aCollection
	package := aPackage.
	info := aVersionInfo.
	snapshot := aSnapshot.
	dependencies := aCollection.
	self addToCache.! !

!MCVersion methodsFor: 'testing' stamp: 'bf 3/22/2005 23:00'!
isCacheable
	^true! !

!MCVersion methodsFor: 'testing' stamp: 'avi 2/13/2004 23:24'!
isDiffy
	^ false! !

!MCVersion methodsFor: 'actions' stamp: 'avi 1/24/2004 20:13'!
load
	MCVersionLoader loadVersion: self! !

!MCVersion methodsFor: 'accessing' stamp: 'CamilloBruni 9/30/2011 16:20'!
loadCompletePackageSnapshot
	| definitions |
	definitions := package snapshot definitions asOrderedCollection.
	
	self dependencies 
		do: [ :each| definitions addAll: each package snapshot definitions ]
		displayingProgress: [ :item| 'Loading dependencies from: ', item package name ].
	
	^ MCSnapshot fromDefinitions: definitions
	
	! !

!MCVersion methodsFor: 'accessing' stamp: 'CamilloBruni 9/30/2011 16:12'!
loadCompleteSnapshot
	|definitions|
	
	definitions := self snapshot definitions asOrderedCollection.
	
	self dependencies 
		do: [ :each| definitions addAll: each resolve completeSnapshot definitions ]
		displayingProgress: [ :item| 'Loading dependencies from: ', item package name ].
	
	^ MCSnapshot fromDefinitions: definitions! !

!MCVersion methodsFor: 'actions' stamp: 'abc 2/13/2004 15:58'!
merge
	MCVersionMerger mergeVersion: self! !

!MCVersion methodsFor: 'accessing' stamp: 'ab 7/7/2003 14:19'!
package
	^ package! !

!MCVersion methodsFor: 'printing' stamp: 'nk 3/8/2004 23:54'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(.
	aStream nextPutAll: self info name.
	aStream nextPut: $).! !

!MCVersion methodsFor: 'initialization' stamp: 'stephaneducasse 2/4/2006 20:47'!
setPackage: aPackage info: aVersionInfo snapshot: aSnapshot dependencies: aCollection
	package := aPackage.
	info := aVersionInfo.
	snapshot := aSnapshot.
	dependencies := aCollection! !

!MCVersion methodsFor: 'accessing' stamp: 'CamilloBruni 4/20/2012 18:21'!
snapshot
	"lazily load snapshot..."
	snapshot isBlock ifTrue: [ 
		snapshot := snapshot value ].
	^ snapshot! !

!MCVersion methodsFor: 'accessing' stamp: 'adrian_lienhard 1/7/2009 17:32'!
summary
	^ String streamContents:
		[:s |
		s nextPutAll: info summaryHeader.
		(dependencies isNil or: [dependencies isEmpty]) ifFalse:
			[s cr; nextPutAll: 'Dependencies: '.
			dependencies
				do: [:ea | s nextPutAll: ea versionInfo name]
				separatedBy: [s nextPutAll: ', ']].
		s cr; cr; nextPutAll: info message]! !

!MCVersion methodsFor: 'enumerating' stamp: 'cwp 11/7/2004 14:24'!
withAllDependenciesDo: aBlock
	self allDependenciesDo: aBlock ifUnresolved: [:ignored].
	aBlock value: self! !

!MCVersion methodsFor: 'enumerating' stamp: 'stephaneducasse 2/4/2006 20:47'!
withAllDependenciesDo: aBlock ifUnresolved: failBlock
	| dict |
	dict := Dictionary new.
	self allDependenciesNotIn: dict do: aBlock ifUnresolved: failBlock.
	aBlock value: self! !

!MCVersion methodsFor: 'accessing' stamp: 'avi 2/12/2004 19:38'!
workingCopy
	^ package workingCopy! !

!MCVersion methodsFor: 'accessing' stamp: 'avi 1/22/2004 12:44'!
writerClass
	^ MCMczWriter ! !

Object subclass: #MCVersionDependency
	instanceVariableNames: 'package versionInfo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCVersionDependency class methodsFor: 'as yet unclassified' stamp: 'avi 1/19/2004 13:13'!
package: aPackage info: aVersionInfo
	^ self basicNew initializeWithPackage: aPackage info: aVersionInfo! !

!MCVersionDependency methodsFor: 'comparing' stamp: 'avi 1/19/2004 16:12'!
= other
	^ other species = self species
		and: [other versionInfo = versionInfo
				and: [other package = package]]! !

!MCVersionDependency methodsFor: 'comparing' stamp: 'avi 1/19/2004 16:06'!
hash
	^ versionInfo hash! !

!MCVersionDependency methodsFor: 'initialization' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithPackage: aPackage info: aVersionInfo
	package := aPackage.
	versionInfo := aVersionInfo! !

!MCVersionDependency methodsFor: 'testing' stamp: 'bf 4/19/2005 16:29'!
isCurrent
	^ package hasWorkingCopy
		and: [self isFulfilled
			and: [package workingCopy modified not]]! !

!MCVersionDependency methodsFor: 'testing' stamp: 'bf 4/19/2005 16:29'!
isFulfilled
	^package hasWorkingCopy
		and: [self isFulfilledBy: package workingCopy ancestry]! !

!MCVersionDependency methodsFor: 'testing' stamp: 'avi 3/4/2004 00:34'!
isFulfilledBy: anAncestry
	^ anAncestry ancestors includes: versionInfo! !

!MCVersionDependency methodsFor: 'testing' stamp: 'bf 4/19/2005 16:29'!
isFulfilledByAncestors
	^ package hasWorkingCopy
		and: [self isFulfilledByAncestorsOf: package workingCopy ancestry]! !

!MCVersionDependency methodsFor: 'testing' stamp: 'nk 7/13/2004 08:45'!
isFulfilledByAncestorsOf: anAncestry
	^ anAncestry hasAncestor: versionInfo! !

!MCVersionDependency methodsFor: 'testing' stamp: 'bf 4/19/2005 16:29'!
isOlder
	"Answer true if I represent an older version of a package that is loaded."
	^ package hasWorkingCopy
		and: [self isFulfilled not
			and: [ self isFulfilledByAncestors
				and: [package workingCopy modified not]]]! !

!MCVersionDependency methodsFor: 'accessing' stamp: 'avi 1/19/2004 15:40'!
package
	^ package! !

!MCVersionDependency methodsFor: 'accessing' stamp: 'avi 2/12/2004 19:38'!
repositoryGroup
	^ self package workingCopy repositoryGroup! !

!MCVersionDependency methodsFor: 'resolving' stamp: 'nk 6/13/2004 19:21'!
resolve
	^ self repositoryGroup
		versionWithInfo: versionInfo
		ifNone: [ MCRepositoryGroup default versionWithInfo: versionInfo ifNone: []]! !

!MCVersionDependency methodsFor: 'accessing' stamp: 'avi 1/19/2004 15:40'!
versionInfo
	^ versionInfo! !

Object subclass: #MCVersionLoader
	instanceVariableNames: 'versions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Loading'!

!MCVersionLoader commentStamp: '<historical>' prior: 0!
A MCVersionLoader is responsible for loading a given version. !

!MCVersionLoader class methodsFor: 'public api' stamp: 'avi 1/24/2004 20:06'!
loadVersion: aVersion
	self new
		addVersion: aVersion;
		load! !

!MCVersionLoader methodsFor: 'loading' stamp: 'cwp 11/7/2004 17:06'!
addDependency: aDependency
	| dep |
	aDependency isCurrent ifTrue: [^ self].
	(self depAgeIsOk: aDependency) ifFalse: [^ self].
	dep := aDependency resolve.
	dep
		ifNil: [self confirmMissingDependency: aDependency]
		ifNotNil: [(versions includes: dep) ifFalse: [self addVersion: dep]]! !

!MCVersionLoader methodsFor: 'loading' stamp: 'cwp 11/7/2004 17:04'!
addVersion: aVersion
	aVersion dependencies do: [ :ea | self addDependency: ea].
	versions add: aVersion.
! !

!MCVersionLoader methodsFor: 'checking' stamp: 'md 3/5/2012 14:28'!
checkForModificationsIfCancel: cancelBlock ifMerge: mergeBlock
	| modifications |
	
	"first check if there are local packages which are dirty"
	modifications := self localModifications.
	modifications isEmpty ifTrue: [ ^ self].
	
	"ask the user what to do.. merge | overwrite | abort"
	^ self warnAboutLosingChangesTo: modifications ifCancel: cancelBlock ifMerge: mergeBlock! !

!MCVersionLoader methodsFor: 'checking' stamp: 'cwp 11/7/2004 17:00'!
checkIfDepIsOlder: aDependency
	^ aDependency isOlder not 
		or: [self confirm: 'load older dependency ', aDependency versionInfo name , '?']! !

!MCVersionLoader methodsFor: 'checking' stamp: 'cwp 11/7/2004 17:06'!
confirmMissingDependency: aDependency
	| name |
	name := aDependency versionInfo name.
	(self confirm: 'Can''t find dependency ', name, '. ignore?')
		ifFalse: [self error: 'Can''t find dependency ', name]! !

!MCVersionLoader methodsFor: 'checking' stamp: 'cwp 11/7/2004 17:02'!
depAgeIsOk: aDependency
	^ aDependency isOlder not 
		or: [self confirm: 'load older dependency ', aDependency versionInfo name , '?']! !

!MCVersionLoader methodsFor: 'testing' stamp: 'CamilloBruni 9/28/2011 17:15'!
hasLocalModifications
	^ self localModifications isEmpty not
	! !

!MCVersionLoader methodsFor: 'testing' stamp: 'sd 3/16/2008 08:53'!
hasVersions

	^ versions isEmpty not! !

!MCVersionLoader methodsFor: 'initialization' stamp: 'alain.plantec 5/28/2009 10:04'!
initialize
	super initialize.
	versions := OrderedCollection new! !

!MCVersionLoader methodsFor: 'loading' stamp: 'bf 3/16/2006 19:03'!
load
	self loadWithNameLike: versions first info name.
! !

!MCVersionLoader methodsFor: 'loading' stamp: 'CamilloBruni 1/16/2012 17:06'!
loadWithNameLike: aString
	| loader |
	
	self checkForModificationsIfCancel: [ ^ self] ifMerge: [ ^ self mergeVersions ].
	
	loader := versions size > 1
		ifTrue: [MCMultiPackageLoader new]
		ifFalse: [MCPackageLoader new].
		
	versions do: [:ea |
		ea canOptimizeLoading
			ifTrue: [ea patch applyTo: loader]
			ifFalse: [loader updatePackage: ea package withSnapshot: ea snapshot]].
		
	loader loadWithNameLike: aString.
	versions do: [:ea | ea workingCopy loaded: ea]! !

!MCVersionLoader methodsFor: 'checking' stamp: 'CamilloBruni 8/4/2011 12:46'!
localModifications
	^ versions select: [:ea | ea package workingCopy modified].
	! !

!MCVersionLoader methodsFor: 'loading' stamp: 'abc 8/8/2011 13:01'!
mergeVersions
	|merger|
	merger :=  MCVersionMerger new.
	merger addVersions: self localModifications.
	merger merge.! !

!MCVersionLoader methodsFor: 'accessing' stamp: 'dkh 9/9/2009 14:51'!
versions

	^versions! !

!MCVersionLoader methodsFor: 'checking' stamp: 'abc 8/8/2011 12:48'!
warnAboutLosingChangesTo: versionCollection
	^ (MCMergeOrLoadWarning signalFor: versionCollection).! !

!MCVersionLoader methodsFor: 'checking' stamp: 'CamilloBruni 1/16/2012 17:42'!
warnAboutLosingChangesTo: versionCollection ifCancel: cancelBlock ifMerge: mergeBlock
	
	| result |

	result := (MCMergeOrLoadWarning signalFor: versionCollection).
	
	result = false ifTrue: [ ^ mergeBlock value ].
	result = nil ifTrue: [ ^ cancelBlock value ].
	
	"otherwise we will continue loading"! !

Object subclass: #MCVersionMerger
	instanceVariableNames: 'records merger'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCVersionMerger class methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 9/30/2011 16:53'!
mergeVersion: aVersion
	self new
		addVersion: aVersion;
		mergeWithNameLike: aVersion info name! !

!MCVersionMerger methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 9/30/2011 16:53'!
addVersion: aVersion
	
	records add: (MCMergeRecord version: aVersion).
	aVersion dependencies 
		do: [:ea | | dep |
			dep := ea resolve.
			(records anySatisfy: [:r | r version = dep]) 
				ifFalse: [self addVersion: dep]]
		displayingProgress: [ :ea| 'Searching dependency: ', ea package name]! !

!MCVersionMerger methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 8/4/2011 13:49'!
addVersions: aCollection
	aCollection do: [ :version|
		self addVersion: version].! !

!MCVersionMerger methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 9/30/2011 16:52'!
gatherChanges
	records do: [:ea | merger addBaseSnapshot: ea packageSnapshot].
	records do: [:ea | merger applyPatch: ea mergePatch].! !

!MCVersionMerger methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 8/4/2011 14:03'!
initialize
	super initialize.
	records := OrderedCollection new.
	merger := MCThreeWayMerger new.! !

!MCVersionMerger methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 1/16/2012 15:49'!
merge

	self gatherChanges.

	self resolveConflicts ifTrue:
		[merger load.
		records do: [:ea | ea updateWorkingCopy].
		^ true].
	
	^ false! !

!MCVersionMerger methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 9/30/2011 16:51'!
mergeWithNameLike: baseName

	self gatherChanges.
	
	self resolveConflicts ifTrue:
		[merger loadWithNameLike: baseName.
		records do: [:ea | ea updateWorkingCopy]].! !

!MCVersionMerger methodsFor: 'as yet unclassified' stamp: 'CamilloBruni 8/4/2011 14:03'!
resolveConflicts
	(records allSatisfy: [:ea | ea isAncestorMerge]) 
		ifTrue: [
			MCNoChangesException signal. 
			^ false].
		
	^ ((MCMergeResolutionRequest new merger: merger)
			signal: 'Merging ', records first version info name) = true! !

Object subclass: #MCVersionNotification
	instanceVariableNames: 'version ancestor repository changes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Repositories'!

!MCVersionNotification class methodsFor: 'as yet unclassified' stamp: 'avi 8/26/2004 14:27'!
version: aVersion repository: aRepository
	^ self basicNew initializeWithVersion: aVersion repository: aRepository! !

!MCVersionNotification methodsFor: 'as yet unclassified' stamp: 'avi 8/26/2004 15:13'!
fromAddress
	^ 'monticello@beta4.com'! !

!MCVersionNotification methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
initializeWithVersion: aVersion repository: aRepository
	version := aVersion.
	repository := aRepository.
	ancestor := repository closestAncestorVersionFor: version info ifNone: []. 
	changes := ancestor
				ifNil: [#()]
				ifNotNil: [(version snapshot patchRelativeToBase: ancestor snapshot) 							operations asSortedCollection]! !

!MCVersionNotification methodsFor: 'as yet unclassified' stamp: 'avi 8/26/2004 15:12'!
messageText
	^ String streamContents:
		[:s |
		s nextPutAll: 'Committed to repository: ', repository description; cr; cr.
		s nextPutAll: version summary.
		changes isEmpty ifFalse:
			[s cr; cr.
			s nextPutAll: '-----------------------------------------------------'; cr.
			s nextPutAll: 'Changes since ', ancestor info name, ':'; cr.
			changes do:
			[:ea |
			s cr; nextPutAll: ea summary; cr.
			s nextPutAll: ea sourceString]]]! !

!MCVersionNotification methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
messageTo: aString
	| message |
	message := MailMessage empty.
	message setField: 'from' toString: self fromAddress.
	message setField: 'to' toString: aString.
	message setField: 'subject' toString: '[MC] ', version info name.
	message body: (MIMEDocument contentType: 'text/plain' content: self messageText).
	^ message! !

!MCVersionNotification methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
notify: aString
	| message |
	message := self messageTo: aString.
	SMTPClient
		deliverMailFrom: message from
		to: (Array with: message to)
		text: message text
		usingServer: MailSender smtpServer! !

Object subclass: #MCVersionSorter
	instanceVariableNames: 'layers depthIndex depths stepparents roots'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

MCVersionSorter subclass: #MCFilteredVersionSorter
	instanceVariableNames: 'target'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

!MCFilteredVersionSorter methodsFor: 'as yet unclassified' stamp: 'bf 5/28/2005 01:14'!
addVersionInfo: aVersionInfo
	(aVersionInfo hasAncestor: target)
		ifTrue: [super addVersionInfo: aVersionInfo]
! !

!MCFilteredVersionSorter methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 12/25/2009 23:08'!
processVersionInfo: aVersionInfo
	| success |
	aVersionInfo = target ifTrue: [^ true].
	(aVersionInfo hasAncestor: target) ifFalse: [^false].
	self pushLayer.
	success := (self knownAncestorsOf: aVersionInfo) anySatisfy:
				[:ea | self processVersionInfo: ea].
	self popLayer.
	success ifTrue: [self addToCurrentLayer: aVersionInfo].
	^ success	! !

!MCFilteredVersionSorter methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
target: aVersionInfo
	target := aVersionInfo! !

!MCVersionSorter methodsFor: 'as yet unclassified' stamp: 'avi 9/11/2004 14:19'!
addAllAncestorsOf: aVersionInfo to: aSet
	(aSet includes: aVersionInfo) ifTrue: [^ self].
	aSet add: aVersionInfo.
	(self knownAncestorsOf: aVersionInfo) do:
		[:ea |
		self addAllAncestorsOf: ea to: aSet]! !

!MCVersionSorter methodsFor: 'as yet unclassified' stamp: 'avi 8/31/2003 21:30'!
addAllVersionInfos: aCollection
	aCollection do: [:ea | self addVersionInfo: ea]! !

!MCVersionSorter methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
addToCurrentLayer: aVersionInfo
	| layer |
	layer := layers at: depthIndex.
	(layer includes: aVersionInfo) ifFalse:
		[depths at: aVersionInfo ifPresent:
			[:i |
			i < depthIndex
				ifTrue: [(layers at: i) remove: aVersionInfo]
				ifFalse: [^ false]].
		layer add: aVersionInfo.
		depths at: aVersionInfo put: depthIndex.
		^ true].
	^ false ! !

!MCVersionSorter methodsFor: 'as yet unclassified' stamp: 'avi 9/11/2004 10:49'!
addVersionInfo: aVersionInfo
	roots add: aVersionInfo.
	self registerStepChildrenOf: aVersionInfo seen: Set new! !

!MCVersionSorter methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
allAncestorsOf: aVersionInfo
	| all |
	all := Set new.
	self addAllAncestorsOf: aVersionInfo to: all.
	^ all! !

!MCVersionSorter methodsFor: 'as yet unclassified' stamp: 'alain.plantec 5/28/2009 10:04'!
initialize
	super initialize.
	stepparents := Dictionary new.
	roots := OrderedCollection new.! !

!MCVersionSorter methodsFor: 'as yet unclassified' stamp: 'avi 9/11/2004 14:37'!
knownAncestorsOf: aVersionInfo
	^ aVersionInfo ancestors, (self stepParentsOf: aVersionInfo) asArray! !

!MCVersionSorter methodsFor: 'as yet unclassified' stamp: 'ab 8/17/2003 15:53'!
layers
	^ layers! !

!MCVersionSorter methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
popLayer
	depthIndex := depthIndex - 1! !

!MCVersionSorter methodsFor: 'as yet unclassified' stamp: 'avi 9/11/2004 10:39'!
processVersionInfo: aVersionInfo
	(self addToCurrentLayer: aVersionInfo) ifTrue:
		[self pushLayer.
		(self knownAncestorsOf: aVersionInfo) do: [:ea | self processVersionInfo: ea].
		self popLayer]
! !

!MCVersionSorter methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
pushLayer
	depthIndex := depthIndex + 1.
	depthIndex > layers size ifTrue: [layers add: OrderedCollection new].
	! !

!MCVersionSorter methodsFor: 'as yet unclassified' stamp: 'avi 9/11/2004 14:34'!
registerStepChildrenOf: aVersionInfo seen: aSet
	(aSet includes: aVersionInfo) ifTrue: [^ self].
	aSet add: aVersionInfo.
	aVersionInfo stepChildren do: [:ea | (self stepParentsOf: ea) add: aVersionInfo].
	aVersionInfo ancestors do: [:ea | self registerStepChildrenOf: ea seen: aSet].! !

!MCVersionSorter methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
sortedVersionInfos
	layers := OrderedCollection with: OrderedCollection new.
	depthIndex := 1.
	depths := Dictionary new.
	roots do: [:ea | self processVersionInfo: ea].
	^ layers gather: [:ea | ea]! !

!MCVersionSorter methodsFor: 'as yet unclassified' stamp: 'avi 9/11/2004 10:40'!
stepParentsOf: aVersionInfo
	^ (stepparents at: aVersionInfo ifAbsentPut: [Set new])! !

Object subclass: #MCWriter
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Storing'!

MCWriter subclass: #MCMczWriter
	instanceVariableNames: 'zip infoWriter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Storing'!

MCMczWriter subclass: #MCMcdWriter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Storing'!

!MCMcdWriter class methodsFor: 'as yet unclassified' stamp: 'avi 2/13/2004 23:09'!
readerClass
	^ MCMcdReader! !

!MCMcdWriter methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
writeBaseInfo: aVersionInfo
	| string |
	string := self serializeVersionInfo: aVersionInfo.
	self addString: string at: 'base'.
! !

!MCMcdWriter methodsFor: 'as yet unclassified' stamp: 'avi 2/17/2004 01:48'!
writeDefinitions: aVersion
	self writeBaseInfo: aVersion baseInfo.
	self writePatch: aVersion patch.! !

!MCMcdWriter methodsFor: 'as yet unclassified' stamp: 'avi 2/17/2004 02:07'!
writeNewDefinitions: aCollection
	self addString: (self serializeDefinitions: aCollection) at: 'new/source.', self snapshotWriterClass extension.! !

!MCMcdWriter methodsFor: 'as yet unclassified' stamp: 'avi 2/17/2004 02:07'!
writeOldDefinitions: aCollection
	self addString: (self serializeDefinitions: aCollection) at: 'old/source.', self snapshotWriterClass extension.! !

!MCMcdWriter methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
writePatch: aPatch
	| old new |
	old := OrderedCollection new.
	new := OrderedCollection new.
	aPatch operations do:
		[:ea |
		ea isRemoval ifTrue: [old add: ea definition].
		ea isAddition ifTrue: [new add: ea definition].
		ea isModification ifTrue: [old add: ea baseDefinition. new add: ea definition]].
	self writeOldDefinitions: old.
	self writeNewDefinitions: new.
	self addString: (self serializeInBinary: aPatch) at: 'patch.bin'.! !

!MCMczWriter class methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
fileOut: aVersion on: aStream
	| inst |
	inst := self on: aStream.
	inst writeVersion: aVersion.
	inst flush.
	
! !

!MCMczWriter class methodsFor: 'as yet unclassified' stamp: 'cwp 8/1/2003 12:35'!
readerClass
	^ MCMczReader! !

!MCMczWriter methodsFor: 'writing' stamp: 'stephaneducasse 2/4/2006 20:47'!
addString: string at: path
	| member |
	member := zip addString: string as: path.
	member desiredCompressionMethod: ZipArchive compressionDeflated 
	! !

!MCMczWriter methodsFor: 'writing' stamp: 'avi 2/17/2004 02:17'!
flush
	zip writeTo: stream.
	stream close! !

!MCMczWriter methodsFor: 'accessing' stamp: 'avi 2/17/2004 01:54'!
format
	^ '1'! !

!MCMczWriter methodsFor: 'initializing' stamp: 'alain.plantec 5/28/2009 10:03'!
initialize
	super initialize.
	zip := ZipArchive new.
! !

!MCMczWriter methodsFor: 'serializing' stamp: 'stephaneducasse 2/4/2006 20:47'!
serializeDefinitions: aCollection
	| writer s |
	s := RWBinaryOrTextStream on: String new.
	writer := self snapshotWriterClass on: s.
	writer writeDefinitions: aCollection.
	^ s contents! !

!MCMczWriter methodsFor: 'serializing' stamp: 'MarianoMartinezPeck 6/17/2012 12:37'!
serializeInBinary: aSnapshot
	| writer s |
	s := RWBinaryOrTextStream on: String new.
	writer := MCDataStream on: s.
	writer nextPut: aSnapshot.
	^ s contents! !

!MCMczWriter methodsFor: 'serializing' stamp: 'cwp 8/13/2003 01:06'!
serializePackage: aPackage
	^ '(name ''', aPackage name, ''')'! !

!MCMczWriter methodsFor: 'serializing' stamp: 'CamilloBruni 7/6/2012 15:52'!
serializeVersionInfo: aVersionInfo
	infoWriter ifNil: [infoWriter := MCVersionInfoWriter new].
	^ String streamContents: [:s |
		infoWriter stream: s.
		infoWriter writeVersionInfo: aVersionInfo]! !

!MCMczWriter methodsFor: 'accessing' stamp: 'avi 2/17/2004 02:07'!
snapshotWriterClass
	^ MCStWriter! !

!MCMczWriter methodsFor: 'visiting' stamp: 'avi 2/17/2004 01:48'!
writeDefinitions: aVersion
	self writeSnapshot: aVersion snapshot! !

!MCMczWriter methodsFor: 'visiting' stamp: 'avi 2/17/2004 01:56'!
writeFormat
"	self addString: self format at: 'format'."! !

!MCMczWriter methodsFor: 'visiting' stamp: 'avi 2/17/2004 01:48'!
writePackage: aPackage
	self addString: (self serializePackage: aPackage) at: 'package'! !

!MCMczWriter methodsFor: 'visiting' stamp: 'avi 9/28/2004 14:26'!
writeSnapshot: aSnapshot
	self addString: (self serializeDefinitions: aSnapshot definitions) at: 'snapshot/source.', self snapshotWriterClass extension.
	self addString: (self serializeInBinary: aSnapshot) at: 'snapshot.bin'! !

!MCMczWriter methodsFor: 'visiting' stamp: 'avi 9/13/2004 16:49'!
writeVersion: aVersion
	self writeFormat.
	self writePackage: aVersion package.
	self writeVersionInfo: aVersion info.
	self writeDefinitions: aVersion.
	aVersion dependencies do: [:ea | self writeVersionDependency: ea]! !

!MCMczWriter methodsFor: 'visiting' stamp: 'stephaneducasse 2/4/2006 20:47'!
writeVersionDependency: aVersionDependency
	| string |
	string := (self serializeVersionInfo: aVersionDependency versionInfo).
	self addString: string at: 'dependencies/', aVersionDependency package name! !

!MCMczWriter methodsFor: 'visiting' stamp: 'stephaneducasse 2/4/2006 20:47'!
writeVersionInfo: aVersionInfo
	| string |
	string := self serializeVersionInfo: aVersionInfo.
	self addString: string at: 'version'.
! !

!MCMczWriter methodsFor: 'accessing' stamp: 'cwp 8/1/2003 00:06'!
zip
	^ zip! !

MCWriter subclass: #MCStWriter
	instanceVariableNames: 'initializers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Storing'!

!MCStWriter commentStamp: 'LaurentLaffont 3/31/2011 21:07' prior: 0!
I write Category/Classes/Methods/.... definitions into Monticello source.st file format found in .mcz packages.    

See also MCStReader

Example:

String streamContents: [:aStream| |writer|
    writer := MCStWriter on: aStream.
    writer writeDefinitions: {True asClassDefinition. False asClassDefinition}.
]!

!MCStWriter class methodsFor: 'as yet unclassified' stamp: 'avi 1/20/2004 00:16'!
readerClass
	^ MCStReader! !

!MCStWriter methodsFor: 'writing' stamp: 'cwp 8/2/2003 02:34'!
chunkContents: aBlock
	stream cr; nextChunkPut: (String streamContents: aBlock); cr! !

!MCStWriter methodsFor: 'visiting' stamp: 'cwp 8/2/2003 11:02'!
visitClassDefinition: definition
	self writeClassDefinition: definition.
	definition hasClassInstanceVariables ifTrue: [self writeMetaclassDefinition: definition].
	definition hasComment ifTrue: [self writeClassComment: definition].! !

!MCStWriter methodsFor: 'visiting' stamp: 'al 10/9/2005 19:52'!
visitClassTraitDefinition: definition
	self chunkContents: [:s | s
		nextPutAll: definition baseTrait;
		nextPutAll: ' classTrait';
		cr; tab;
		nextPutAll: 'uses: ';
		nextPutAll: (definition classTraitComposition ifNil: ['{}'])]
! !

!MCStWriter methodsFor: 'visiting' stamp: 'al 10/9/2005 19:40'!
visitMetaclassDefinition: definition
	self writeMetaclassDefinition: definition! !

!MCStWriter methodsFor: 'visiting' stamp: 'avi 2/17/2004 02:23'!
visitMethodDefinition: definition
	self writeMethodPreamble: definition.
	self writeMethodSource: definition.
	self writeMethodPostscript.
	self writeMethodInitializer: definition.! !

!MCStWriter methodsFor: 'visiting' stamp: 'cwp 8/2/2003 11:02'!
visitOrganizationDefinition: defintion
	defintion categories do: [:cat | self writeCategory: cat].
! !

!MCStWriter methodsFor: 'visiting' stamp: 'bf 8/12/2009 21:41'!
visitScriptDefinition: definition
	self writeScriptDefinition: definition
! !

!MCStWriter methodsFor: 'visiting' stamp: 'al 10/9/2005 19:40'!
visitTraitDefinition: definition
	self writeClassDefinition: definition.
	definition hasComment ifTrue: [self writeClassComment: definition].! !

!MCStWriter methodsFor: 'writing' stamp: 'cwp 8/2/2003 01:46'!
writeCategory: categoryName
	stream
		nextChunkPut: 'SystemOrganization addCategory: ', categoryName printString;
		cr! !

!MCStWriter methodsFor: 'writing' stamp: 'ab 8/17/2003 17:09'!
writeClassComment: definition
	stream
		cr;
		nextPut: $!!;
		nextPutAll: definition className;
		nextPutAll: ' commentStamp: ';
		store: definition commentStamp;
		nextPutAll: ' prior: 0!!';
		cr;
		nextChunkPut: definition comment;
		cr.! !

!MCStWriter methodsFor: 'writing' stamp: 'cwp 8/2/2003 02:16'!
writeClassDefinition: definition
	self chunkContents: [:s | definition printDefinitionOn: stream]! !

!MCStWriter methodsFor: 'writing' stamp: 'pavel.krivanek 10/23/2010 19:40'!
writeDefinitions: aCollection
	
	"the correct initialization order is unknown if some classes are missing in the image"
	
	| presentInitializers notPresentInitializers orderedClasses |
	
	initializers := Set new.

	(MCDependencySorter sortItems: aCollection)
		do: [:ea | ea accept: self]
		displayingProgress: 'Writing definitions...'.
		
	presentInitializers := initializers select: [:each | Smalltalk hasClassNamed: each key ].
	notPresentInitializers := initializers reject: [:each | Smalltalk hasClassNamed: each key ].
	
	orderedClasses := (Class superclassOrder: (presentInitializers collect: [:each | Smalltalk classOrTraitNamed: each key]))
		collect: [:each | each name ].
				
	orderedClasses do: [:className |
		stream nextPutAll: (presentInitializers detect: [:each | each key = className]) value contents].
	
	(notPresentInitializers asSortedCollection: [:a :b | a key <= b key]) do: [:association |
		stream nextPutAll: association value contents ].! !

!MCStWriter methodsFor: 'writing' stamp: 'al 12/2/2005 15:17'!
writeMetaclassDefinition: definition
	self chunkContents: [:str |
		str	nextPutAll: definition className;
			nextPutAll: ' class';
			cr; tab.
			definition hasClassTraitComposition ifTrue: [
				str	nextPutAll: 'uses: ';
					nextPutAll: definition classTraitCompositionString;
					cr; tab].
			str	nextPutAll: 'instanceVariableNames: ''';
				nextPutAll: definition classInstanceVariablesString;
				nextPut: $']! !

!MCStWriter methodsFor: 'writing' stamp: 'pavel.krivanek 10/23/2010 18:51'!
writeMethodInitializer: aMethodDefinition

	| initializationStream |
	
	aMethodDefinition isInitializer 
		ifTrue: [
			initializationStream := String new writeStream.			
			initializationStream nextChunkPut: aMethodDefinition className, ' initialize'; cr.
			initializers add: (aMethodDefinition className->initializationStream)].! !

!MCStWriter methodsFor: 'writing' stamp: 'cwp 8/2/2003 12:43'!
writeMethodPostscript
	stream
		space;
		nextPut: $!!;
		cr! !

!MCStWriter methodsFor: 'writing' stamp: 'avi 9/23/2003 17:42'!
writeMethodPreamble: definition
	stream
		cr;
		nextPut: $!!;
		nextPutAll: definition fullClassName;
		nextPutAll: ' methodsFor: ';
		nextPutAll: definition category asString printString;
		nextPutAll: ' stamp: ';
		nextPutAll: definition timeStamp asString printString;
		nextPutAll: '!!';
		cr! !

!MCStWriter methodsFor: 'writing' stamp: 'cwp 8/4/2003 01:35'!
writeMethodSource: definition
	stream nextChunkPut: definition source! !

!MCStWriter methodsFor: 'writing' stamp: 'bf 8/13/2009 00:21'!
writeScriptDefinition: definition
	stream nextChunkPut: (
		'(PackageInfo named: {1}) {2}: {3}'
		format: {
			"{1}" definition packageName printString.
			"{2}" definition scriptSelector. 
			"{3}" definition script printString
		}); cr! !

!MCStWriter methodsFor: 'writing' stamp: 'avi 2/17/2004 02:25'!
writeSnapshot: aSnapshot
	self writeDefinitions: aSnapshot definitions! !

MCWriter subclass: #MCVersionInfoWriter
	instanceVariableNames: 'written'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Storing'!

!MCVersionInfoWriter commentStamp: '<historical>' prior: 0!
I serialize an MCVersionInfo into a given stream.
If the version info has been serialized before I output a compacted version consiting of the id.
!

!MCVersionInfoWriter methodsFor: 'testing' stamp: 'avi 1/22/2004 21:10'!
isWritten: aVersionInfo
	^ self written includes: aVersionInfo! !

!MCVersionInfoWriter methodsFor: 'serialization' stamp: 'IgorStasenko 11/28/2012 16:02'!
writeVersionInfo: aVersionInfo
	(self isWritten: aVersionInfo)
		ifTrue: [^ stream nextPutAll: '(id '; print: aVersionInfo id asString; nextPut: $) ].
	stream nextPut: $(.
	#(name message id date time author) do: [:sel | 
		stream 
			nextPutAll: sel; space;
			print: (((aVersionInfo perform: sel) ifNil: ['']) asString convertToEncoding: 'latin-1' ); space ].
	stream nextPutAll: 'ancestors ('.
	aVersionInfo ancestors do: [:ea | self writeVersionInfo: ea].
	stream nextPutAll: ') stepChildren ('.
	aVersionInfo stepChildren do: [:ea | self writeVersionInfo: ea].
	stream nextPutAll: '))'.
	
	self wrote: aVersionInfo! !

!MCVersionInfoWriter methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
written
	^ written ifNil: [written := Set new]! !

!MCVersionInfoWriter methodsFor: 'serialization' stamp: 'avi 1/22/2004 21:10'!
wrote: aVersionInfo
	self written add: aVersionInfo! !

!MCWriter class methodsFor: 'accessing' stamp: 'cwp 8/1/2003 15:00'!
extension
	^ self readerClass extension! !

!MCWriter class methodsFor: 'writing' stamp: 'cwp 8/1/2003 01:16'!
on: aStream
	^ self new stream: aStream! !

!MCWriter class methodsFor: 'accessing' stamp: 'cwp 7/28/2003 23:46'!
readerClass
	^ self subclassResponsibility ! !

!MCWriter methodsFor: 'as yet unclassified' stamp: 'cwp 8/1/2003 01:14'!
stream
	^ stream! !

!MCWriter methodsFor: 'as yet unclassified' stamp: 'stephaneducasse 2/4/2006 20:47'!
stream: aStream
	stream := aStream! !

Object subclass: #MethodAddition
	instanceVariableNames: 'text category changeStamp requestor logSource myClass methodAndNode selector compiledMethod priorMethodOrNil priorCategoryOrNil'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Loading'!

!MethodAddition commentStamp: 'rej 2/25/2007 19:30' prior: 0!
I represent the addition of a method to a class.  I can produce the CompiledMethod, install it, and then notify the system that the method has been added.  This allows Monticello to implement atomic addition.  A loader can compile all classes and methods first and then install all methods only after they have been all compiled, and in a way that executes little code.!

!MethodAddition methodsFor: 'compilation' stamp: 'rej 2/26/2007 10:51'!
compile
    "This method is the how compiling a method used to work.  All these steps were done at once.
     This method should not normally be used, because the whole point of MethodAddition is to let
	you first create a compiled method and then install the method later."
	self createCompiledMethod.
	self installMethod.
	self notifyObservers.
	^selector! !

!MethodAddition methodsFor: 'compilation' stamp: 'rej 2/25/2007 20:36'!
compile: aString classified: aString1 withStamp: aString2 notifying: aRequestor logSource: aBoolean inClass: aClass

	text := aString.
	category := aString1.
	changeStamp := aString2.
	requestor := aRequestor.
	logSource := aBoolean.
	myClass := aClass! !

!MethodAddition methodsFor: 'operations' stamp: 'PavelKrivanek 12/6/2012 22:58'!
createCompiledMethod
	methodAndNode := myClass compile: text asString classified: category notifying: requestor
							trailer: myClass defaultMethodTrailer ifFail: [^nil].
	selector := methodAndNode selector.
	compiledMethod := methodAndNode method.
	self writeSourceToLog.
	priorMethodOrNil := myClass compiledMethodAt: selector ifAbsent: [nil].
	priorCategoryOrNil := myClass  organization categoryOfElement: selector.! !

!MethodAddition methodsFor: 'operations' stamp: 'rej 2/25/2007 22:09'!
installMethod
	myClass addSelectorSilently: selector withMethod: compiledMethod.
! !

!MethodAddition methodsFor: 'notifying' stamp: 'PavelKrivanek 12/6/2012 23:00'!
notifyObservers
	SystemAnnouncer uniqueInstance 
		suspendAllWhile: [myClass organization classify: selector under: category].
	priorMethodOrNil isNil
		ifTrue: [SystemAnnouncer uniqueInstance methodAdded: compiledMethod selector: selector inProtocol: category class: myClass requestor: requestor]
		ifFalse: [
			SystemAnnouncer uniqueInstance methodChangedFrom: priorMethodOrNil to: compiledMethod selector: selector inClass: myClass requestor: requestor.
			priorCategoryOrNil = category ifFalse: [
       			SystemAnnouncer uniqueInstance selector: selector recategorizedFrom: priorCategoryOrNil to: category inClass: myClass] ].
	"The following code doesn't seem to do anything."
	myClass instanceSide noteCompilationOf: selector meta: myClass isClassSide.
! !

!MethodAddition methodsFor: 'accessing' stamp: 'PavelKrivanek 12/6/2012 22:54'!
priorCategoryOrNil
	^ priorCategoryOrNil! !

!MethodAddition methodsFor: 'accessing' stamp: 'PavelKrivanek 12/6/2012 22:54'!
priorCategoryOrNil: anObject
	priorCategoryOrNil := anObject! !

!MethodAddition methodsFor: 'operations' stamp: 'rej 2/25/2007 20:42'!
writeSourceToLog
	logSource ifTrue: [
		myClass logMethodSource: text forMethodWithNode: methodAndNode 
			inCategory: category withStamp: changeStamp notifying: requestor.
	].
! !

!Object methodsFor: '*Monticello-Storing' stamp: 'CamilloBruni 8/1/2012 16:02'!
comeFullyUpOnReload: smartRefStream
	"Normally this read-in object is exactly what we want to store."

	^ self! !

!Object methodsFor: '*Monticello-Storing' stamp: 'tk 4/8/1999 12:05'!
readDataFrom: aDataStream size: varsOnDisk
	"Fill in the fields of self based on the contents of aDataStream.  Return self.
	 Read in the instance-variables written by Object>>storeDataOn:.
	 NOTE: This method must send beginReference: before reading any objects from aDataStream that might reference it.
	 Allow aDataStream to have fewer inst vars.  See SmartRefStream."
	| cntInstVars cntIndexedVars |

	cntInstVars := self class instSize.
	self class isVariable
		ifTrue: [cntIndexedVars := varsOnDisk - cntInstVars.
				cntIndexedVars < 0 ifTrue: [
					self error: 'Class has changed too much.  Define a convertxxx method']]
		ifFalse: [cntIndexedVars := 0.
				cntInstVars := varsOnDisk]. 	"OK if fewer than now"

	aDataStream beginReference: self.
	1 to: cntInstVars do:
		[:i | self instVarAt: i put: aDataStream next].
	1 to: cntIndexedVars do:
		[:i | self basicAt: i put: aDataStream next].
	"Total number read MUST be equal to varsOnDisk!!"
	^ self	"If we ever return something other than self, fix calls 
			on (super readDataFrom: aDataStream size: anInteger)"! !

!Object methodsFor: '*Monticello-Storing' stamp: 'MarianoMartinezPeck 5/21/2012 22:52'!
storeDataOn: aDataStream
	"Store myself on a DataStream.  Answer self.  This is a low-level DataStream/ReferenceStream method. See also objectToStoreOnDataStream.  NOTE: This method must send 'aDataStream beginInstance:size:' and then (nextPut:/nextPutWeak:) its subobjects.  readDataFrom:size: reads back what we write here."
	| cntInstVars cntIndexedVars |

	cntInstVars := self class instSize.
	cntIndexedVars := self basicSize.
	aDataStream
		beginInstance: self class
		size: cntInstVars + cntIndexedVars.
	1 to: cntInstVars do:
		[:i | aDataStream nextPut: (self instVarAt: i)].

	"Write fields of a variable length object. "
	(self class isBits) ifFalse: [
		1 to: cntIndexedVars do:
			[:i | aDataStream nextPut: (self basicAt: i)]].
! !

Warning subclass: #MCMergeOrLoadWarning
	instanceVariableNames: 'versions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Loading'!

!MCMergeOrLoadWarning commentStamp: '<historical>' prior: 0!
A MCMergeOrLoadWarning is a simple warning used when amerge or a load action may override existing entities.!

!MCMergeOrLoadWarning class methodsFor: 'as yet unclassified' stamp: 'abc 8/8/2011 12:50'!
signalFor: aVersionCollection
	^ self new
		versions: aVersionCollection;
		signal! !

!MCMergeOrLoadWarning methodsFor: 'exceptionDescription' stamp: 'abc 8/8/2011 12:56'!
defaultAction
	^ ( UIManager default
		confirm: self messageText
		trueChoice: 'Load' translated
		falseChoice: 'Merge' translated
		cancelChoice: 'Cancel' translated
		default: nil ).! !

!MCMergeOrLoadWarning methodsFor: 'accessing' stamp: 'CamilloBruni 1/16/2012 17:42'!
messageText
	| packageNames |
	
	packageNames := versions collect: [ :ea|  ea package name ].
		
	^ String streamContents: [:s |
		s nextPutAll: 'You are about to load new versions of the following packages'; cr;
			nextPutAll: 'that have unsaved changes in the image:'; cr.
			
		packageNames sort do: [:name |
			s cr; space; space; nextPutAll: name].
		
		s cr cr
			nextPutAll: 'If you continue, you will lose these changes:']! !

!MCMergeOrLoadWarning methodsFor: 'accessing' stamp: 'abc 8/8/2011 12:53'!
versions: aCollection
	versions := aCollection! !
MCDirtyPackageInfo initialize!
MCEmptyPackageInfo initialize!
MCDataStream initialize!
MCVersionReader initialize!
MCMethodDefinition initialize!
MCCacheRepository initialize!
MCPackageManager initialize!
MCWorkingCopy initialize!
